/*
   first.c

   SCCSid:	@(#)first.c	1.1	5/15/92

Cyclic communication

28/10/90 , HJE

changed 08/01/92 by Tony PARK, Parsytec Aachen
changed 92/02/14 by Markus Sigg, Parsytec Aachen

*/

#include <sys/root.h>			/* for PARIX */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>			/* #include <syslib.h> */
#include <cdl/cdl.h>			/* for PARIX */

#include "pipeop.c"


int             receiver (void);

static char    *send, *rec;
static word     limit, interval, bsize;
Semaphore       run_sem, print_sem;


int
main (int argc, char *argv[])
{
    word            i, nit;
    int             status;		/* needed for StartThread */

    if (argc != 4) {
	fprintf (stderr, "Usage: first <# iterations> <packet size> <monitor interval>\n");
	exit (1);
    }
    limit = nit = atoi (argv[1]);
    bsize = atoi (argv[2]);
    interval = atoi (argv[3]);

    send = Malloc (bsize);
    rec = Malloc (bsize);
    if (send == NULL || rec == NULL) {
	fprintf (stderr, "Unable to allocate buffer for 'first'\n");
	exit (1);
    }
    memset (send, '-', bsize);
    memset (rec, 0, bsize);

    InitSemaphore (&run_sem, 1);
    InitSemaphore (&print_sem, 1);	/* currently necessary to avoid
					 * printf conflicts */

    if (full_write (7, (char *) &nit, 4))
	exit (1);
    if (full_read (6, (char *) &nit, 4))
	exit (1);
    if (nit == limit) {
	fprintf (stderr, "All components initialized\n");
	fflush (stderr);
    }
    else {
	fprintf (stderr, "Initialization failed\n");
	fflush (stderr);
	exit (1);
    }

 /*
  * if (Fork(5000, receiver, NULL) == 0) { printf("Unable to fork receiver
  * process\n"); exit(1); }
  */

    if (NULL == StartThread (receiver, 5000, &status, 0)) {
	fprintf (stderr, "Unable to \"fork\" receiver process\n");
	fflush (stderr);
	exit (1);
    }

    for (i = 1; i <= nit; i++) {
	if (i == nit)
	    send[bsize - 2] = 'C';

	if (full_write (7, send, bsize)) {
	    fprintf (stderr, "Sender failed to write packet\n");
	    fflush (stderr);
	    exit (1);
	}
	if (i % interval == 0) {
	    Wait (&print_sem);
	    printf ("send: %d\n", i);
	    fflush (stdout);
	    Signal (&print_sem);
	}
    }

    Wait (&run_sem);
    exit (0);
}

int
receiver (void)
{
    word            j;

    Wait (&run_sem);
    for (j = 0;; j++) {
	memset (rec, 0, bsize);
	if (full_read (6, (char *) rec, bsize)) {
	    fprintf (stderr, "Receiver failed reading (exp. packet: %d)\n", j);
	    fflush (stderr);
	    break;
	}
	if (memcmp (send, rec, bsize - 4) != 0) {
	    fprintf (stderr, "Got bad data back from the ring! (packet %d)\n",
		     j);
	    fflush (stderr);
	    break;
	}
	if (rec[bsize - 2] == 'C') {
	    printf ("received last packet\n");
	    fflush (stdout);
	    break;
	}
	if (j % interval == 0) {
	    Wait (&print_sem);
	    printf ("got : %d\n", j);
	    fflush (stdout);
	    Signal (&print_sem);
	}
    }
    Signal (&run_sem);
}
