/* 92/02/13 Markus Sigg, PARSYTEC Aachen 
   SCCSid:	@(#)slave.c	1.2	5/20/92
 */

#include <sys/root.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include <sys/sem.h>
#include <sys/sys_rpc.h>
#include <unistd.h>

#define MESSLEN	       100
#define MAXBUFLEN      16 * 1024

int             x, y;
Semaphore_t     print_sem;

/* send to all neighbors: */

int
_w (void)
{
    int             i;
    static char     buf[MESSLEN];

    for (i = 5; i < 13; i = i + 2) {
	sprintf (buf, "Mess. from slave (%d, %d) (desc. %d).",
		 x, y, i);
	write (i, buf, strlen (buf));
    }
}

/* receive from all neighbors: */

int
_r (void)
{
    int             i;
    static char     buf[MESSLEN];

    for (i = 4; i < 12; i = i + 2) {
	read (i, buf, MESSLEN);
	Wait (&print_sem);
	fprintf (stderr, "Slave (%d, %d) : %s (got on desc. %d).\n",
		 x, y, buf, i);
	fflush (stderr);
	Signal (&print_sem);
    }
}

/* heavy send to one neighbor: */

int
_wh (int size, int count, int desc)
{
    unsigned int    i;
    double	    t, t1;
    static char     buf[MAXBUFLEN];

    write (desc, buf, 1);		/* synchronize */
    t = (double) TimeNowLow ();
    for (i = 0; i < count; i++)
	write (desc, buf, size);
    t1 = ((double) TimeNowLow () - t) / ((double) CLK_TCK_LOW / 1000.0);
    Wait (&print_sem);
    fprintf (stderr, "(%d,%d): sent %d bytes %d times (= %d k) via desc. %d in %.2f msec = %0.2f k/s.\n",
	     x, y, size, count, size * count / 1024, desc, t1,
	     (double) size * count / 1024 * 1000 / t1);
    fflush (stderr);
    Signal (&print_sem);
}

/* heavy read from one neighbor: */

int
_rh (int size, int count, int desc)
{
    int             i;
    static char     buf[MAXBUFLEN];

    read (desc, buf, 1);		/* synchronize */
    for (i = 0; i < count; i++)
	read (desc, buf, size);
}

int
main (int argc, char **argv)
{
    int             myx, myy;
    int             i, size, werr, rerr;
    static char     buf[MESSLEN];
    Thread_t       *tp1, *tp2;
    char            pbuf[100];

    setvbuf (stderr, pbuf, _IOLBF, sizeof (pbuf));
    InitSem (&print_sem, 1);

    myx = GET_ROOT ()->ProcRoot->MyX;
    myy = GET_ROOT ()->ProcRoot->MyY;
    x = atoi (argv[1]);
    y = atoi (argv[2]);

    read (0, buf, MESSLEN);
    fprintf (stderr, "Slave (%d, %d) on proc. (%d, %d): %s (got on desc. 0).\n",
	     x, y, myx, myy, buf);
    fflush (stderr);

    if ((tp1 = StartThread (_w, 5000, &werr, 0)) == NULL) {
	LogError (EC_ERROR,
		  "slave",
		  "Couldn't start thread - out of memory.");
	AbortServer (1);
    }
    if ((tp2 = StartThread (_r, 5000, &rerr, 0)) == NULL) {
	LogError (EC_ERROR,
		  "slave",
		  "Couldn't start thread - out of memory.");
	AbortServer (1);
    }

    WaitThread (tp1, NULL);
    WaitThread (tp2, NULL);

    sprintf (buf, "Mess. from slave (%d, %d) (desc. 1).", x, y);
    write (1, buf, strlen (buf) + 1);

    fprintf (stderr, "Slave (%d, %d): done.\n", x, y);
    fflush (stderr);
    fprintf (stderr, "Slave (%d, %d): starting heavy communication.\n", x, y);
    fflush (stderr);

    for (size = 1; size < MAXBUFLEN + 1; size *= 2) {
	int             count;

	if (size < 256)
	    count = 256;
	else
	    count = 16;

	for (i = 4; i < 12; i = i + 2) {

	    if (StartThread (_wh, MAXBUFLEN + 5000, &werr, 12,
			     size, count, i + 1) == NULL) {
		LogError (EC_ERROR,
			  "slave",
			  "Couldn't start thread - out of memory.");
		AbortServer (1);
	    }
	    if (StartThread (_rh, MAXBUFLEN + 5000, &rerr, 12,
			     size, count, i) == NULL) {
		LogError (EC_ERROR,
			  "slave",
			  "Couldn't start thread - out of memory.");
		AbortServer (1);
	    }
	}

	for (i = 0; i < 8; i++) {
	    Thread_t       *tp;

	    tp = WaitThread (NULL, NULL);
	    WaitThread (tp, NULL);
	}
    }

    fprintf (stderr, "Slave (%d, %d): DONE !\n", x, y);
    fflush (stderr);

    exit (0);
}




