/* SCCSid:	@(#)SendLink.c	1.1	5/15/92
 */

#include <sys/root.h>
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>

#define		ONE_SIZE	524288
#define		BUFSIZE		(65536 / sizeof (int))
#define		ONESEC_HI	1000000
#define		ONEMSEC_HI	1000

#define		LOOPS		32

int             sbuf[BUFSIZE];
int             rbuf[BUFSIZE];
int             psizes[] = {1, 4, 16, 64, 256, 1024, 4096, 16384, 65536};
int             dT;			/* time delta from measurement */

int
CheckBuf (int p_size)
{
    int             i, epos, err = 0;

    for (i = 0; i < (p_size / sizeof (int)); i++) {
	if (sbuf[i] != rbuf[i]) {
	    if (!err)
		epos = i;
	    err++;
	}
    }
    if (err)
	LogError (EC_ERROR, "CheckBuf",
		  "%d error%s found, first at %d",
		  err, err != 1 ? "s" : "", epos);
    return err;
}

#define	PC_PosX(id)	((id) % PC_DimX)
#define PC_PosY(id)	(((id) / PC_DimX) % PC_DimY)
#define PC_PosZ(id)	(((id) / PC_DimX) / PC_DimY)

int 
calc_hops (int id1, int id2)
{
    int             x1, x2, y1, y2, z1, z2;

    x1 = PC_PosX (id1);
    y1 = PC_PosY (id1);
    z1 = PC_PosZ (id1);
    x2 = PC_PosX (id2);
    y2 = PC_PosY (id2);
    z2 = PC_PosZ (id2);

    return abs (x1 - x2) + abs (y1 - y2) + abs (z1 - z2);
}

int 
do_active (int src_id, int dest_id, int p_size)
{
    char           *fname = "do_active";
    LinkCB_t       *link;
    int             error;

    int             t0, t1, tcount, dt, tsum = 0;
    int             tmin = INT_MAX;
    int             tmax = 0;

    link = MakeLink (dest_id, 0, &error);
    if (link == NULL) {
	LogError (EC_ERROR, fname, "MakeLink failed 0x%x", error);
	return EC_ERROR;
    }
    for (tcount = 0; tcount < LOOPS; tcount++) {
	t0 = TimeNowHigh ();
	error = SendLink (link, sbuf, p_size);
	error = RecvLink (link, sbuf, p_size);
	t1 = TimeNowHigh ();
	if (error < 0) {
	    LogError (EC_ERROR, fname, "SendLink failed 0x%x", error);
	    return EC_ERROR;
	}
	dt = (t1 - t0 - dT) / 2;	/* use calculated meaasurement delay */

	if (dt < 0)			/* counter overflow, ignore result */
	    tcount--;
	else {
	    if (dt < tmin)
		tmin = dt;
	    if (dt > tmax)
		tmax = dt;
	    tsum += dt;
	}
    }
    printf ("%3d <-> %3d (%3d Hop%s %5d byte): %4d KB/sec, (%6d %6d %6d usec)\n",
	    src_id, dest_id, calc_hops (src_id, dest_id), 
	    calc_hops (src_id, dest_id) == 1 ? ", " : "s,", p_size,
	    ((((p_size * LOOPS * 1000) / 1024) * 1000) / tsum),
	    tmin, tmax, tsum / LOOPS);

    error = BreakLink (link);
    if (error < 0) {
	LogError (EC_ERROR, fname, "BreakLink failed 0x%x", error);
	return EC_ERROR;
    }
    return 0;
}

int 
do_passive (int src_id, int p_size)
{
    char           *fname = "do_passive";
    LinkCB_t       *link;
    int             error;
    int             i;

    link = GetLink (src_id, 0, &error);
    if (link == NULL) {
	LogError (EC_ERROR, fname, "GetLink failed 0x%x", error);
	return EC_ERROR;
    }
    for (i = 0; i < LOOPS; i++) {
	error = RecvLink (link, rbuf, p_size);
	error = SendLink (link, rbuf, p_size);
	if (error < 0) {
	    LogError (EC_ERROR, fname, "RecvLink failed 0x%x", error);
	    return EC_ERROR;
	}
    }

    error = CheckBuf (p_size);

    error = BreakLink (link);
    if (error < 0) {
	LogError (EC_ERROR, fname, "BreakLink failed 0x%x", error);
	return EC_ERROR;
    }
    return 0;
}

int
main (int argc, char **argv)
{
    int             my_id = GET_ROOT ()->ProcRoot->MyProcID;
    int             out_id = 0;
    int             nProcs = GET_ROOT ()->ProcRoot->nProcs;
    int             i, j;

    if (argc > 1)
	out_id = atoi (argv[1]);

    if (my_id == out_id) {
	int             t0, t1, tcount, dt, tsum = 0;
	int             tmin = INT_MAX;
	int             tmax = 0;

	printf ("===========================================\n");
	printf ("SendLink/RecvLink Analytic Performance Test\n");
	printf ("===========================================\n");
	printf ("Written: C. Fleischer 920110\n\n");
	printf ("Results from Processor %d\n", my_id);
	printf ("\n");

	for (tcount = 0; tcount < LOOPS; tcount++) {
	    t0 = TimeNowHigh ();
	    t1 = TimeNowHigh ();
	    dt = t1 - t0;
	    if (dt < 0)			/* counter overflow, ignore result */
		tcount--;
	    else {
		if (dt < tmin)
		    tmin = dt;
		if (dt > tmax)
		    tmax = dt;
		tsum += dt;
	    }
	}
	printf ("dt = min %d usec, max %d usec, avg %d usec\n",
		tmin, tmax, tsum / tcount);
	dT = tmin;			/* delay from time measurement */
    }
 /* initialize send buffer */
    for (i = 0; i < BUFSIZE; i++) {
	sbuf[i] = i;
    }

    if (my_id == out_id) {
	for (j = 0; j < sizeof (psizes) / sizeof (int); j++) {
	    for (i = 0; i < nProcs; i++)
		if (i != out_id)
		    do_active (out_id, i, psizes[j]);
	    printf ("\n");
	}
    } else {
	for (j = 0; j < sizeof (psizes) / sizeof (int); j++)
	    do_passive (out_id, psizes[j]);
    }
    if (my_id == out_id)
	printf ("\n\n\n");
}
