/**MO******************* PARIX Test Program ******************************

Name	: Send.c
Version : 1.11
Purpose : Measure performance of Send/Recv with variation
          of hops and packet sizes.
	  Range of hops: 0..(n_procs - 2)
	  packet sizes: see below PSIZE1..PSIZE6
Topology: hardware topology "pipe" 1 x n
Args    : <1 optional packetsize>
          PSIZE1 is substituted with <1 optional packetsize>
Argset 1: -
Argset 2: 1
Argset 3: 
History	: created Feb.24.1992 by Sianturi Saleh, Parsytec Aachen

SCCSid:	@(#)Send.c	1.2	5/18/92

***/

/**HE*************************   Headers   *******************************/

#include <sys/root.h>
#include <stdio.h>
#include <stdlib.h>

/**PD************   Private Defines and Data structures   ****************/

#define		N_PSIZE		6

/* packet sizes */
#define 	PSIZE1		32
#define 	PSIZE2		256
#define 	PSIZE3		1024
#define 	PSIZE4		4096
#define 	PSIZE5		16384
#define 	PSIZE6		32768

#define		C_COUNT		100
#define		BUFSIZE		65536
#define		ASIZE		(65536 / sizeof (int))
#define		ONESEC_HI	1000000
#define		ONEMSEC_HI	1000

#define		SAY_HELLO_PROC	0

/**SV*********************   Static variables   **************************/

int             my_id, n_procs;
int             psize1;
int             total_size;
int             sbuf[ASIZE];
int             rbuf[ASIZE];

/**SF**********************   Static Functions   ***************************/

void
say_hello (void)
{
    if (SAY_HELLO_PROC == my_id) {
	printf ("************************************************\n");
	printf ("* PARIX test program                           *\n");
	printf ("* Send                                         *\n");
	printf ("* Performance Test                             *\n");
	printf ("* -------------------------------------------- *\n");
	printf ("* Written: S. Saleh                            *\n");
	printf ("************************************************\n");
	printf ("Going to transfer %d KB\n",
		total_size / 1024);

	printf ("%d messages/performance test\n", C_COUNT);
    }
}

int
CheckBuf (int p_size)
{
    int             i, epos, err = 0;

    for (i = 0; i < (p_size / sizeof (int)); i++) {
	if (sbuf[i] != rbuf[i]) {
	    if (!err)
		epos = i;
	    err++;
	}
    }
    if (err)
	LogError (EC_ERROR, "CheckBuf",
		  "%d error%s found, first at %d",
		  err, err != 1 ? "s" : "", epos);
    return err;
}

int
do_active (int dest_id, int p_size)
{
    char           *fname = "do_active";
    int             error;
    int             i;
    int             t0, t1;
    LinkCB_t       *link;
    int             top_id, log_id;
    int             performance;

    link = MakeLink (dest_id, 0, &error);
    if (link == NULL) {
	LogError (EC_ERROR, fname, "MakeLink failed 0x%x", error);
	return EC_ERROR;
    }
    top_id = NewTop (4);
    log_id = AddTop (top_id, link);

    t0 = TimeNowHigh ();
    for (i = 0; i < C_COUNT; ++i) {
	error = Send (top_id, log_id, (char *) sbuf, p_size);
	if (error < 0) {
	    LogError (EC_ERROR, fname, "SendLink failed");
	    return EC_ERROR;
	}
    }
    t1 = TimeNowHigh ();

    performance = C_COUNT * p_size / ((t1 - t0) / ONEMSEC_HI) * 1000 / 1024;

    printf ("Hops: %2d, Packetsize %6d bytes, Time %5d ms. Rate: %4d KB/sec\n",
	    dest_id - 1,
	    p_size,
	    (t1 - t0) / ONEMSEC_HI,
	    performance);

    error = FreeTop (top_id);
    if (error < 0) {
	LogError (EC_ERROR, fname, "FreeTop failed");
	return EC_ERROR;
    }
    return (t1 - t0);
}

int
do_passive (int dest_id, int p_size)
{
    char           *fname = "do_passive";
    int             error;
    int             i;
    LinkCB_t       *link;
    int             top_id, log_id;

    link = GetLink (dest_id, 0, &error);
    if (link == NULL) {
	LogError (EC_ERROR, fname, "GetLink failed 0x%x", error);
	return EC_ERROR;
    }
    top_id = NewTop (4);
    log_id = AddTop (top_id, link);

    for (i = 0; i < C_COUNT; ++i) {
	error = Recv (top_id, log_id, (char *) rbuf, p_size);
	if (error < 0) {
	    LogError (EC_ERROR, fname, "RecvLink failed 0x%x", error);
	    return EC_ERROR;
	}
    }

 /* check correctness of last message received */
    error = CheckBuf (p_size);

    error = FreeTop (top_id);
    if (error < 0) {
	LogError (EC_ERROR, fname, "FreeTop failed");
	return EC_ERROR;
    }
    return 0;
}

int
main (int argc, char *argv[])
{
    int             i;
    int             total_time;

    my_id = GET_ROOT ()->ProcRoot->MyProcID;
    n_procs = GET_ROOT ()->ProcRoot->nProcs;

    psize1 = PSIZE1;
    if (argc > 1)
	psize1 = atoi (argv[1]);
    if ((psize1 < 0) || (psize1 > BUFSIZE))
	psize1 = PSIZE1;

    total_size = (n_procs - 1) * C_COUNT * (psize1 + PSIZE2 + PSIZE3 +
					    PSIZE4 + PSIZE5 + PSIZE6);

    say_hello ();

 /* is the topology a pipe? */
    if ((GET_ROOT ()->ProcRoot->DimX != 1) &&
	(GET_ROOT ()->ProcRoot->DimY != 1)) {
	if (my_id == 0) 
	printf ("Configuration Error: Topology is not a pipe!\n");
	return -1;
    }
 /* more than just one processor? */
    if (n_procs < 2) {
	if (my_id == 0)
	    printf ("Configuration Error: Partition contains just one processor!\n");
	return -1;
    }
 /* initialize send buffer */
    for (i = 0; i < ASIZE; i++) {
	sbuf[i] = i;
    }

    if (my_id == 0) {
	total_time = 0;
	for (i = 1; i < n_procs; ++i) {
	    total_time += do_active (i, PSIZE6);
	    total_time += do_active (i, PSIZE5);
	    total_time += do_active (i, PSIZE4);
	    total_time += do_active (i, PSIZE3);
	    total_time += do_active (i, PSIZE2);
	    total_time += do_active (i, psize1);
	}
	printf ("Total: %d KB in %d ms. Rate: %d KB/sec\n",
		total_size / 1024,
		total_time / ONEMSEC_HI,
		total_size / 1024 * ONEMSEC_HI / (total_time / ONEMSEC_HI));
    } else {
	do_passive (0, PSIZE6);
	do_passive (0, PSIZE5);
	do_passive (0, PSIZE4);
	do_passive (0, PSIZE3);
	do_passive (0, PSIZE2);
	do_passive (0, psize1);
    }

    return 0;
}
