/**MO******************* PARIX Test Program ******************************

Name	: SendNode.c
Version : 1.11
Purpose	: performance test of function SendNode/RecvNode
          this program varies
	  - packetsize
	  - count of hops
Topology: pipe 1 x n
Args    : <1 optional packetsize>
Argset 1: -
Argset 2: 1
Argset 3:
History	: created Dec.18.1991 by Sianturi Saleh, Parsytec Aachen

SCCSid:	@(#)SendNode.c	1.2	5/18/92

***/

/**HE*************************   Headers   *******************************/

#include <sys/root.h>
#include <stdio.h>
#include <stdlib.h>

/**PD************   Private Defines and Data structures   ****************/

#define		N_PSIZE		6

/* packet sizes */
#define 	PSIZE1		32
#define 	PSIZE2		256
#define 	PSIZE3		1024
#define 	PSIZE4		4096
#define 	PSIZE5		16384
#define 	PSIZE6		32768

#define		C_COUNT		100
#define		BUFSIZE		65536
#define		ASIZE		(65536 / sizeof (int))
#define		ONESEC_HI	1000000
#define		ONEMSEC_HI	1000

#define		SAY_HELLO_PROC	0

/**SV*********************   Static variables   **************************/

static int      my_id, n_procs;
static int      psize1;
static int      total_size;
static int      sbuf[ASIZE];
static int      rbuf[ASIZE];

/**DF********************   Declared Functions   *************************/

/**SF**********************   Static Functions   ***************************/

void
say_hello (void)
{
    if (SAY_HELLO_PROC == my_id) {
	printf ("************************************************\n");
	printf ("* PARIX test program                           *\n");
	printf ("* SendNode                                     *\n");
	printf ("* -------------------------------------------- *\n");
	printf ("* Written: S. Saleh                            *\n");
	printf ("************************************************\n");
	printf ("Going to transfer %d KB\n",
		total_size / 1024);

	printf ("%d messages/performance test\n", C_COUNT);
    }
}

int
CheckBuf (int p_size)
{
    int             i, epos, err = 0;

    for (i = 0; i < (p_size / sizeof (int)); i++) {
	if (sbuf[i] != rbuf[i]) {
	    if (!err)
		epos = i;
	    err++;
	}
    }
    if (err)
	LogError (EC_ERROR, "CheckBuf",
		  "%d error%s found, first at %d",
		  err, err != 1 ? "s" : "", epos);
    return err;
}

int
do_active (int dest_id, int p_size)
{
    char           *fname = "do_active";
    int             error;
    int             i;
    int             t0, t1;

    t0 = TimeNowHigh ();
    for (i = 0; i < C_COUNT; ++i) {
	error = SendNode (dest_id, 0, (byte *) sbuf, p_size);
	if (error < 0) {
	    LogError (EC_ERROR, fname, "SendLink failed");
	    return EC_ERROR;
	}
    }
    t1 = TimeNowHigh ();

    printf ("Hops: %2d, Packetsize %6d bytes, Time %5d ms. Rate: %4d KB/sec\n",
	    dest_id - 1,
	    p_size,
	    (t1 - t0) / ONEMSEC_HI,
	    C_COUNT * p_size / ((t1 - t0) / ONEMSEC_HI) * 1000 / 1024);

    return (t1 - t0);
}

int
do_passive (int dest_id, int p_size)
{
    char           *fname = "do_passive";
    int             error;
    int             i;

    for (i = 0; i < C_COUNT; ++i) {
	error = RecvNode(dest_id, 0, (byte *) rbuf, p_size);
	if (error < 0) {
	    LogError (EC_ERROR, fname, "RecvLink failed 0x%x", error);
	    return EC_ERROR;
	}
    }

 /* check correctness of last message received */
    error = CheckBuf (p_size);

    return 0;
}

int
main (int argc, char *argv[])
{
    int             i;
    int             total_time;

    my_id = GET_ROOT ()->ProcRoot->MyProcID;
    n_procs = GET_ROOT ()->ProcRoot->nProcs;

    psize1 = PSIZE1;
    if (argc > 1)
	psize1 = atoi (argv[1]);
    if ((psize1 < 0) || (psize1 > BUFSIZE))
	psize1 = PSIZE1;

    total_size = (n_procs - 1) * C_COUNT * (psize1 + PSIZE2 + PSIZE3 +
					    PSIZE4 + PSIZE5 + PSIZE6);

    say_hello ();

 /* initialize send buffer */
    for (i = 0; i < ASIZE; i++) {
	sbuf[i] = i;
    }

    if (my_id == 0) {
	total_time = 0;
	for (i = 1; i < n_procs; ++i) {
	    total_time += do_active (i, PSIZE6);
	    total_time += do_active (i, PSIZE5);
	    total_time += do_active (i, PSIZE4);
	    total_time += do_active (i, PSIZE3);
	    total_time += do_active (i, PSIZE2);
	    total_time += do_active (i, psize1);
	}
	printf ("Total: %d KB in %d ms. Rate: %d KB/sec\n",
		total_size / 1024,
		total_time / ONEMSEC_HI,
		total_size / 1024 * ONEMSEC_HI / (total_time / ONEMSEC_HI));
    } else {
	do_passive (0, PSIZE6);
	do_passive (0, PSIZE5);
	do_passive (0, PSIZE4);
	do_passive (0, PSIZE3);
	do_passive (0, PSIZE2);
	do_passive (0, psize1);
    }

    return 0;
}

