/* SCCSid:	@(#)hyper.c	1.1	5/15/92
 */

#include <errno.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/root.h>
#include <sys/logerror.h>

#define DEBUG

int
main (int argc, char **argv)
{
    int             my_id = GET_ROOT ()->ProcRoot->MyProcID;
    int             nProcs = GET_ROOT ()->ProcRoot->nProcs;

    int		    t1, t2;
    int             report;
    int             order;
    int             top_id;
    int             mask;
    int             i;

/*
 * check argument count
 */
    if (argc > 2) {
	if (0 == my_id) {
	    printf ("Usage: hyper [<reporting processor>]\n");
	}
	exit (1);
    }

/*
 * select reporting processor
 */
    if (2 == argc)
	report = atoi (argv[1]);
    else
	report = nProcs - 1;

/*
 * calculate network order and report it
 */
    for (i = nProcs - 1, order = 0; i != 0; i >>= 1, order++);
    if (my_id == report)
	printf ("Network size: %d, order %d\n", nProcs, order);

/*
 * create new topology
 */
    top_id = NewTop (order);

#ifdef  DEBUG
    LogError (0, "main", "top_id = %d", top_id);
#endif

/*
 * create the links
 */
    for (i = 0, mask = 1; i < order; i++, mask <<= 1) {
	int             target_id = my_id ^ mask;
	int             link_id;
	int             errcode = 0;

    /*
     * watch for nonexistent processor numbers
     */
	if (target_id < nProcs) {

	/*
	 * check whether I am active or passive side, create the link and add
	 * it to the topology
	 */
	    if ((my_id & mask) != 0) {

#ifdef	DEBUG
		LogError (EC_INFO, "main", "1 %d. MakeLink (%d)", i, target_id);
#endif

		link_id = AddTop (top_id, MakeLink (target_id, 0, &errcode));
	    } else {

#ifdef	DEBUG
		LogError (EC_INFO, "main", "1 %d. GetLink (%d)", i, target_id);
#endif

		link_id = AddTop (top_id, GetLink (target_id, 0, &errcode));
	    }
	} else {

	/*
	 * the other processor does not exist, add a null link to the
	 * topology
	 */
	    link_id = AddTop (top_id, NULL);
	}
	if (link_id < 0 || errcode != 0)
	    LogError (0xb0000000, "main", "failed to create link %d to processor %d", i, target_id);
    }

#ifdef	DEBUG
    LogError (EC_INFO, "main", "2 All links created.");
#endif

/*
 * at this point, the Hypercube topology has been established between the
 * existing prcessors and is ready to use.
 */
    for (i = 0, mask = 1; i < order; i++, mask <<= 1) {
	int             target_id = my_id ^ mask;
	int             errcode;
	char            buf[100];

    /*
     * check whether my partner exists, skip transfer if not
     */
	if (target_id >= nProcs)
	    continue;

    /*
     * distinguish between active and passive side
     */
	if ((my_id & mask) != 0) {
	    sprintf (buf, "Message from Processor %2d", my_id);

#ifdef	DEBUG
	    LogError (EC_INFO, "main", "3 %d. Send", i);
#endif

	    errcode = Send (top_id, i, buf, strlen (buf) + 1);

#ifdef	DEBUG
	    LogError (EC_INFO, "main", "3 %d. Recv", i);
#endif

	    errcode = Recv (top_id, i, buf, sizeof (buf));

	    if (errcode < 0)
		printf ("Error from Processor %d: %d,%d\n", my_id, errno, errcode);
	    else
		printf ("Processor %2d got Message #%d :'%s'\n", my_id, i, buf);

	} else {

#ifdef	DEBUG
	    LogError (EC_INFO, "main", "3 %d. Recv", i);
#endif

	    errcode = Recv (top_id, i, buf, sizeof (buf));
	    if (errcode < 0)
		printf ("Error from Processor %d: %d,%d\n", my_id, errno, errcode);
	    else
		printf ("Processor %2d got Message #%d :'%s'\n", my_id, i, buf);
	    sprintf (buf, "Message from Processor %2d", my_id);

#ifdef	DEBUG
	    LogError (EC_INFO, "main", "3 %d. Send", i);
#endif

	    errcode = Send (top_id, i, buf, strlen (buf) + 1);
	}
    }
}
