/* SCCSid:	@(#)Select.c	1.2	5/18/92
 */

#include <sys/root.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/sem.h>
#include <sys/sys_rpc.h>
#include <unistd.h>

#define MASTER_PROC		0
#define MESSAGES_PER_SLAVE	5
#define MAX_MESS_SIZE		50
#define TICKS_PER_SECOND	15625


int             Master (int nSlaves, int *SlaveProcList);
int             Slave (int SlaveID);


int
main (int argc, char **argv)
{
    char           *FName = "main";

    int             nProcs, MyProcID, nSlaves;
    int             slave, *SlaveProcList;
    Thread_t       *MasterThread, **SlaveThreadList;
    int             error, result;
    static char	    buf[100];

    setvbuf (stdout, buf, _IOLBF, sizeof (buf));

    nProcs = GET_ROOT ()->ProcRoot->nProcs;
    MyProcID = GET_ROOT ()->ProcRoot->MyProcID;
    if (argc < 2)
	nSlaves = 2;
    else
	nSlaves = atoi (argv[1]);

    if (MyProcID == 0)
	printf ("%d processors, %d slaves.\n", nProcs, nSlaves);

    SlaveProcList = (int *) malloc (nSlaves * sizeof (int));
    SlaveThreadList = (Thread_t **) malloc (nSlaves * sizeof (Thread_t *));
    if (SlaveProcList == NULL || SlaveThreadList == NULL) {
	LogError (EC_ERROR,
		  FName,
		  "Failed to alloc memory for SlaveList.");
	AbortServer (1);
    }

 /* generate SlaveList: mapping of Slaves to processors */
    for (slave = 0; slave < nSlaves; slave++)
	SlaveProcList[slave] = slave % nProcs;

 /* start Master on processor 0 */
    if (MyProcID == MASTER_PROC) {
	MasterThread = StartThread (Master, 4096, &error,
				  sizeof (nSlaves) + sizeof (SlaveProcList),
				    nSlaves, SlaveProcList);
	if (MasterThread == NULL) {
	    LogError (EC_ERROR,
		      FName,
		      "Failed to start Master.");
	    AbortServer (1);
	}
    }

 /* start Slaves */
    for (slave = 0; slave < nSlaves; slave++) {
	if (MyProcID == SlaveProcList[slave]) {
	    SlaveThreadList[slave] = StartThread (Slave, 4096, &error,
						  sizeof (slave), slave);
	    if (SlaveThreadList[slave] == NULL) {
		LogError (EC_ERROR,
			  FName,
			  "Failed to start Slave %d.",
			  slave);
		AbortServer (1);
	    }
	}
    }

 /* wait for termination */
    if (MyProcID == MASTER_PROC) {
	if (WaitThread (MasterThread, &result) == NULL) {
	    LogError (EC_ERROR,
		      FName,
		      "Error waiting for termination of Master.");
	    AbortServer (1);
	}
    }

    for (slave = 0; slave < nSlaves; slave++) {
	if (MyProcID == SlaveProcList[slave]) {
	    if (WaitThread (SlaveThreadList[slave], &result) == NULL) {
		LogError (EC_ERROR,
			  FName,
			  "Error waiting for termination of Slave %d.",
			  slave);
	    }
	}
    }

    exit (0);
}


int
Master (int nSlaves, int *SlaveProcList)
{
    char           *FName = "Master";

    int             slave;
    LinkCB_t      **SlaveLinkList;
    int             error, i;
    Option_t       *Options;

    SlaveLinkList = (LinkCB_t **) malloc (nSlaves * sizeof (LinkCB_t *));
    if (SlaveLinkList == NULL) {
	LogError (EC_ERROR,
		  FName,
		  "Failed to alloc memory for SlaveLinkList.");
	AbortServer (1);
    }

    for (slave = 0; slave < nSlaves; slave++) {
	SlaveLinkList[slave] = MakeLink (SlaveProcList[slave],
					 slave,
					 &error);
	if (SlaveLinkList[slave] == NULL) {
	    LogError (EC_ERROR,
		      FName,
		      "Master: Failed to MakeLink to Slave %d.",
		      slave);
	    AbortServer (1);
	}
    }

 /* create options for Select */
    Options = (Option_t *) malloc ((nSlaves + 1) * sizeof (Option_t));
    if (Options == NULL) {
	LogError (EC_ERROR,
		  FName,
		  "Failed to alloc memory for Options.");
	AbortServer (1);
    }

    for (slave = 0; slave < nSlaves; slave++)
	Options[slave] = ReceiveOption (SlaveLinkList[slave]);

    for (i = 0; i < nSlaves * MESSAGES_PER_SLAVE;) {
	Options[nSlaves] = TimeAfterOption (TimeNowLow () + TICKS_PER_SECOND);

    /* wait for message from Slave or timeout */
	slave = SelectList (nSlaves + 1, Options);

	if (slave == nSlaves)
	    printf ("Timeout on Select.\n");
	else {
	    char            message[MAX_MESS_SIZE];

	    RecvLink (SlaveLinkList[slave], message, MAX_MESS_SIZE);
	    printf ("%s\n", message);
	    i++;
	}
    }
}


int
Slave (int slave)
{
    char           *FName = "Slave";

    LinkCB_t       *MasterLink;
    int             error, i, MyProcID;
    char            message[MAX_MESS_SIZE];

    MyProcID = GET_ROOT ()->ProcRoot->MyProcID;

    MasterLink = GetLink (MASTER_PROC, slave, &error);
    if (MasterLink == NULL) {
	LogError (EC_ERROR,
		  FName,
		  "Slave %d: Failed to GetLink from Master.",
		  slave);
	AbortServer (1);
    }

    for (i = 0; i < MESSAGES_PER_SLAVE; i++) {
	sprintf (message,
		 "Slave %d on proc %d: message %d.",
		 slave, MyProcID, i);
	SendLink (MasterLink, (byte *) message, MAX_MESS_SIZE);
	TimeWaitLow (TimeNowLow () + 2 * TICKS_PER_SECOND);
    }
}




