#ifndef _MATH_H
#define _MATH_H

/*
 * SCCSID("@(#)math.h	92.1 (92/12/09)");
 *
 * (c) 1985 ACE	Associated Computer Experts bv,
 *		Amsterdam,
 *		The Netherlands.
 */

#ifndef __STDC__

extern double atof(), frexp(), ldexp(), modf();
extern double j0(), j1(), jn(), y0(), y1(), yn();
extern double erf(), erfc();
extern double exp(), log(), log10(), pow(), sqrt();
extern double floor(), ceil(), fmod(), fabs();
extern double gamma(), lgamma();
extern double hypot();
extern double sinh(), cosh(), tanh();
extern double sin(), cos(), tan(), asin(), acos(), atan(), atan2();

#define signgam	(*_OS_signgam_ptr())
extern int signgam;

extern int isnan();
extern int isinf();

#else /* __STDC__ */

double	acos(double _x);
double	asin(double _x);
double	atan(double _x);
double	atan2(double _y, double _x);

double	cos(double _x);
double	sin(double _x);
double	tan(double _x);

double	cosh(double _x);
double	sinh(double _x);
double	tanh(double _x);

double	exp(double _x);
double	log(double _x);
double	log10(double _x);

double	sqrt(double _x);
double	ceil(double _x);
double	fabs(double _x);
double	floor(double _x);

double	pow(double _x, double _y);

double	frexp(double _x, int *_exp);
double	ldexp(double _x, int _exp);
double	modf(double _x, double *_iptr);
double	fmod(double _x, double _y);

double	j0(double _x);
double	j1(double _x);
double	jn(int _n, double _x);

double	y0(double _x);
double	y1(double _x);
double	yn(int _n, double _x);

double	erf(double _x);
double	erfc(double _x);

double	gamma(double _x);
double	lgamma(double _x);
double	hypot(double _x, double _y);

#endif /* __STDC__ */

#ifndef __STDC__
/*	for MAXFLOAT */
#include	<maxflt.h>
#endif

#ifdef NO_IEEE754_INFINITY
# include	<limits.h>
# define HUGE_VAL	DBL_MAX
#else
extern double	_infinity();
# define HUGE_VAL	(_infinity())
#endif

/* some useful constants */
#define M_E	2.7182818284590452354
#define M_LOG2E	1.4426950408889634074
#define M_LOG10E	0.43429448190325182765
#define M_LN2	0.69314718055994530942
#define M_LN10	2.30258509299404568402
#define M_PI	3.14159265358979323846
#define M_PI_2	1.57079632679489661923
#define M_PI_4	0.78539816339744830962
#define M_1_PI	0.31830988618379067154
#define M_2_PI	0.63661977236758134308
#define M_2_SQRTPI	1.12837916709551257390
#define M_SQRT2	1.41421356237309504880
#define M_SQRT1_2	0.70710678118654752440

#define _ABS(x)	((x) < 0 ? -(x) : (x))
#define _REDUCE(TYPE, X, XN, C1, C2)	{ \
	double x1 = (double)(TYPE)X, x2 = X - x1; \
	X = x1 - (XN) * (C1); X += x2; X -= (XN) * (C2); }
#define _POLY1(x, c)	((c)[0] * (x) + (c)[1])
#define _POLY2(x, c)	(_POLY1((x), (c)) * (x) + (c)[2])
#define _POLY3(x, c)	(_POLY2((x), (c)) * (x) + (c)[3])
#define _POLY4(x, c)	(_POLY3((x), (c)) * (x) + (c)[4])
#define _POLY5(x, c)	(_POLY4((x), (c)) * (x) + (c)[5])
#define _POLY6(x, c)	(_POLY5((x), (c)) * (x) + (c)[6])
#define _POLY7(x, c)	(_POLY6((x), (c)) * (x) + (c)[7])
#define _POLY8(x, c)	(_POLY7((x), (c)) * (x) + (c)[8])
#define _POLY9(x, c)	(_POLY8((x), (c)) * (x) + (c)[9])

#endif /* _MATH_H */
