#ifndef RELOC_H
#define RELOC_H

/*
 * 	@(#)reloc.h	92.1	92/12/09	ACE
 *
 * ACE-COFF Utilities.
 *
 * This source file is part of the ACE Compiler Family.
 * Copyright reserved. This source file, or parts of it, may not
 * be reproduced or transmitted in any form, including reprinting,
 * photocopying or microfilming, or by any means, electronic,
 * mechanical or otherwise, or stored in a retrieval system,
 * or used for any purpose, without the prior written permission
 * of ACE Associated Computer Experts bv.
 * This source file is derived from UNIX System 5.0 (Copyright AT&T)
 *
 * Copyright 1992 ACE Associated Computer Experts bv, Amsterdam
 */


struct reloc {
	long	r_vaddr;	/* (virtual) address of reference */
	long	r_symndx;	/* index into symbol table */
	unsigned short	r_type;		/* relocation type */
};


/*
 *   relocation types for all products and generics
 */

/*
 * All generics
 *	reloc. already performed to symbol in the same section
 */
#define  R_ABS		0

/*
 * X86 generic
 *	8-bit offset reference in 8-bits
 *	8-bit offset reference in 16-bits 
 *	12-bit segment reference
 *	auxiliary relocation entry
 */
#define	R_OFF8		07
#define R_OFF16		010
#define	R_SEG12		011
#define	R_AUX		013

/*
 * B16 and X86 generics
 *	16-bit direct reference
 *	16-bit "relative" reference
 *	16-bit "indirect" (TV) reference
 */
#define  R_DIR16	01
#define  R_REL16	02
#define  R_IND16	03

/*
 * 3B generic
 *	24-bit direct reference
 *	24-bit "relative" reference
 *	16-bit optimized "indirect" TV reference
 *	24-bit "indirect" TV reference
 *	32-bit "indirect" TV reference
 */
#define  R_DIR24	04
#define  R_REL24	05
#define  R_OPT16	014
#define  R_IND24	015
#define  R_IND32	016

/* 
 * XL generics
 *	10-bit direct reference
 *	10-bit "relative" reference
 *	32-bit "relative" reference
 */
#define	R_DIR10		025
#define R_REL10		026
#define R_REL32		027

/*
 * 3B and M32 generics
 *	32-bit direct reference
 */
#define  R_DIR32	06

/*
 * M32 generic
 *	32-bit direct reference with bytes swapped
 */
#define  R_DIR32S	012
 
 /*
  * DEC Processors  VAX 11/780 and VAX 11/750 
  *
  */
 
#define R_RELBYTE	017
#define R_RELWORD	020
#define R_RELLONG	021
#define R_PCRBYTE	022
#define R_PCRWORD	023
#define R_PCRLONG	024


#define	RELOC	struct reloc
#define	RELSZ	10	/* sizeof(RELOC) */

#ifdef ACE_COFF

	/* only long relocation used */
#define RLONG		2
#define RSEG_LONG	3

	/* add the others for completeness */
#define RBYTE		4
#define RSEG_BYTE	5
#define RWORD		6
#define RSEG_WORD	7

#endif /* ACE_COFF */

/*
 *	Transputer (T800/T9000) relocation types.
 *		SEG	segment relative
 *		PCR	PC relative
 *		ABS	absolute (0 relative)
 *
 *		BYTEOFF	byte address
 *		WORDOFF word address
 *
 *		INSTR_FIX	fixed number of pfix/nfix
 *		INSTR_FLEX	changeable number of pfix/nfix
 *		WORD		32 bits
 */
#define R_TRANS_SEG_BYTEOFF_INSTR_FIX	040
#define R_TRANS_SEG_BYTEOFF_INSTR_FLEX	041
#define R_TRANS_SEG_BYTEOFF_WORD	042
#define R_TRANS_SEG_WORDOFF_INSTR_FIX	043
#define R_TRANS_SEG_WORDOFF_INSTR_FLEX	044
#define R_TRANS_SEG_WORDOFF_WORD	045
#define R_TRANS_PCR_BYTEOFF_INSTR_FIX	046
#define R_TRANS_PCR_BYTEOFF_INSTR_FLEX	047
#define R_TRANS_PCR_BYTEOFF_WORD	050
#define R_TRANS_ABS_BYTEOFF_INSTR_FIX	051
#define R_TRANS_ABS_BYTEOFF_INSTR_FLEX	052
#define R_TRANS_ABS_BYTEOFF_WORD	053

#endif /* RELOC_H */
