#ifndef	SCNHDR_H
#define	SCNHDR_H
/*
 *	@(#)scnhdr.h	92.1	92/12/09	ACE
 *
 * ACE-COFF Utilities.
 *
 * This source file is part of the ACE Compiler Family.
 * Copyright reserved. This source file, or parts of it, may not
 * be reproduced or transmitted in any form, including reprinting,
 * photocopying of microfilming, or by any means, electronic,
 * mechanical or otherwise, or stored in a retrieval system,
 * or used for any purpose, without the prior written permission
 * of ACE Associated Computer Experts bv.
 * This source file is derived from UNIX System 5.0 (Copyright AT&T)
 *
 * (c) 1985 ACE Associated Computer Experts bv, Amsterdam
 *
 */

struct scnhdr {
	char		s_name[8];	/* section name */
	long		s_paddr;	/* physical address */
	long		s_vaddr;	/* virtual address */
	long		s_size;		/* section size */
	long		s_scnptr;	/* file ptr to raw data for section */
	long		s_relptr;	/* file ptr to relocation */
	long		s_lnnoptr;	/* file ptr to line numbers */
	unsigned short	s_nreloc;	/* number of relocation entries */
	unsigned short	s_nlnno;	/* number of line number entries */
#ifndef ACE_COFF
	long		s_flags;	/* flags */
#else
	union {
		struct {
			short		s_ft_attrs;	/* attributes */
			short		s_ft_type;	/* type */
		} s_t;
		long		s_f_flags;	/* flags */
	} s_f;
	short		s_vstamp;	/* Version Stamp */
#endif
};

#ifdef ACE_COFF
#define	s_type	s_f.s_t.s_ft_type
#define	s_attrs	s_f.s_t.s_ft_attrs
#define	s_flags	s_f.s_f_flags
#endif

#define	SCNHDR	struct scnhdr
#ifndef ACE_COFF
#define SCNHSZ	40
#else
#define	SCNHSZ	42
#define	SCNHSZ5	40
#endif

/*
 * Define constants for names of "special" sections
 */

#define _TEXT	".text"
#define _DATA	".data"
#define _BSS	".bss"
#define _TV	".tv"
#define _LIB	".lib"
#define _INIT	".init"
#define _FINI	".fini"
#ifdef ACE_COFF
#define _STACK	".stack"
#define _USTRUCT ".ustruct"
#define _IDENT	".ident"
#endif
# ifdef SV3
# define _IDENT ".comment"
# endif

/*
 *	In ACE COFF section types are stored in s_type.
 *	In System V COFF the lower 2 bytes are used as section type.
 *	Some types are in fact flags.
 *	The comments after each type have meaning as follows:
 *		allocated:	memory is allocated for the section.
 *		relocated:	the section contents is relocated.
 *		loaded:		the section contents is written to file
 *				(relocated implies loaded).
 */

#define STYP_REG	0x00	/* "regular" section:
				 * allocated, relocated, loaded
				 */
#define STYP_DSECT	0x01	/* "dummy" section:
				 * not allocated, not relocated, not loaded
				 */
#define STYP_NOLOAD	0x02	/* "noload" section:
				 * allocated, not relocated, not loaded
				 */
#define STYP_GROUP	0x04	/* "grouped" section:
				 * used internally by ld
				 */
#define STYP_PAD	0x08	/* "padding" section:
				 * will not appear in output file
				 */
#define STYP_COPY	0x10	/* "copy" section:
				 * not allocated, not relocated, loaded
				 */
#define STYP_TEXT	0x20	/* "text" section:
				 * system 5 flag for .text
				 */
#define STYP_DATA	0x40	/* "data" section:
				 * system 5 flag for .data
				 */
#ifdef ACE_COFF
#define STYP_V2BSS	0x80	/* "bss" section:
				 * system 5 flag for .bss
				 */
#else
#define STYP_BSS	0x80	/* idem */
#endif
/*			0x100	   not used				*/
#define STYP_INFO	0x200	/* "comment" section:
				 * treated like STYP_COPY section
				 */
#define STYP_OVER	0x400	/* "overlay" section":
				 * not allocated, relocated, loaded
				 */
#define STYP_LIB	0x800	/* "lib" section:
				 * for .lib; treated like STYP_COPY?
				 */
#ifdef ACE_COFF
#define STYP_CLEAR	0x1000	/* "clear" section:
				 * allocated, not relocated, not loaded;
				 * contents is cleared by kernel on load/exec
				 */
#define STYP_BSS	0x1000	/* idem; this name is for historical reasons */
#define STYP_INDIR	0x2000	/* "indirect" section:
				 * section contains path name of file
				 * containing real section or yet another
				 * indirect section.
				 */
#define STYP_ASSOC	0x3000	/* "associated" section: ???		*/
#endif

#ifndef ACE_COFF
/*
 *  In a minimal file or an update file, a new function
 *  (as compared with a replaced function) is indicated by S_NEWFCN
 */
#define S_NEWFCN  0x10

/*
 * In 3b Update Files (output of ogen), sections which appear in SHARED
 * segments of the Pfile will have the S_SHRSEG flag set by ogen, to inform
 * dufr that updating 1 copy of the proc. will update all process invocations.
 */
#define S_SHRSEG	0x20
#endif

#endif
