#ifndef SEGMATTRS_H
#define SEGMATTRS_H

/*
 * SCCS:	@(#)segmattrs.h	92.1	92/12/09	(c) ACE
 *
 * ACE-COFF Utilities.
 *
 * This source file is part of the ACE Compiler Family.
 * Copyright reserved. This source file, or parts of it, may not
 * be reproduced or transmitted in any form, including reprinting,
 * photocopying or microfilming, or by any means, electronic,
 * mechanical or otherwise, or stored in a retrieval system,
 * or used for any purpose, without the prior written permission
 * of ACE Associated Computer Experts bv.
 * This source file is derived from UNIX System 5.0 (Copyright AT&T)
 *
 * (c) 1985 ACE Associated Computer Experts bv, Amsterdam
 *
 */



/*
 * Section, segment and view attributes:
 *	
 *	SG_SHARED	"shared" flag 
 *				indicate sharing of segment;
 *				the effective value of this flag is the
 *				minimum	of the values along the INDIR chain;
 *				if the segment is already present, and the value
 *				is true, the segment must be shared;
 *				the value of this flag then influences the
 *				interpretation of most other flags:
 *				 -  if shared: effective flag value is minimum
 *					of flag values along INDIR chain
 *				 -  if not shared: effective flag is 
 *					flag value of section in orig. program
 *	SG_READ    	"readable" flag
 *				segment contains readable data
 *	SG_WRIT    	"writable" flag
 *				segment contains writable data
 *	SG_EXEC      	"executable" flag
 *				segment contains instructions
 *	SG_UPEXT	"upwards extendible" flag
 *				segment will be extendible to the
 *				higher addresses dynamically 
 *				(not yet supported for shared segments)
 *				(excludes downward extensibility)
 *	SG_DOWNEXT	"downwards extendible" flag
 *				segment will be extendible to the
 *				lower addresses dynamically 
 *				(not yet supported for shared segments)
 *				(excludes upward extensibility)
 *	SG_POSINDEP	"position independent" flag
 *				segment can be placed at any
 *				virtual address
 *				(hint not yet supported)
 *	SG_LOD		"load on demand" flag
 *				segment will be loaded on demand 
 *				(hint not yet supported)
 *	SG_WBACK	"write back" flag
 *				when the last ref to the segment
 *				has been released,
 *				the contents of the section in the
 *				COFF file will correspond to the
 *				last contents of the segment 
 *				(not yet supported by kernel)
 *	SG_VSI		"version stamp ignore" flag
 *				the version stamp in this section header
 *				must be ignored 
 *	SG_PRIV		"privileged" flag
 *				the segment will be created in the
 *				UNIX kernel's environment
 *				(not externally made available)
 *				(internally used for kernel private segments)
 *	SG_BONDED	"bonded" flag
 *				the section's virtual address must
 *				not be changed by a link-edit session;
 *				it has already been bonded (in ole
 *				terminology)
 *				(internally used by ole)
 *				(ignored by loader)
 *	SG_GROUP	"grouped" flag
 *				this section and the next one belong
 *				to the same group of sections;
 *				they will be loaded as a unity.
 *	SG_STACK	"stack" flag
 *				the segment created from this section
 *				will be the initial stack segment.
 *	SG_RESERVED	reserved for future expansion
 *	SG_USERDEF	semantics to be defined by the user
 */

/*
 * values of section, segment and view attributes
 */
#define SG_FREE		0x0000		/* section not valid */

#define SG_SHARED	0x0001		/* "shared" flag */
#define SG_READ    	0x0002		/* "readable" flag */
#define	SG_WRIT    	0x0004		/* "writable" flag */
#define SG_EXEC      	0x0008		/* "executable" flag */
#define	SG_UPEXT	0x0010		/* "upwards extendible" flag */
#define	SG_DOWNEXT	0x0020		/* "downwards extendible" flag */
#define	SG_POSINDEP	0x0040		/* "position independent" flag */
#define	SG_LOD		0x0080		/* "load on demand" flag */
#define	SG_WBACK	0x0100		/* "write back" flag */
#define	SG_VSI		0x0200		/* "version stamp ignore" flag */
#define	SG_PRIV		0x0400		/* "privileged" flag */
#define	SG_BONDED	0x0800		/* "bonded" flag (internal in ole) */
#define SG_GROUP	0x1000		/* "group" flag */
#define	SG_STACK	0x2000		/* "stack" flag */
#define SG_RESERVED	0x4000		/* reserved */
#define	SG_USERDEF	0x8000		/* user defined */

#define SG_RWX		(SG_READ|SG_WRIT|SG_EXEC)
#define SG_EXT		(SG_UPEXT|SG_DOWNEXT)

#endif /* SEGMATTRS_H */
