/*
 *	@(#)stdio.h	92.1	92/12/09	ACE
 *
 *	Input/Output.
 */

#ifndef _STDIO_H
#define _STDIO_H

#include <null.h>

#include <sys/size_t.h>

typedef long	fpos_t;

#define _NFILE	20
#define FOPEN_MAX	_NFILE
#define FILENAME_MAX	14

#define BUFSIZ	1024

/* buffer size for multi-character output to unbuffered files */
#define _SBFSIZ 8

typedef struct {
	int	_cnt;
	unsigned char	*_ptr;
	unsigned char	*_base;
	unsigned char	_flag;
	char	_file;
} FILE;

/*
 * _IOLBF means that a file's output will be buffered line by line
 * In addition to being flags, _IONBF, _IOLBF and _IOFBF are possible
 * values for "type" in setvbuf.
 */
#define _IOFBF		0000
#define _IOREAD		0001
#define _IOWRT		0002
#define _IONBF		0004
#define _IOMYBUF	0010
#define _IOEOF		0020
#define _IOERR		0040
#define _IOLBF		0100
#define _IORW		0200

#include <seek.h>

#define EOF		(-1)

#define stdin		(&_iob[0])
#define stdout		(&_iob[1])
#define stderr		(&_iob[2])

#define _bufend(p)	_bufendtab[(p)->_file]
#define _bufsiz(p)	(_bufend(p) - (p)->_base)

#ifdef __STDC__
/*
 *	Prototype declarations should be put before macros #define-s.
 */
int	remove(const char *_filename);
int	rename(const char *_old, const char *_new);
FILE	*tmpfile(void);
char	*tmpnam(char *_s);
int	fclose(FILE *_stream);
int	fflush(FILE *_stream);
FILE	*fopen(const char *_filename, const char *_mode);
FILE	*freopen(const char *_filename, const char *_mode, FILE *_stream);
void	setbuf(FILE *_stream, char *_buf);
int	setvbuf(FILE *_stream, char *_buf, int _mode, size_t _size);
int	fprintf(FILE *_stream, const char *_format, ...);
int	fscanf(FILE *_stream, const char *_format, ...);
int	printf(const char *_format, ...);
int	scanf(const char *_format, ...);
int	sprintf(char *_s, const char *_format, ...);
int	sscanf(char *_s, const char *_format, ...);
int	vfprintf(FILE *_stream, const char *_format, char *_arg);
int	vprintf(const char *_format, char *_arg);
int	vsprintf(char *_s, const char *_format, char *_arg);
int	fgetc(FILE *_stream);
char	*fgets(char *_s, int _n, FILE *_stream);
int	fputc(int _c, FILE *_stream);
int	fputs(const char *_s, FILE *_stream);
int	getc(FILE *_stream);
int	getchar(void);
char	*gets(char *_s);
int	putc(int _c, FILE *_stream);
int	putchar(int _c);
int	puts(const char *_s);
int	ungetc(int _c, FILE *_stream);
size_t	fread(void *_ptr, size_t _size, size_t _nmemb, FILE *_stream);
size_t	fwrite(const void *_ptr, size_t _size, size_t _nmemb, FILE *_stream);
int	fgetpos(FILE *_stream, fpos_t *_pos);
int	fseek(FILE *_stream, long _offset, int _whence);
int	fsetpos(FILE *_stream, fpos_t *_pos);
long	ftell(FILE *_stream);
void	rewind(FILE *_stream);
void	clearerr(FILE *_stream);
int	feof(FILE *_stream);
int	ferror(FILE *_stream);
void	perror(const char *_s);

int	_filbuf(FILE *_stream);
int	_flsbuf(int _c, FILE *_stream);
#endif

#ifndef lint
#define getc(p)		(--(p)->_cnt < 0 ? _filbuf(p) : (int) *(p)->_ptr++)
#define putc(x, p)	(--(p)->_cnt < 0 ? \
			_flsbuf((unsigned char) (x), (p)) : \
			(int) (*(p)->_ptr++ = (unsigned char) (x)))
#define getchar()	getc(stdin)
#define putchar(x)	putc((x), stdout)
#define clearerr(p)	((void) ((p)->_flag &= ~(_IOERR | _IOEOF)))
#define feof(p)		((p)->_flag & _IOEOF)
#define ferror(p)	((p)->_flag & _IOERR)
#define fileno(p)	(p)->_file
#endif

extern FILE	_iob[_NFILE];
extern unsigned char *_bufendtab[];

#ifndef __STDC__
extern FILE	*fopen(), *fdopen(), *freopen(), *tmpfile();
extern long	ftell();
extern void	rewind(), setbuf(), perror();
extern char	*ctermid(), *cuserid(), *fgets(), *gets(), *tempnam();
extern char 	*tmpnam();
extern int 	fclose(), fflush(), fgetc(), fseek(), getw(), rename(), scanf();
extern int	setvbuf(), sprintf(), sscanf(), fprintf(), fputc(), fputs();
extern int	fscanf(), pclose(), printf(), putw(), remove(), ungetc();
extern int	vfprintf(), vprintf(), vsprintf(), puts();
extern size_t	fwrite(), fread();
#endif

#define L_ctermid	9
#define L_cuserid	9
#define P_tmpdir	"/tmp/"
#define L_tmpnam	(sizeof(P_tmpdir) + 42)	/* hitchhacker's? */

#define TMP_MAX		10000	/* See limits.h */

#endif
