/**MO**************************   Module   *******************************

Name	: boot.h
Purpose	:
Notes	:
SccsId	: @(#)boot.h	1.4	2/1/93	
History	: 13.09.91	created by Nowak, Parsytec

***/

#ifndef _boot_h_
#define _boot_h_

/**HE*************************   Headers   *******************************/

#include <sys/link.h>

/**GD************   Global Defines and Data structures   *****************/

#define NO_SWAPPING			0x12345678
#define SWAPPING_NEEDED(HostCB)	((HostCB)->Host->ByteOrder != NO_SWAPPING)
#define BOOT_HOST			(&(GET_ROOT()->HostRoot->HostLinks[0]))
/*
 * processor position
 */

typedef struct {
    int             x;
    int             y;
    int             z;
}               Pos_t;

/*
 * host position
 */

typedef struct {
    Pos_t           proc;
    int             link;
}               HostPos_t;

/*
 * processor description
 */

typedef struct {
    int             MyProcID;		/* my processor number		 */
    int             MyX;		/* my x coordinate		 */
    int             MyY;		/* my y coordinate		 */
    int             MyZ;		/* my z coordinate		 */
    int             nProcs;		/* number of Processors		 */
    int             DimX;		/* array dim. in x direction	 */
    int             DimY;		/* array dim. in y direction	 */
    int             DimZ;		/* array dim. in z direction	 */
}               RootProc_t;

/*
 * description of a single host
 * (the term 'Host' is used for any kind of external
 * processor, e.g. frontends, local or global filesystems)
 */

typedef struct {
    Pos_t           Proc;		/* position and link no. of the
					 * regular	 */
    int             LinkNo;		/* processor that is directly
					 * connected to the host				 */
    int             ID;			/* Host ID (for router)			 */
    int             ByteOrder;
    int             MaxCSize;		/* Limits for size of Ctrl- and	 */
    int             MaxDSize;		/* Data vector */
					/* 0:	unchecked */
                                        /* -1:	unlimited */
    char           *name;		/* Host name				 */
}               HostInfo_t;


/*
 * description of Host Link
 */

typedef struct {
    LinkCB_t       *Link;
    HostInfo_t     *Host;
}               HostLinkCB_t;

/*
 * description of Hosts
 * any processor which is not part of user's partition is
 * regarded as a 'Host'
 */

typedef struct {
    int             nHosts;		/* Number of Hosts	 */
    HostInfo_t     *Hosts;		/* Description of Hosts	 */
    HostLinkCB_t   *HostLinks;
}               RootHosts_t;

/*
 * placing router in memory:
 * 
 * ROUTER_FULL_INTERN : code and stack intern.
 * ROUTER_HALF_INTERN : code extern and stack intern.
 * ROUTER_FULL_EXTERN : code and stack extern.
 */

#define	ROUTER_FULL_INTERN	0
#define	ROUTER_HALF_INTERN	1
#define	ROUTER_FULL_EXTERN	2

#endif					/* _boot_h_ */

/*** end of boot.h ***/
