/**MO*************************** Module ********************************

Name    :   context.h
Purpose :
Notes   :   
SccsId	:   @(#)context.h	1.7	2/1/93	
History :   created Sep/13/91 by Nowak, Parsytec Chemnitz
            modified Oct/24/91 by Schneider, Parsytec Chemnitz
	    modified Nov/18/91 by Schneider, Parsytec Chemnitz
	    changed Jan/06/92 by Schneider, Parsytec Chemnitz
            changed Feb/20/92 by Schneider, Parsytec Chemnitz

***/

#ifndef _context_h_
#define _context_h_

/**HE************************* Headers ********************************/

#include <sys/list.h>
#include <sys/sem.h>
#include <sys/thread.h>


/**GD************* Global Defines and Data Structures *****************/

#define  MAX_NEIGHBOURS     4
#define  CACHE_TABLE_SIZE   1024

/*
 *  sizes of headers
 */
#define  FILE_HDR_SIZE  20
#define  AOUT_HDR_SIZE  32
#define  SCN_HDR_SIZE   42
#define  COFF_HDR_SIZE  (FILE_HDR_SIZE + AOUT_HDR_SIZE + \
			 (3 * SCN_HDR_SIZE))
#define  RELOC_SIZE     10

/*
 *  offsets of headers within a COFF header
 */
#define  FILE_HDR   0
#define  AOUT_HDR   FILE_HDR_SIZE
#define  DATA_HDR   (FILE_HDR_SIZE + AOUT_HDR_SIZE)
#define	 BSS_HDR    (FILE_HDR_SIZE + AOUT_HDR_SIZE + SCN_HDR_SIZE)
#define	 TEXT_HDR   (FILE_HDR_SIZE + AOUT_HDR_SIZE + (2 * SCN_HDR_SIZE))

/*
 *  offsets within the file header
 */
#define  F_MAGIC    0
#define  F_NSCNS    2
#define  F_TIMDAT   4
#define  F_SYMPTR   8
#define  F_NSYMS    12
#define  F_OPTHDR   16
#define  F_FLAGS    18

/*
 *  offsets within the optional header
 */
#define  MAGIC      0
#define  VSTAMP     2
#define  TSIZE      4
#define  DSIZE      8
#define  BSIZE      12
#define  ENTRY      16  
#define  TEXT_START 20
#define  DATA_START 24
#define  SSIZE      28

/*
 *  offsets within the section header
 */
#define  S_NAME     0
#define  S_PADDR    8
#define  S_VADDR    12
#define  S_SIZE     16
#define  S_SCNPTR   20
#define  S_RELPTR   24
#define  S_LNNOPTR  28
#define  S_NRELOC   32
#define  S_NLNNO    34
#define  S_ATTRS    36
#define  S_TYPE     38
#define  S_VSTAMP   40

/*
 *  offsets within a relocation entry
 */
#define  R_VADDR    0
#define  R_SYMNDX   4
#define  R_TYPE     8

/*
 *  header of an executable COFF file (ACE like)
 */
typedef struct {
    byte            Hdr [COFF_HDR_SIZE];
} COFF_t;

/*
 *  relocation entry
 */
typedef struct {
    byte            Rel [RELOC_SIZE];
} Reloc_t;


/*
 *  header of a COFF file (converted in internal PARIX COFF format)
 */

typedef struct {
    int             f_magic;
    int             f_nscns;
    int             f_timdat;
    int             f_symptr;
    int             f_nsyms;
    int             f_opthdr;
    int             f_flags;
} PFileHdr_t;

typedef struct {
    int             magic;
    int             vstamp;
    int             tsize;
    int             dsize;
    int             bsize;
    int             entry;
    int             text_start;
    int             data_start;
    int             ssize;
} PAOutHdr_t;

typedef struct {
    char            s_name [8];
    int             s_paddr;
    int             s_vaddr;
    int             s_size;
    int             s_scnptr;
    int             s_relptr;
    int             s_lnnoptr;
    int             s_nreloc;
    int             s_nlnno;
    int             s_attrs;
    int             s_type;
    int             s_vstamp;
} PScnHdr_t;

typedef struct {
    int             r_vaddr;
    int             r_symndx;
    int             r_type;
} PReloc_t;


/*
 *  structure of an executable PARIX COFF file
 */
typedef struct {
    PFileHdr_t      FileHdr;
    PAOutHdr_t      AOutHdr;
    PScnHdr_t       DataHdr;
    PScnHdr_t       BssHdr;
    PScnHdr_t       TextHdr;
} PCOFF_t;

typedef enum   {MINUS_X, MINUS_Y, PLUS_X, PLUS_Y} Dir_t;
typedef enum   {REQUESTED, AVAILABLE, INVALID} CCStatus_t;

/*
 *  control structure of an executable file (used for code caching)
 */ 
typedef struct {
    char           *tag;
    PCOFF_t        *code;
    int             code_size;
    int             nAccess;
    int             nPackets;
    int             nUser;
    CCStatus_t      status;
    Semaphore_t     ready;
} CodeCB_t;

/*
 *  structure of a process context
 */

typedef struct {
    Node_t          link;           /* Link in context list           */
    List_t          MemBlocks;      /* List of allocated memory segs. */
    List_t          threads;        /* List of threads                */
    int             nThread;        /* Number of threads              */
    Semaphore_t     any_terminate;
    PCOFF_t        *code;           /* Begin of PARIX COFF file       */
    byte           *stack_addr;
    int             stack_size;
    byte           *static_addr;
    int             static_size;
    byte           *text_addr;
    int             text_size;
    int             id;             /* Context identifier             */
} Context_t;


/*
 *  structure of context environment
 */

typedef struct {
    char          **ArgV;           /* Arguments                      */
    char          **EnvV;           /* Environment strings            */
    char          **DirV;           /* root and current dir           */
    void           *LinkV;          /* Links to inherit               */
    void           *TopV;           /* Topologies to inherit          */
} Environ_t;

/*
 * extern reference for own parix environment
 */
extern Environ_t * parix_environ;

/**DF**************** Declared Functions **********************/

#ifdef __STDC__

extern int         _InitCacheHandling (void);
extern Thread_t *  _InitContext (Context_t *);
extern void        _FreeContext (Context_t *);
extern Thread_t *  _InitThread (Context_t *, int *);
extern int         RunMain (COFF_t *, Environ_t *);
extern int         Execute (char *, Environ_t *);
extern void        Exit (int);

#else

extern int         _InitCacheHandling ( );
extern Thread_t *  _InitContext ( );
extern void        _FreeContext ( );
extern Thread_t *  _InitThread ( );
extern int         RunMain ( );
extern int         Execute ( );
extern void        Exit ( );

#endif /* __STDC__ */

#endif /* _context_h_ */

/*** end of header context.h ***/
