/* @(#)crt.h	92.1 92/12/09 (C)ACE */

/*
**	This is the central reference file for the T800 C runtime parameters. 
**	The bare essentials of the general interfaces should go here.
**   	This file is included both by C source and assembler source so:
**   	DO NOT USE EXPRESSIONS in the #defines. 
*/

/*
	General remarks:

	'Lo' indicates the lowest memory address of a segment.
	'Hi' indicates the address just beyond the segment.

	All segments should be word aligned.
*/

/* 
	Memory Descriptor List:
	
	These are word offsets in the MDL where certain pointers should be
	stored.

*/

/*	magic cookie, should be set to SYS_MAGIC_MDL if in ACE rt model	*/
#define SYS_mdl_Magic		0

/*	REQUIRED:	pointer to the start of the data segment (data+bss) */
#define	SYS_mdl_DataLo		1

/*	OPTIONAL:	pointer to the end of the data segment */
#define	SYS_mdl_DataHi		2

/*	REQUIRED:	pointer to the start of the stack */	
#define SYS_mdl_StackLo		3

/*	REQUIRED:	pointer to the end of the stack */
#define	SYS_mdl_StackHi		4

/*	INITIALLY SET BY OS, MODIFIED BY ACE: pointer to end of overflow region. */
#define	SYS_mdl_StackOvfl	5

/*	OPTIONAL:	pointer to the start of the text segment */
#define	SYS_mdl_TextLo		6

/*	OPTIONAL:	pointer to the end of the text segment */
#define	SYS_mdl_TextHi		7

/*	REQUIRED:	Initial stack class or 0 */
#define SYS_mdl_StackClass	8

/*	Number of words used by runtime in MDL */
#define	SYS_mdl_WordLength	9

/*	MDL[SYS_mdl_Magic] */
#define	SYS_MAGIC_MDL		6646416

/* 
	Stack Fragment Information:

	These are word offsets from StackHi, related to information
	concerning the scattered stack.

*/

/*	INIT to SYS_NULL_ptr:	pointer to the next stack fragment	*/ 
#define	SYS_sfi_Next	-1
/*	INIT to SYS_NULL_ptr:	pointer to the previous stack fragment	*/
#define	SYS_sfi_Prev	-2
/*	REQUIRED:	pointer to StackLo of this fragment	*/
#define	SYS_sfi_Lo	-3
/*	total length: */
#define	SYS_sfi_Length	4


/*	
 *	The Stack Extension Routine
 */

/* 	the name of the main routines */
#define SYS_ser_barename	__CRT_stack_extender
#define SYS_ser_name		"__CRT_stack_extender"

#define SYS_ser_fatal_C_name	_CRT_stack_fatal
#define SYS_ser_fatal_barename	__CRT_stack_fatal
#define SYS_ser_fatal_name		"__CRT_stack_fatal"

/*	word offsets in stack directly after calling the stack extension routine */
#define SYS_ser_retadd		0
#define SYS_ser_ResMech		1
#define	SYS_ser_NeedBytes	2
#define	SYS_ser_PrefBytes	3

/*
**	stack extension class range constants:
*/
#define	SYS_ser_classLo		0
#define	SYS_ser_classHi		15

/*	
 *	shifts and masks related to stack extension flag bitfield.
 *	to extract value, first shift, then mask. 
 */
#define SYS_ser_srm_shift			0
#define SYS_ser_pref_class_shift	3
#define SYS_ser_force_class_shift	7
#define SYS_ser_srm_mask			0x1
#define SYS_ser_class_mask			0xf

/*	possible settings for SYS_ser_ResMech: */
#define SYS_srm_VOID	0
#define	SYS_srm_Areg	2
#define	SYS_srm_FAreg	4
#define SYS_srm_sn		0
#define SYS_srm_db		1
#define SYS_srm_snFAreg	4		/*	FAreg | sn	*/
#define	SYS_srm_dbFAreg	5		/*	FAreg | db	*/

/*	
 * 	Function entry constants 
 */

/*	Word offsets in stack directly after function entry. */	
#define	SYS_fnent_retadd	0	/*	return address	*/
#define	SYS_fnent_MDLptr	1	/*	pointer to Memory Descriptor List	*/
#define	SYS_fnent_argptr	2	/*	pointer to argument list	*/
#define	SYS_fnent_resptr	3	/*	pointer to structure return value	*/

/*	Word offsets in the stack during function body, after prologue	*/

/*	Frame Link and Entry Point, help debuggers unwind the stack	*/
#define SYS_fnbod_FrameLnk	1
#define	SYS_fnbod_EntryPnt	2

/* Assorted Constants	*/
#define SYS_NULL_ptr	0
