
/*-------------------------------------------------------------*\ 
|								|
		@(#)limits.h	92.1 92/12/09 (c) ACE
|								|
|	***** Implementation specific constants. *****		|
|								|
|   Conforms to :				|
|    - X/Open Portability Guide (Dec '88): limits(5)		|
|    - ANSI-C standard X3.159 1989 ( OSI/IEC DIS 9899 )		|
|								|
|	(c)	ACE Associated Computer Experts bv.		|
|		Amsterdam, The Netherlands.			|
|								|
\*-------------------------------------------------------------*/

#ifndef _LIMITS_H
#define _LIMITS_H

#ifndef __STDC__    
	/* standard:	X/OPEN Portability Guide (July 1985) */
	/*		inconsistent with ANSI-C standard    */	
#ifndef _FLOAT_H
#include <float.h>
#endif

#endif	/* ifndef __STDC__ */

/*
 *	Each define depends on facilities offered by
 *	certain system entities; the following ones are recognized:
 *		ARCHITECTURE:	for sizes of ints, longs, etc.
 *		FLOATING:	for precision of doubles, etc.
 *		KERNEL:		for system configuration facts, etc.
 *		DISTRIBUTION:	for other entities.
 *
 *	Several definitions are ACE proposed extensions to X/OPEN
 *		to make the set of facilities and names consistent.
 *
 *	Note that some KERNEL values are redefined during configuration
 *		(SYS_OPEN, PROC_MAX and LOCK_MAX)
 *		 and the values here are mere maxima.
 */

	/* max length of argument to exec(2) */ /* 4096 */
	/* defined by:	KERNEL */
	/* standard:	X/OPEN Portability Guide (July 1985) */
#define	ARG_MAX		5120

	/* no. of bits in a char */ /* 8 */
	/* defined by:	ARCHITECTURE */
	/* standard:	X/OPEN Portability Guide (July 1985) */
	/* standard:	ANSI-C X3.159 1989 */
#define	CHAR_BIT	8

	/* max integer value of a char */ /* 127 */
	/* defined by:	ARCHITECTURE */
	/* standard:	X/OPEN Portability Guide (July 1985) */
	/* standard:	ANSI-C X3.159 1989 */
#define CHAR_MAX	SCHAR_MAX

	/* min integer value of a char */ /* 0 */
	/* defined by:	ARCHITECTURE */
	/* standard:	X/OPEN Portability Guide (July 1985) */
	/* standard:	ANSI-C X3.159 1989 */
#define	CHAR_MIN	SCHAR_MIN 

#ifdef __STDC__
	/* number of bytes in multibyte character */
#define MB_LEN_MAX	1
#endif

	/* max no. of processes per user id */ /* 4 */
	/* defined by:	KERNEL */
	/* standard:	X/OPEN Portability Guide (July 1985) */
#define CHILD_MAX	25

	/* max size of a file in bytes */ /* 1000000 */
	/* defined by:	KERNEL */
	/* standard:	X/OPEN Portability Guide (July 1985) */
#define FCHR_MAX	1082201088

	/* no. of bits in a "word" or int */ /* 16 */
	/* defined by:	ARCHITECTURE */
	/* standard:	ACE proposal for X/OPEN extension */
#define	INT_BIT		32

	/* max decimal value of an int */ /* 32767 */
	/* defined by:	ARCHITECTURE */
	/* standard:	X/OPEN Portability Guide (July 1985) */
	/* standard:	ANSI-C X3.159 1989 */
#define INT_MAX		2147483647

	/* min decimal value of a int */ /* -32768 */
	/* defined by:	ARCHITECTURE */
	/* standard:	X/OPEN Portability Guide (July 1985) */
	/* standard:	ANSI-C X3.159 1989 */
#define	INT_MIN		(-2147483647 - 1)

	/* max no. of links to a single file */ /* 8 */
	/* defined by:	KERNEL */
	/* standard:	X/OPEN Portability Guide (July 1985) */
#define LINK_MAX	SHRT_MAX

	/* max no. of entries in system lock table */ /* 32 */
	/* defined by:	KERNEL */
	/* standard:	X/OPEN Portability Guide (July 1985) */
#define LOCK_MAX	2001

	/* no. of bits in a long */ /* 32 */
	/* defined by:	ARCHITECTURE */
	/* standard:	X/OPEN Portability Guide (July 1985) */
#define LONG_BIT	32

	/* max decimal value of a long */ /* 2147483647 */
	/* defined by:	ARCHITECTURE */
	/* standard:	X/OPEN Portability Guide (July 1985) */
	/* standard:	ANSI-C X3.159 1989 */
#define LONG_MAX	2147483647L

	/* min decimal value of a long */ /* -2147483648 */
	/* defined by:	ARCHITECTURE */
	/* standard:	X/OPEN Portability Guide (July 1985) */
	/* standard:	ANSI-C X3.159 1989 */
#define	LONG_MIN	(-2147483647L - 1L)

	/* max size of terminal input line */ /* 255 */
	/* defined by:	KERNEL */
	/* standard:	X/OPEN Portability Guide (July 1991) */
#define MAX_CANON	256

	/* max size of character input buffer */ /* 255 */
	/* defined by:	KERNEL */
	/* standard:	X/OPEN Portability Guide (July 1991) */
#define MAX_INPUT	256

	/* max no. of characters in a file name */ /* 14 */
	/* defined by:	KERNEL */
	/* standard:	X/OPEN Portability Guide (July 1985) */
#define NAME_MAX	14

	/* max no. of characters in a file name */ /* 0 */
	/* defined by:	KERNEL */
	/* standard:	X/OPEN Portability Guide (Dec. 1988) */
#define NGROUPS_MAX	0

	/* max no. of files a process can have open */ /* 16 */
	/* defined by:	KERNEL */
	/* standard:	X/OPEN Portability Guide (July 1985) */
#define OPEN_MAX	20

	/* max no. of significant characters in a password */ /* 8 */
	/* defined by:	DISTRIBUTION */
	/* standard:	X/OPEN Portability Guide (July 1985) */
#define PASS_MAX	8

	/* max no. of characters in a path name */ /* 255 */
	/* defined by:	DISTRIBUTION */
	/* standard:	X/OPEN Portability Guide (July 1985) */
#define PATH_MAX	SHRT_MAX

	/* max value for a process ID */ /* 32000 */
	/* defined by:	KERNEL */
	/* standard:	X/OPEN Portability Guide (July 1985) */
#define PID_MAX		32000

	/* max no. bytes atomic in write to a pipe */ /* 512 */
	/* defined by:	KERNEL */
	/* standard:	X/OPEN Portability Guide (July 1985) */
#define PIPE_BUF	5120

	/* max no. bytes written to a pipe in a write */ /* 4096 */
	/* defined by:	KERNEL */
	/* standard:	X/OPEN Portability Guide (July 1985) */
#define PIPE_MAX	5120

	/* max no. of simultaneous processes, system wide */ /* 8 */
	/* defined by:	KERNEL */
	/* standard:	X/OPEN Portability Guide (July 1985) */
#define PROC_MAX	256

	/* max integer value of a signed char */ /* 127 */
	/* defined by:	ARCHITECTURE */
	/* standard:	ANSI-C X3.159 1989 */
#define	SCHAR_MAX	127

	/* min integer value of a signed char */ /* 0 */
	/* defined by:	ARCHITECTURE */
	/* standard:	ANSI-C X3.159 1989 */
#define	SCHAR_MIN	(-128)

	/* no. of bits in a short */ /* 16 */
	/* defined by:	ARCHITECTURE */
	/* standard:	ACE proposal for X/OPEN extension */
#define SHRT_BIT	16

	/* max decimal value of a short */ /* 32767 */
	/* defined by:	ARCHITECTURE */
	/* standard:	X/OPEN Portability Guide (July 1985) */
	/* standard:	ANSI-C X3.159 1989 */
#define	SHRT_MAX	32767

	/* min decimal value of a short */ /* -32768 */
	/* defined by:	ARCHITECTURE */
	/* standard:	X/OPEN Portability Guide (July 1985) */
	/* standard:	ANSI-C X3.159 1989 */
#define	SHRT_MIN	(-32767 - 1)

	/* no. bytes in a physical IO block */ /* 256 */
	/* defined by:	KERNEL */
	/* standard:	X/OPEN Portability Guide (July 1985) */
#define	STD_BLK		512

	/* max pid of system processes */ /* 1 */
	/* defined by:	KERNEL */
	/* standard:	X/OPEN Portability Guide (July 1985) */
#define SYSPID_MAX	10

	/* no. of chars in uname returned strings */ /* 8 */
	/* defined by:	KERNEL */
	/* standard:	X/OPEN Portability Guide (July 1985) */
#define SYS_NMLN	9

	/* max no. of files open on system */ /* 16 */
	/* defined by:	KERNEL */
	/* standard:	X/OPEN Portability Guide (July 1985) */
#define	SYS_OPEN	800

	/* max no. of unique names generated by tmpnam(3S) */ /* 10000 */
	/* defined by:	DISTRIBUTION */
	/* standard:	X/OPEN Portability Guide (July 1985) */
#define	TMP_MAX		10000

	/* max value for a user or group ID */ /* 32000 */
	/* defined by:	KERNEL */
	/* standard:	X/OPEN Portability Guide (July 1985) */
#define	UID_MAX		60000

	/* max decimal value of an unsigned char */ /* 127 */
	/* defined by:	ARCHITECTURE */
	/* standard:	ACE proposal for X/OPEN extension */
#define	USC_MAX		255
	/* standard:	ANSI-C X3.159 1989 */
#define	UCHAR_MAX	USC_MAX

	/* max decimal value of an unsigned */ /* 65535 */
	/* defined by:	ARCHITECTURE */
	/* standard:	X/OPEN Portability Guide (July 1985) */
#define	USI_MAX		4294967295
#ifdef __STDC__
	/* standard:	ANSI-C X3.159 1989 */
#define	UINT_MAX	4294967295U
#else
#define	UINT_MAX	USI_MAX
#endif

	/* max decimal value of an unsigned long */ /* 65535 */
	/* defined by:	ARCHITECTURE */
	/* standard:	ACE proposal for X/OPEN extension */
#define	USL_MAX		4294967295
#ifdef __STDC__
	/* standard:	ANSI-C X3.159 1989 */
#define	ULONG_MAX	4294967295UL
#else
#define	ULONG_MAX	USL_MAX
#endif

	/* max decimal value of an unsigned short */ /* 65535 */
	/* defined by:	ARCHITECTURE */
	/* standard:	ACE proposal for X/OPEN extension */
#define	USS_MAX		65535
#ifdef __STDC__
	/* standard:	ANSI-C X3.159 1989 */
#define	USHRT_MAX	65535U
#else
#define	USHRT_MAX	USS_MAX
#endif

	/* no. of bits in a "word" or int */ /* 16 */
	/* defined by:	ARCHITECTURE */
	/* standard:	X/OPEN Portability Guide (July 1985) */
#define	WORD_BIT	32

/*
 *	XPG3 extension to POSIX.
 */
#define NL_ARGMAX	9
#define NL_LANGMAX	14
#define NL_MSGMAX	32767
#define NL_NMAX		1
#define NL_SETMAX	255
#define NL_TEXTMAX	255
#define NZERO		20

/*
 *	Invariant constants.
 */
#define _POSIX_ARG_MAX		4096
#define _POSIX_CHILD_MAX	6
#define _POSIX_LINK_MAX		8
#define _POSIX_MAX_CANON	255
#define _POSIX_MAX_INPUT	255
#define _POSIX_NAME_MAX		14
#define _POSIX_NGROUPS_MAX	0
#define _POSIX_OPEN_MAX		16
#define _POSIX_PATH_MAX		255
#define _POSIX_PIPE_BUF		512

#endif /* _LIMITS_H */
