/**MO************************************************************

Name	: memory.h
Purpose	:
Notes	:
SccsId	: @(#)memory.h	1.5	2/1/93	
History	: 14.09.91	created by Nowak, Parsytec
	  27.02.92	modified by Erxleben, Parsytec
	  01.12.92	corrected by Fleischer, Parsytec
	  		#define's for memory functions deleted

***/

#ifndef _memory_h_
#define _memory_h_

/**HE*************************   Headers   *******************************/

#include <stddef.h>
#include <sys/types.h>
#include <sys/sem.h>

/**GD************   Global Defines and Data structures   *****************/

/* memory banking constants */

#define	MC_EXTERNAL		0
#define	MC_INTERNAL		1

#define	NO_OF_MEM_BANKS		2

#define	MC_DEFAULT		MC_EXTERNAL

/* intern transputer memory borders */

#define	INTERN_MEM_START	(void*)0x80000070
#define	INTERN_MEM_END		(void*)0x80001000

/* header of a free memory block linked in freelist queue */

typedef struct header {
    size_t          size;		/* size of this block		 */
    struct header  *left;		/* pointer to the left free neighbour */
    struct header  *right;		/* pointer to the right free
					 * neighbour	 */
}               header_t;

/* structure for function mallinfo */

typedef struct {
    int             arena;		/* total space in arena (in bytes) */
    int             freearena;		/* total free space (in bytes)	 */
    int             usedblocks;		/* number of allocated blocks	 */
    int             freeblocks;		/* number of unused blocks	 */
}               mallinfo_t;

/* memory bank control block */

typedef struct {
    byte           *MemStart;		/* pointer to my memory arena	 */
    byte           *MemEnd;		/* pointer to end of memory arena */
    mallinfo_t      info;		/* info's about the memory management */
    header_t        _dummy;		/* its a always free block in
					 * freelist	 */
    header_t       *_freelist;		/* queue of free memory blocks	 */
    Semaphore_t	   Lock;		/* protection for _freelist */
}               MemoryCB_t;

/* global memory root, includes all memory banks */

typedef struct {
    MemoryCB_t	   MemBank[NO_OF_MEM_BANKS];
}		RootMemory_t;

/**DF********************   Declared Functions   *************************/

#ifdef __STDC__

extern void		*Malloc (size_t, int);
extern void		*Calloc (size_t, size_t, int);
extern void		*Realloc (void *, size_t, int);
extern void		Free (void *);
extern mallinfo_t	MallInfo (int);
extern size_t           MallSize (void *);
extern void		_MemInit (byte *, byte *, int);

#else

extern void		*Malloc ();
extern void		*Calloc ();
extern void		*Realloc ();
extern void		Free ();
extern mallinfo_t	MallInfo ();
extern size_t           MallSize ();
extern void		_MemInit ();

#endif					/* __STDC__ */

#endif					/* _memory_h_ */

/*** end of memory.h ***/
