/**MO**************************	  Module   *******************************

Name	: rrouter.h
Purpose	:
Notes	:
SccsId	: @(#)rrouter.h	1.6	2/1/93	
History : 02.10.91	created  by Erxleben, Parsytec
	  08.11.91	modified by Saleh, Parsytec
	  29.01.92	modified by Erxleben, Parsytec

	  05.02.92	changed  by Erxleben, Parsytec
	  19.02.92	modified by Erxleben, Parsytec

	  17.03.92	modified by Erxleben, Parsytec
	  26.03.92	modified by Erxleben, Parsytec
	  09.04.92	modified by Erxleben, Parsytec

***/

#ifndef _rrouter_h
#define _rrouter_h

/**GD************   Global Defines and Data structures	 *****************/

/*
 * message types for usage by system routines (0x30000000 ... 0x3FFFFFFF)
 */

#define MSG_TYPE_COMM		0x30000000	/* for MakeLink/GetLink */
#define	MSG_TYPE_CONNECT	0x30000001	/* for ConnectLink */
#define	MSG_TYPE_SEND		0x30000002	/* for SendNode/RecvNode */

#define	MSG_TYPE_CC		0x31000000

/*
 * message types for usage by user            (0x00000000 .. 0x2FFFFFFF)
 */

#define MSG_TYPE_USER_START	0x00000000
#define	MSG_TYPE_USER_END	0x2FFFFFFF

/*
 * configuration parameters of software router
 */

#define MAX_MESSAGE_SIZE	1024	/* max size of msg - body */
#define MAX_MAILBOX_SIZE	1000	/* max number of msg in MailBox */

/*
 * max number of parallel software routers on different topologies
 */

#define	MAX_MAILBOX_COUNT	16

#define	SYS_MAILBOX_ID		0

/*
 * flags for reply messages and reply request by ExchangeMessage
 * this flags are set in Msgtype,
 * RR_REPLY_REQUEST is only used by ExchangeMessage
 */

#define	RR_REPLY_REQUEST	0x40000000	/* this message needs a reply */
#define	RR_IS_REPLY		0x80000000	/* its a reply message */

#define NO_TIMEOUT		-1	/* wait for ever */

/* timeout for system reply messages */

#define ERR_TIMEOUT		NO_TIMEOUT

/*
 * terminate code for user router threads;
 * threads terminates, if (Size == Code == ReqId == RR_TERMINATE_CODE)
 */

#define	RR_TERMINATE_CODE	-123	/* must be greater then error codes */
#define	RR_TERMINATE_REQUEST	0x3FFFFFFE
#define	RR_TERMINATE_NOW	0x3FFFFFFF

/*
 * Errorcodes for RandomRouter
 */

#define RR_OUT_OF_MEM		-2	/* out of system-memory */
#define RR_MAILBOX_FULL		-3	/* no place in MailBox */
#define	RR_TIMEOUT		-4	/* send timeout reply message */

/*
 * Errorcodes for GetMessage
 */

#define	GM_TIMEOUT		-5	/* GetMessage timeout arraived */

/*
 * type of message header
 */

typedef struct {
    int             Size;
    int             DestProcId;
    int             ReqId;
    int             MsgType;
    int             SourceProcId;
    int             Code;
    int             Timeout;
}               RR_Header_t;

/*
 * type of whole message
 */

typedef struct {
    RR_Header_t     Header;
    byte            Body[MAX_MESSAGE_SIZE];
}               RR_Message_t;

/*
 * type of wait list for received messages
 */

typedef struct {
    Node_t          Node;
    RR_Message_t    Msg;
}               RR_WaitList_t;

/*
 * type of wait list for expected messsages
 */

typedef struct {
    Node_t          Node;
    RR_Header_t     Header;
    Semaphore_t     Sem;
    RR_WaitList_t  *MsgPointer;
}               RR_RecvWaitList_t;

/*
 * type of elements of the reply thread list
 */

typedef struct {
    Node_t          Node;
    int		    DestId;
    int		    ReqId;
    int		    MsgType;
    int		    Code;
}               RR_ReplyNode_t;

/*
 * type of mailbox control block for one topology
 */

typedef struct {
    Semaphore_t     Lock;		/* for protect Mail and RecvBox    */
    int             MailBoxSize;	/* number of MailBox entry's	   */
    List_t          MailBox;		/* components RR_WaitList_t        */
    List_t          RecvBox;		/* components RR_RecvWaitList_t    */
    List_t	    ReplyBox;		/* for replys only		   */
    Topology_t     *MyTop;		/* pointer to used topology 	   */
    int            (*RoutingDirection) ();
    Semaphore_t	    ReplySync;		/* syncronize reply sending	   */
    List_t	    ReplyList;		/* list for reply messages	   */
    int		    ReplyCount;		/* number of reply list entries    */
    Semaphore_t	    ReplyLock;		/* protect reply list		   */
    Thread_t	   *ReplyThread;	/* for WaitThread		   */
}               MailBoxCB_t;

/*
 * type of mailbox root
 */

typedef struct {
    int             MailBoxCount;	/* number of used MailBoxCBs */
    MailBoxCB_t    *MailBoxCBs[MAX_MAILBOX_COUNT];
    Semaphore_t	    ConnectLinkSem;	/* for connect local links */
}               RootMailBox_t;


/**DF********************   Declared Functions	 *************************/

#ifdef __STDC__
extern int      UserGetMessage (int, int, int, int, int, RR_Message_t *);
extern int      UserPutMessage (int, int, int, int, int, int, void *, int);
extern int      UserExchangeMessage
                (int, int, int, int, int, int, void *, int, RR_Message_t *);

extern int	GetMessage (int, int, int, int, RR_Message_t *);
extern int	PutMessage (int, int, int, int, int, void *, int);
extern int      ExchangeMessage 
		(int, int, int, int, int, void *, int, RR_Message_t *);

extern int	SysRoutingDirection (int);
extern int      InitUserRouter (int, int (*) ());
extern int      InitMailBoxRoot (void);
extern int	TerminateUserRouter (int);

#else

extern int      UserGetMessage ();
extern int      UserPutMessage ();
extern int      UserExchangeMessage ();

extern int      GetMessage ();
extern int      PutMessage ();
extern int      ExchangeMessage ();

extern int	SysRoutingDirection ();
extern int      InitSysRouter ();
extern int      InitUserRouter ();
extern int      InitMailBoxRoot ();
extern int	TerminateUserRouter ();

#endif					/* __STDC__ */

#endif					/* _rrouter_h */

/*** end of rrouter.h ***/
