/**MO**************************   Module   *******************************

Name	: topology.h
Purpose	: data structure for virtual topologies
Notes	:
SccsId	: @(#)topology.h	1.4	2/1/93	
History	: 13.09.91  created by Nowak, Parsytec
           8.08.92  modified by Sianuri Saleh,Regina Wagner, Parsytec Aachen
***/

#ifndef _topology_h_
#define _topology_h_

/**GD************   Global Defines and Data structures   *****************/

/*
 * constants for coding topology identifiers
 */
#define TOPID_MASK_IX     0x0000ffff	/* own index in topology table */
#define TOPID_MASK_COUNT  0x00ff0000	/* usage count                 */
#define TOPID_MASK_INC    0x00010000	/* usage count                 */
#define TOPID_MASK_USED   0xff000000	/* 0 if not used otherwise 1   */

/* #define MAX_TOPOLOGIES	64 */

/*
 * asynchronous communication counter control
 */
typedef struct {
    Semaphore_t async_sem;
    int async_cnt;
    int comm_cnt;
} ACControl_t;

/*
 * asynchronous communication thread descriptor
 */
typedef struct ATData_t {
    Semaphore_t for_work;
    ACControl_t *acc;
    LinkCB_t *link;
    int *result;
    byte *data;
    int size;
    int ifbuffer;
    int action;
    Thread_t *tp;
} ATData_t;

/*
 * global data for managing asynchonous communications
 */
typedef struct ASyncData_t {
    Semaphore_t lock;
    int n_acc;
    ACControl_t acc0;
    ACControl_t *acc;
    int size;
    int used_size;
    int	n_threads;
    int	n_threads_const;
    int p_ready_thread;
    ATData_t **ready_thread;
} ASyncData_t;

typedef struct Topology_t {
    Semaphore_t     Lock;
    LinkCB_t      **Links;		/* MaxLinks elements */
    void           *UserData;
    ASyncData_t    *ad;
    int             MyId;
    int             MaxLinks;
    int             UsedLinks;
    Context_t      *Owner;
}               Topology_t;

typedef struct {
    Topology_t    **Tops;		/* array of Topology pointers	 */
    int             nTops;		/* array size			 */
    int             uTops;		/* number of used slots		 */
    Semaphore_t     Lock;		/* lock for ptr reallocation	 */
}               RootTops_t;

/**DF********************   Declared Functions   *************************/

#ifdef	__STDC__
extern int      NewTop (int TopId);
extern int      AddTop (int TopId, LinkCB_t * Link);
extern int      AddTop_Data (int TopId, void *Data);
extern void    *GetTop_Data (int TopId, int *Err_code);
extern LinkCB_t *GetLinkCB (int TopId, int LogLinkId, int *Err_code);
extern int      FreeTop (int TopId);

#else

extern int      NewTop ();
extern int      AddTop ();
extern int      AddTop_Data ();
extern void    *GetTop_Data ();
extern LinkCB_t *GetLinkCB ();
extern int      FreeTop ();

#endif					/* __STDC__ */

#endif					/* _topology_h_ */

/*** end of topology.h ***/
