/**MO**************************   Module   *******************************

Name	: vlink.h
Purpose	:
Notes	:
SccsId	: @(#)vlink.h	1.3	2/1/93	
History	: 04.09.91	created by Nowak, Parsytec
          17.12.91	rejoined by Fleischer, Parsytec

***/

#ifndef _vlink_h
#define _vlink_h

/**GD************   Global Defines and Data structures   *****************/

#ifdef T8
/* types and parameters for the software router */

#define PACKETSIZE	120
#define HEADERSIZE	4
#define EOP		0
#define EOM		1
#define ACK		( ( EOP << 7 ) & 0x80 )	/* EOP and length = 0 */
#define ESC		( ( EOM << 7 ) & 0x80 )	/* ??? EOM and length = 0 ??? */

#define _SELECT_ON_

/*
 * structure of a virtual link control block
 */

typedef struct {
    int             status;		/* comm errors, mode etc. */
    int             header;		/* header of the communication
					 * partner */
    Channel_t       ctrlin;		/* channel to the input control
					 * process */
    Channel_t       ctrlout;		/* channel to the output control
					 * proc. */
    byte           *outbuf;		/* address from which data may be
					 * sent */
    int             outlimit;		/* count of bytes to send */
    byte           *inbuf;		/* address to which data may be
					 * written */
    int             inlimit;		/* max. count of bytes to receive */
    void           *RCptr;		/* next in resource channel queue */
    int             RCid;		/* id of resource channel */
#ifdef _SELECT_ON_
    int		    select_ws[5];
#endif
    byte            buffer[PACKETSIZE];	/* input packet buffer */
}               VLCB_t;

/*
 * definition of VLCB.status
 */

#define VlinkUsed	0x40		/* VLCB is in use */
#define VlinkResource	0x4000		/* Virtual Link is operating in
					 * resource channel mode */


#define VlinkInError	0x100		/* error on input channel */
#define VlinkOutError	0x200		/* error on output channel */

/* the following states of a Vlink are important for error recovery */
#define VlinkPSent	0x400		/* packet sent but no acknowledge
					 * received */
#define VlinkPRecv	0x800		/* packet received but no acknowledge
					 * sent (no user input) */
#define VlinkPAckn	0x1000		/* packet received and acknowledge
					 * sent and not EOM */

/* the following states of a Vlink are used  for the syncronisation
   of the router */

#define VlinkStop	0x2000		/* channel is stopped (by user or
					 * kernel) - no acknowledges send */
#define BUF_FREE	0x4		/* packet received, stored in
					 * vlcb_buffer */
#define READY_TO_REC	0x8		/* the vcp is ready to receive, the
					 * input_pointer in the vlcb is valid */
#define OVERFLOW	0x10		/* more bytes received than requested */
#define EOM_RECEIVED	0x20		/* EOM seen by input link guardien */
#define PACKET_ARRIVED	0x40		/* 1. packet in VLCB-Buffer */
#define READY_TO_OUTPUT	0x80		/* outputrequest by user received */
#define ACK_RECEIVED	0x100		/* pre ack. of 1. packet */
#define READY_FOR_IN_SYNC	0x200


/*
 * structure of a virtual link (it simulates the mapping of a virtual link
 * on a virtual link control block which is done automatically on T9000
 */

typedef struct {
    Channel_t       in;			/* input channel of one virtual link */
    Channel_t       out;		/* output channel ... */

#ifdef _SELECT_ON_
    Channel_t       select;		/* channel to enable transputers
					 * alternative instructions */
#endif					/* _SELECT_ON_ */
}               VLink_t;

/*
 * structure of an user input or output request
 */

typedef struct {
    byte           *BuffPtr;
    int             BuffSize;
}               CommArgs_t;

/*
 * structure of an router intervall
 */

typedef struct {
    int             LowerBound;
    int             UpperBound;
    int             PhysOutLink;
}               Intervall_t;

#endif					/* T8 */

/**DF********************   Declared Functions   *************************/

#ifdef __STDC__
extern int      AllocVLink (void);
extern int      FreeVLink (int);
extern int      SetHeader (int, byte *, int);
extern int      GetHeader (int, byte *, int *);

extern int      StopChannel (int, int);
extern int      ResetChannel (int, int);
extern int      InspectHeader (int, byte *, int *);

#else

extern int      AllocVLink ();
extern int      FreeVLink ();
extern int      SetHeader ();
extern int      GetHeader ();

extern int      StopChannel ();
extern int      ResetChannel ();
extern int      InspectHeader ();

#endif					/* __STDC__ */

#endif					/* _vlink_h */

/*** end of vlink.h ***/
