#ifndef TERMIOS_H
#define TERMIOS_H

/*
 * SCCSID("@(#)termios.h	92.1 (92/12/09)");
 *
 * (c) 1991 ACE	Associated Computer Experts bv,
 *		Amsterdam,
 *		The Netherlands.
 *
 * Terminal I/O control; X/Open PG3 extension.
 */

typedef unsigned int	speed_t;
typedef unsigned int	tcflag_t;
typedef unsigned int	cc_t;

# define	NCCS	10	/* control chars */

struct termios
{
	tcflag_t	c_iflag;	/* input mode */
	tcflag_t	c_oflag;	/* output mode */
	tcflag_t	c_cflag;	/* control mode */
	tcflag_t	c_lflag;	/* line discipline mode */
	cc_t	c_cc [NCCS];		/* control char list */
};

/* Control characters; index in c_cc[] of struct termios. */
# define VINTR	0	/* SIGINT character */
# define VQUIT	1	/* SIGQUIT character */
# define VERASE	2	/* erase character */
# define VKILL	3	/* kill line character */
# define VMIN	4	/* MIN character */
# define VEOF	4	/* end-of-file character */
# define VEOL	5	/* 1st end-of-line character */
# define VTIME	5	/* time query character */
# define VEOL2	6	/* 2nd end-of-line character */
# define VSUSP	7	/* job suspend character */
# define VSWTCH	7	/* shl(1) switch character */
# define VSTART	8	/* start flow character */
# define VSTOP	9	/* stop flow character */

#define	IOCTYPE	0xff00

# define TIOC		('T'<<8)
# define TCGETA		(TIOC|1)
# define TCSETA		(TIOC|2)
# define TCSETAW	(TIOC|3)
# define TCSETAF	(TIOC|4)
# define TCSBRK		(TIOC|5)
# define TCXONC		(TIOC|6)
# define TCFLSH		(TIOC|7)
# define TCDSET		(TIOC|32)

/* Action; 2nd arg of tcsetaddr(). */
# define TCANOW		1
# define TCADRAIN	2
# define TCAFLUSH	4

# define TCOWAIT	070

# define TCIFLUSH	0
# define TCOFLUSH	1
# define TCIOFLUSH	2

# define TCOOFF		0
# define TCOON		1
# define TCIOFF		2
# define TCION		3


/* default control chars */
#define	CNUL	0
#define	CDEL	0377
#define	CESC	'\\'
#define	CINTR	0177	/* DEL */
#define	CQUIT	034	/* FS, cntl-| */
#define	CERASE	'#'
#define	CKILL	'@'
#define	CEOF	04	/* cntl-D */
#define	CSTART	021	/* cntl-Q */
#define	CSTOP	023	/* cntl-S */
#define	CSWTCH	032	/* cntl-Z */
#define	CNSWTCH	0

/* input modes */
#define	IGNBRK	0000001
#define	BRKINT	0000002
#define	IGNPAR	0000004
#define	PARMRK	0000010
#define	INPCK	0000020
#define	ISTRIP	0000040
#define	INLCR	0000100
#define	IGNCR	0000200
#define	ICRNL	0000400
#define	IUCLC	0001000
#define	IXON	0002000
#define	IXANY	0004000
#define	IXOFF	0010000

/* output modes */
#define	OPOST	0000001
#define	OLCUC	0000002
#define	ONLCR	0000004
#define	OCRNL	0000010
#define	ONOCR	0000020
#define	ONLRET	0000040
#define	OFILL	0000100
#define	OFDEL	0000200
#define	NLDLY	0000400
#define	NL0	0
#define	NL1	0000400
#define	CRDLY	0003000
#define	CR0	0
#define	CR1	0001000
#define	CR2	0002000
#define	CR3	0003000
#define	TABDLY	0014000
#define	TAB0	0
#define	TAB1	0004000
#define	TAB2	0010000
#define	TAB3	0014000
#define	BSDLY	0020000
#define	BS0	0
#define	BS1	0020000
#define	VTDLY	0040000
#define	VT0	0
#define	VT1	0040000
#define	FFDLY	0100000
#define	FF0	0
#define	FF1	0100000

/* baud rate selection */
#define	B0	0
#define	B50	0000001
#define	B75	0000002
#define	B110	0000003
#define	B134	0000004
#define	B150	0000005
#define	B200	0000006
#define	B300	0000007
#define	B600	0000010
#define	B1200	0000011
#define	B1800	0000012
#define	B2400	0000013
#define	B4800	0000014
#define	B9600	0000015
#define	B19200	0000016
#define	B38400	0000017
#define	CBAUD	0000017	/* mask */

/* control modes */
#define	CSIZE	0000060
#define	CS5	0
#define	CS6	0000020
#define	CS7	0000040
#define	CS8	0000060
#define	CSTOPB	0000100
#define	CREAD	0000200
#define	PARENB	0000400
#define	PARODD	0001000
#define	HUPCL	0002000
#define	CLOCAL	0004000
#define	LOBLK	0010000

/* line discipline modes */
#define	ISIG	0000001
#define	ICANON	0000002
#define	XCASE	0000004
#define	ECHO	0000010
#define	ECHOE	0000020
#define	ECHOK	0000040
#define	ECHONL	0000100
#define	NOFLSH	0000200


/* Query functions as separate code. */
extern speed_t	cfgetispeed ();
extern speed_t	cfgetospeed ();
extern int	cfsetispeed ();
extern int	cfsetospeed ();

/* Query functions as macros. */
# define cfgetispeed(T)		(T->c_iflag & CBAUD)
# define cfgetospeed(T)		(T->c_oflag & CBAUD)
# define cfsetispeed(T,S)	(T->c_iflag=(t->c_iflag &~CBAUD) | ((S)& CBAUD))
# define cfsetospeed(T,S)	(T->c_oflag=(t->c_oflag &~CBAUD) | ((S)& CBAUD))

#ifdef __STDC__

int tcdrain (int _fildes);
int tcflow (int _fildes, int _action);
int tcflush (int _fildes, int _queue_selector);
int tcgetattr (int _fildes, struct termios *_termios_p);
int tcsendbreak (int _fildes, int _duration);
int tcsetattr (int _fildes, int _optional_actions, struct termios *_termios_p);

#else /* !__STDC__ */

int tcdrain ();
int tcflow ();
int tcflush ();
int tcgetattr ();
int tcsendbreak ();
int tcsetattr ();

#endif /* __STDC__ */

extern int	_tcstuff ();

#endif	/* TERMIOS_H */
