#ifndef	VIRT_TOP_H
#define	VIRT_TOP_H

/*
**	SccsId:		@(#)virt_top.h	1.2 11/27/92
**	file:		virt_top.h
**	version:	3.8.1992
**	author:		Jens Simon, Uni-Paderborn
**
**	comment:
**		virtual topologies: pipe, ring, 2d-and 3d-grid
**				    2d- and 3d-torus, hypercube and tree
**
*/

#ifndef _ERRNO_H
#include <errno.h>
#endif
#ifndef	NO_ERROR
#define	NO_ERROR	0		/* No error */
#endif
#define	ENOTEPROCS	ENOSPC		/* Not enough processors */
#define	EINPART		ENODEV		/* Wrong slice-parameter of partition */

#define	MINSLICE	-1		/* replacement of min-sliceparameter */
#define	MAXSLICE	-1		/* replacement of max-sliceparameter */

typedef struct {
    int             x;
    int             y;
    int             z;
}               GCPos_t, PPos_t;        /* abstract datatype for
					 * GC- and partition coordinates */
typedef struct {
    int             x;
    int             y;
    int             z;
}               Pos3D_t;		/* abstract datatype for
					 * 3d-grid coordinates */
typedef struct {
    int             x;
    int             y;
}               Pos2D_t;		/* abstract datatype for
					 * 2d-grid coordinates */

/*--------------------------------------------------------------------------- */
/* the 'clique' structure: */

#define	CLIQUE_TYPE	"clique"
#define	CLIQUE_NONE	0
#define	CLIQUE_IN	1

#define	MAXCLIQUE	-1

typedef	struct	{
    char	    type[20];		/* topology name "clique" */
    int		    id;			/* logical number of the node in
					 * clique */
    int		    size;		/* size of the "clique" */
    int		    status;		/* status of the prozessor */
/* internal variables */
    int             dilation;		/* max. dilation */
    PPos_t          p_Dim;		/* dimension of the partition */
    GCPos_t         gc_Offset;		/* offset to the PPos's */
}		CliqueData_t;

extern	int
MakeClique (int reqId, int size,
	    int xmin, int xmax, int ymin, int ymax, int zmin, int zmax);

extern	int
FreeClique (int topId);

extern	CliqueData_t *
GetClique_Data (int topId);

/*--------------------------------------------------------------------------- */
/* the 'pipe' structure: */

#define	PIPE_TYPE	"pipe"

#define	PIPE_NONE	0		/* the processor does not belong to the
					 * virtual topology pipe */
#define PIPE_HEAD	1		/* the first processor of the pipe */
#define PIPE_TAIL	2		/* the last processor of pipe */
#define PIPE_IN		3		/* a processor in the middle of the
					 * pipe */

typedef struct {
    char            type[20];		/* topology name "pipe" */
    int             id;			/* logical number of the node in
					 * the pipe */
    int             status;		/* status of the node in the pipe
					 * ( PIPE_HEAD, PIPE_IN, PIPE_TAIL,
					 *   PIPE_NONE ) */
    int             size;		/* length of the pipe */
/* internal variables */
    int             dilation;		/* max. dilation */
    PPos_t          p_Dim;		/* dimension of the partition */
}               PipeData_t;

extern int
MakePipe (int reqId, int size,
	  int xmin, int xmax, int ymin, int ymax, int zmin, int zmax);

extern int
FreePipe (int topId);

extern PipeData_t *
GetPipe_Data (int topId);

/*--------------------------------------------------------------------------- */
/* the 'ring' structure: */

#define	RING_TYPE	"ring"

#define	RING_NONE	0		/* the processor does not belong to the
					 * virtual topology ring */
#define	RING_IN		3		/* the processor does belong to the
					 * ring */

typedef struct {
    char            type[20];		/* topology name "ring" */
    int             id;			/* logical number of the processor
					 * in the ring */

    int             status;		/* status of the prozessor in the ring
					 * ( RING_IN, RING_NONE ) */
    int             size;		/* length of the ring */
/* internal variables */
    int             dilation;		/* max. dilation */
    PPos_t          p_Dim;		/* dimension of the partition */
}		RingData_t;

extern int
MakeRing (int reqId, int size,
	  int xmin, int xmax, int ymin, int ymax, int zmin, int zmax);

extern int
FreeRing (int topId);

extern RingData_t *
GetRing_Data (int topId);

/*----------------------------------------------------------------------------*/
/* the '2d-grid' structure: */

#define	GRID2D_TYPE	"grid2D"

#define	GRID2D_NONE	0		/* The processor does not belong to the 
					 * virtual topology 2d-grid */
#define	GRID2D_IN	0x01		/* Bit set <=> the processor is a
					 * node of the grid */
#define	GRID2D_DOWNX	0x02		/* Bit set <=> there is a left
					 * grid-neighbour */
#define	GRID2D_UPX	0x04		/* Bit set <=> there is a right
					 * grid-neighbour */
#define	GRID2D_DOWNY	0x08		/* Bit set <=> there is an upper
					 * grid-neighbour */
#define	GRID2D_UPY	0x10		/* Bit set <=> there is a lower
					 * grid-neighbour */

typedef struct {
    char            type[20];		/* topology name "grid2D" */
    Pos2D_t         id;			/* grid id */
    int             status;		/* status of the node in the grid
					 * (GRID2D_DOWNX || GRID2D_UPX
					 * || GRID2D_DOWNY || GRID2D_UPY),
					 * GRID2D_NONE ) */
    Pos2D_t         dim;		/* dimension of the grid */
/* internal variables */
    int             dilation;		/* max. dilation */
    PPos_t          p_Dim;		/* dimension of the partition */
    GCPos_t         gc_Offset;		/* offset to the PPos's */
}               Grid2DData_t;

extern int
Make2DGrid (int reqId, int dim_x, int dim_y,
	  int xmin, int xmax, int ymin, int ymax, int zmin, int zmax);

extern int
Free2DGrid (int topId);

extern Grid2DData_t *
Get2DGrid_Data (int topId);

/*----------------------------------------------------------------------------*/
/* the '3d-grid' structure: */

#define	GRID3D_TYPE	"grid3D"

#define	GRID3D_NONE	0		/* The processor does not belong to the
					 * virtual topology 3d-grid */
#define	GRID3D_IN	0x01		/* Bit set => than the processor is a
					 * node of the grid */
#define	GRID3D_DOWNX	0x02		/* Bit set <=> there is a neighbour to
					 * the left */
#define	GRID3D_UPX	0x04		/* Bit set <=> there is a neighbour to 
					 * the right */
#define	GRID3D_DOWNY	0x08		/* Bit set <=> there is an upper
					 * neighbour */
#define	GRID3D_UPY	0x10		/* Bit set <=> there is a lower
					 * neighbour */
#define	GRID3D_DOWNZ	0x20		/* Bit set <=> there is a neighbour in 
					 * the front */
#define	GRID3D_UPZ	0x40		/* Bit set <=> there is a neighbour in
					 * the back */

typedef struct {
    char            type[20];		/* topology- ame "grid3D" */
    Pos3D_t         id;			/* grid id */
    int             status;		/* status of the node in the grid */
    Pos3D_t         dim;		/* dimension of the grid */
/* internal variables */
    int             dilation;		/* max. dilation */
    PPos_t          p_Dim;		/* dimension of the partition */
    GCPos_t         gc_Offset;		/* offset to the PPos's */
}               Grid3DData_t;

extern int
Make3DGrid (int reqId, int dim_x, int dim_y, int dim_z,
	  int xmin, int xmax, int ymin, int ymax, int zmin, int zmax);

extern int
Free3DGrid (int topId);

extern Grid3DData_t *
Get3DGrid_Data (int topId);

/*----------------------------------------------------------------------------*/
/* the '2d-torus' structure: */

#define	TORUS2D_TYPE	"torus2D"

#define	TORUS2D_NONE	0		/* The processor does not belong to the
					 * virtual topology 'd-torus */
#define	TORUS2D_IN	0x01		/* Bit set => than the processor is a
					 * node of the torus */
#define	TORUS2D_DOWNX	0x02		/* left neighbor of the node? */
#define	TORUS2D_UPX	0x04		/* right neighbor of the node? */
#define	TORUS2D_DOWNY	0x08		/* upper neighbor of the node? */
#define	TORUS2D_UPY	0x10		/* lower neighbor of the node? */

typedef struct {
    char            type[20];		/* topology name "torus2D" */
    Pos2D_t         id;			/* torus id */
    int             status;		/* status of the node in the torus */
    Pos2D_t         dim;		/* dimension of the torus */
/* internal variables */
    int             dilation;		/* max. dilation */
    PPos_t          p_Dim;		/* dimension of the partition */
    GCPos_t         gc_Offset;		/* offset to PPos */
}               Torus2DData_t;

extern int
Make2DTorus (int reqId, int dim_x, int dim_y,
	  int xmin, int xmax, int ymin, int ymax, int zmin, int zmax);

extern int
Free2DTorus (int topId);

extern Torus2DData_t *
Get2DTorus_Data (int topId);

/*----------------------------------------------------------------------------*/
/* the '3d-torus' structure: */

#define	TORUS3D_TYPE	"torus3D"

#define	TORUS3D_NONE	0		/* the processor does not belong to the
					 * virtual topology torus */
#define	TORUS3D_IN	0x01		/* Bit set => than the processor is a
					 * node of the torus */
#define	TORUS3D_DOWNX	0x02		/* left neighbor of the node? */
#define	TORUS3D_UPX	0x04		/* right neighbor of the node? */
#define	TORUS3D_DOWNY	0x08		/* upper neigbor of the node? */
#define	TORUS3D_UPY	0x10		/* lower neigbor of the node? */
#define	TORUS3D_DOWNZ	0x20		/* front neigbor of the node? */
#define	TORUS3D_UPZ	0x40		/* back neighbor of the node? */

typedef struct {
    char            type[20];		/* topology name "torus2D" */
    Pos3D_t         id;			/* torus id */
    int             status;		/* status of the node in the torus */
    Pos3D_t         dim;		/* dimension of the torus */
/* internal variables */
    int             dilation;		/* max. dilation */
    PPos_t          p_Dim;		/* dimension of the partition */
    GCPos_t         gc_Offset;		/* offset to PPos */
}               Torus3DData_t;

extern int
Make3DTorus (int reqId, int dim_x, int dim_y, int dim_z,
	  int xmin, int xmax, int ymin, int ymax, int zmin, int zmax);

extern int
Free3DTorus (int topId);

extern Torus3DData_t *
Get3DTorus_Data (int topId);

/*----------------------------------------------------------------------------*/
/* the 'hypercube' structure: */

#define	HCUBE_TYPE	"hypercube"

#define HCUBE_NONE	-1		/* the processor does not belong to the
                                           virtual topology Hypercube */

#define	MAXHCUBE	-1

typedef struct {
    char            type[20];		/* Topology name "hypercube" */
    int             id;			/* The ID of the proc. in the cube
                                           HCUBE_NONE, if the proc. does not belong
                                           to the cube */
    int	            dim;		/* Dimension of the hypercube */
    int             status;	        /* status & 2^i != 0 <=>
                                           there is a neighbour in dimension i */
/* internal variables */
    int             dilation;		/* max. dilation */
    PPos_t          p_Dim;		/* Partition-dimensions */
    GCPos_t         gc_Offset;		/* Offset to the PPos's */
}               HCubeData_t;

extern int
MakeHCube (int reqId, int dim,
	  int xmin, int xmax, int ymin, int ymax, int zmin, int zmax);

extern int
FreeHCube (int topId);

extern HCubeData_t *
GetHCube_Data (int topId);

/*----------------------------------------------------------------------------*/
/* the 'tree' structure: */

#define	TREE_TYPE	"tree"

#define	TREE_NONE	0		/* no node of the tree */

#define	MAXHEIGHT	-1
#define INFHEIGHT	-2

typedef	struct	{
    int		level;
    int		pos;
}		TPos_t;

typedef	struct	{
    int		degree;
    int		height;
}		TDim_t;

typedef	struct	{
    char	type[20];		/* topology name "tree" */
    TPos_t	id;                     /* node identification */
    TDim_t      dim;                    /* dimension of the topology */
/* internal variables */
    void	*status;                /* node status */
    int             dilation;		/* max. dilation */
    PPos_t          p_Dim;		/* dimensions of  partition */
    GCPos_t         gc_Offset;		/* offset to PPos's */
}		TreeData_t;


extern int
MakeRTree (int reqId, int degree, int depth,
	  int xmin, int xmax, int ymin, int ymax, int zmin, int zmax,
	  int xroot, int yroot, int zroot);

extern	int
MakeTree (int reqId, int degree, int depth,
	  int xmin, int xmax, int ymin, int ymax, int zmin, int zmax);

extern	int
FreeTree (int topId);

extern	TreeData_t *
GetTree_Data (int topId);

extern	TPos_t
Parent (int topId, TPos_t id);
extern	TPos_t
Son (int topId, TPos_t id, int i);
extern	int
Root (int topId, TPos_t t_id);
extern	int
Leave (int topId, TPos_t t_id);

extern int
HasSon( int whichSon, TreeData_t *treeData );

extern int
HasParent( TreeData_t *treeData );

/*----------------------------------------------------------------------------*/
/* the 'DeBruijn' structure: */

#define	DEB_TYPE	"DeBruijn"

#define	DEB_NONE	0		/* processor isn't a node of
					 * the topology */
#define	DEB_IN		1		/* processor isn't a node of
					 * the topology */

#define	MAXDEB		-1		/* dim == MAXDEB => topology
					 * with maximal dimension */

typedef	struct	{
    char	type[20];		/* topology name "tree" */
    int		id;			/* node identification */
    int		status;			/* node status */
    int		dim;			/* dimension of the topology */
/* internal variables */
    int             dilation;		/* max. dilation */
    PPos_t          p_Dim;		/* dimensions of  partition */
    GCPos_t         gc_Offset;		/* offset to PPos's */
}		DebData_t;


extern int
MakeDeb (int reqId, int dim,
	  int xmin, int xmax, int ymin, int ymax, int zmin, int zmax);

extern	DebData_t *
GetDeb_Data (int topId);

extern	int
FreeDeb (int topId);


#endif
