#define NOTEST
/* for debuging, define TEST or TEST1 */
/*
 * File:	mkdesc.c
 *
 * Author:	Christian Fleischer, Parsytec Computer GmbH
 *
 * Created:	30.10.1991
 *
 * Version:	@(#)mkdesc.c	1.5	2/8/93	
 *
 * Changes:	911219 CLA	error message line 142 (illegal ptr) fixed
 *
 * Description:
 *
 *	This program is used internally for PARIX 'run' command.
 *	It generates two files, "NAME1.EXT" for the NCM and
 *	"NAME2.EXT" as the PARIX descriptor.
 *	The files will be created in the current directory with
 *	mode 0444.
 *	If the optional third parameter, the link-name, is supplied,
 *	the "NAME?" will be concatenated by that value.
 *	The reason: several user can work without interference simultaneously
 *	on different links using this option.
 *
 * Still to do:
 *	The NCM descriptor For configurable machines is wrong
 *	iff the host is not connected to Processor 0.
 *
 * Changes:	920102 FLE	changed for GC configuration
 *
 * Description:
 *	Generates network descriptor for GC simulation on configurable
 *	machines (SuperCluster, MultiCluster) using the chessboard
 *	algorithm: (W = white node, B = black node)
 *
 *
 *   +---> +x
 *   |
 *   |	     |          |          |          |
 *   V	   --2--      --3--      --2--      --3--
 *   +	  |     |    |     |    |     |    |     |
 *   y	--0  W  1----1  B  0----0  W  1----1  B  0--
 *   	  |     |    |     |    |     |    |     |
 *   	   --3--      --2--      --3--      --2--
 *	     |          |          |          |
 *	     |          |          |          |
 *	   --3--      --2--      --3--      --2--
 *   	  |     |    |     |    |     |    |     |
 *   	--1  B  0----0  W  1----1  B  0----0  W  1--
 *   	  |     |    |     |    |     |    |     |
 *   	   --2--      --3--      --2--      --3--
 *   	     |          |          |          |
 *   	     |          |          |          |
 *   	   --2--      --3--      --2--      --3--
 *   	  |     |    |     |    |     |    |     |
 *   	--0  W  1----1  B  0----0  W  1----1  B  0--
 *   	  |     |    |     |    |     |    |     |
 *   	   --3--      --2--      --3--      --2--
 *	     |          |          |          |
 *	     |          |          |          |
 *	   --3--      --2--      --3--      --2--
 *   	  |     |    |     |    |     |    |     |
 *   	--1  B  0----0  W  1----1  B  0----0  W  1--
 *   	  |     |    |     |    |     |    |     |
 *   	   --2--      --3--      --2--      --3--
 *   	     |          |          |          |
 *
 *	Iff (MyX & 1) == (MyY & 1), the node is `white'.
 *
 * Changes:	920110 FLE	changed for GC test environment
 *
 * Description:
 *	Iff 'NCM' is defined, gcdesc will generate a ncm descriptor.
 *	The PARIX descriptor specifies the host to be connected to
 *	the network via Link 2 of processor 0.
 *
 * Changes:	920116 FLE	changed for GC test environment
 *
 * Description:
 *	takes now 4 arguments:
 *	Arg1 : X dimension
 *	Arg2 : Y dimension
 *	Arg3 : parix descriptor name ('-' for none)
 *	Arg4 : ncm descriptor name ('-' for none)
 *	Valid Options:
 *	-l <linkno>	hostlink for Parix and NCM descriptor (if not link 0)
 *	-p <procno>	root processor connection (if not processor 0)
 *	-g		gc network type
 * 	-H <count>	number of hostlinks
 * 	-L <count>	number of local links
 * 	-V <count>	number of virtual links
 *
 * Changes:	920116 FLE	added some comments
 *
 * Description:
 *	Reworked and commented for public availability
 *
 * Changes:	920526 SIG	multi host support
 *
 * Description:
 *	List of host connections can be given after descriptor
 *      names. Kept old options -lp for compatibility reasons.
 *
 * Changes:	920619 SIG	new GC linkmap
 *
 * Description: Option -G produces descriptor with new gc linkmap.
 *              Option -u for unicolored linkmap.
 *
 *   +---> +x
 *   |
 *   |	     |          |          |          |
 *   V	   --2--      --3--      --0--      --1--
 *   +	  |     |    |     |    |     |    |     |
 *   y	--0  W  1----1  B  2----2  R  3----3  G  0--
 *   	  |     |    |     |    |     |    |     |
 *   	   --3--      --0--      --1--      --2--
 *	     |          |          |          |
 *	     |          |          |          |
 *	   --3--      --0--      --1--      --2--
 *   	  |     |    |     |    |     |    |     |
 *   	--1  B  2----2  R  3----3  G  0----0  W  1--
 *   	  |     |    |     |    |     |    |     |
 *   	   --0--      --1--      --2--      --3--
 *   	     |          |          |          |
 *   	     |          |          |          |
 *   	   --0--      --1--      --2--      --3--
 *   	  |     |    |     |    |     |    |     |
 *   	--2  R  3----3  G  0----0  W  1----1  B  2--
 *   	  |     |    |     |    |     |    |     |
 *   	   --1--      --2--      --3--      --0--
 *	     |          |          |          |
 *	     |          |          |          |
 *	   --1--      --2--      --3--      --0--
 *   	  |     |    |     |    |     |    |     |
 *   	--3  G  0----0  W  1----1  B  2----2  R  3--
 *   	  |     |    |     |    |     |    |     |
 *   	   --2--      --3--      --0--      --1--
 *   	     |          |          |          |
 *
 * Changes:	920623 SIG	error messages
 *
 * Description: Improved some error messages and directed them to stderr
 *		instead of stdout. Included <stdlib.h>, removed declaration
 *		"extern char *getenv()".
 *
 * Changes:	930125 KOT	vlink defaults changed
 *                              add new remapping procedures designed by FLE
 *
 * Description:	Map a (smaller) Net, described by the user for a (bigger)
 *		Partition. 
 */

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>
#include <malloc.h>

/* Limits for the network */

#define	MAX		16384		/* maximum number of nodes	 */
#define	MIN		1		/* minimum number of nodes	 */
#define	MAX_LINES	4		/* max. no. of linkmap lines	 */


#define elif		else if

#define max(x,y)	((x)<(y)?(y):(x))
#define min(x,y)	((x)<(y)?(x):(y))

typedef struct {			/* link table element		 */
    int             proc;		/* target processor, -2 if connected
					 * to host */
    int             link;		/* link on target processor	 */
    char           *host;		/* if connected to host		 */
}               linkref_t[4];

typedef struct {			/* host table element		 */
    char           *name;		/* host name			 */
    int             proc;		/* target processor		 */
    int             link;		/* link on target processor	 */
}               hostref_t;

typedef struct {			/* network type definition	 */
    char           *name;		/* type name (from NRM)	 	 */
    int             dx;			/* x direction step		 */
    int             dy;			/* y direction step	  	 */
    int             maps;		/* no. of linkmap lines		 */
    int             lines[MAX_LINES][4];/* linkmap lines		 */
}               netdef_t;

int		routmem = -1;		/* mem-type to place code and    *
					 * data of the router 		 *
					 * 0 = code&data->intern	 *
					 * 1 = data->intern,code->extern *
					 * 2 = code&data->extern	 */
int             proc = -1;		/* root processor		 */
int             link = -1;		/* root processor link		 */
int             nhosts = 0;		/* number of hosts	 	 */
int             gctype = -1;		/* 0 = unicolored, 1 = oldgc,    *
					 * 2 = newgc 			 */
int             usr_dx = 0;		/* user network dimensions	 */
int             usr_dy = 0;
int             usr_dxy;

int             startx;			/* ueser's reference point	 */
int             starty;

int             net_dx = 0;		/* physical network dimensions	 */
int             net_dy = 0;
int             net_dxy;

int             hlinks = -1;		/* host links			 */
int             llinks = -1;		/* local links			 */
int             vlinks = -1;		/* virtual links		 */

linkref_t      *linkmap;		/* connection table		 */
hostref_t      *hosttab;		/* table of NRM-defined hosts	 */

netdef_t        netdefs[] = {		/* network typed from NRM	 */
    {"UNICOLOR", 1, 1, 1,
	{0, 1, 2, 3}
    },
    {"OLDGC", 1, 1, 2,
	{0, 2, 1, 3},
	{1, 3, 0, 2}
    },
    {"NEWGC", 1, 1, 4,
	{0, 2, 1, 3},
	{1, 3, 2, 0},
	{2, 0, 3, 1},
	{3, 1, 0, 2}
    }
};

#define	nNETDEFS	(sizeof (netdefs) / sizeof (netdef_t))

char           *pdname;			/* parix descriptor name 	 */
char           *ndname;			/* ncm descriptor name 		 */
extern char    *optarg;			/* for getopt()			 */
extern int      optind;

char           *myname;

Usage ()
{
    printf ("%s: Generate Parix and NCM descriptor\n", myname);
    printf ("for a grid of the specified dimension x * y\n");
    printf ("Usage: %s [-ugGmplHLV] <Usr_Dx> <Usr_Dy> <Parix desc. name> <NCM desc. name>\n", myname);
    printf ("       %*s [<Net_Dx> <Net_Dy> <LinkMap> <host1> <proc1> <plink1>\n", strlen (myname), "");
    printf ("       %*s  [ ... <hostN> <procN> <plinkN>]]\n", strlen (myname), "");
    printf ("   -u         generate descriptors for a unicolored network (default)\n");
    printf ("   -g         generate descriptors for an old GCel network\n");
    printf ("   -G         generate descriptors for new GCel network\n");
    printf ("   -m value   RAM to place code/data of the VC-router.\n");
    printf ("   -p procno  root processor connected to the host (if not 0)\n");
    printf ("   -l linkno  link number of root processor (if not 0)\n");
    printf ("   -H count   number of hostlinks\n");
    printf ("   -L count   number of local links\n");
    printf ("   -V count   number of virtual links\n");
    printf ("<Usr_Dx> and <Usr_Dy> specify the users network size.\n");
    printf ("<Net_Dx>, <Net_Dy>, <LinkMap> and the following parameters are taken from\n");
    printf ("the NRM result. Host <hosti> will be connected to\n");
    printf ("processor <proci>, link <plinki>, and the network must be booted\nvia <proc1>, link <plink1>.\n");
    printf ("If -l and -p are given, <proc1> and <plink1> are ignored.\n");
    printf ("If -u, -g or -G are given, <LinkMap> is ignored.\n");
    exit (1);
}

int
getlink (px, py, dir, netdef)
    int             px;
    int             py;
    int             dir;
    netdef_t       *netdef;
{
    int             l = (px * netdef->dx + py * netdef->dy) % netdef->maps;

    return netdef->lines[l][dir];
}

/*
 * initialise the map
 */

int
initmap (netdef, dx, dy)
    netdef_t       *netdef;
{
    int             n, l;
    int             px, py;

    /*
     *	clear map
     */
    for (n = 0; n < net_dx * net_dy; n++) {
	for (l = 0; l < 4; l++) {
	    linkmap[n][l].proc = -1;
	    linkmap[n][l].link = -1;
	    linkmap[n][l].host = NULL;
	}
    }
    /*
     *	setup net-type
     */
    for (py = 0; py < dy; py++) {
	for (px = 0; px < dx; px++) {
	    int             p = py * dx + px;

	    if (px > 0) {
		l = getlink (px, py, 0, netdef);
		linkmap[p][l].proc = (py * dx) + px - 1;
		linkmap[p][l].link = getlink (px - 1, py, 2, netdef);
	    }
	    if (px < dx - 1) {
		l = getlink (px, py, 2, netdef);
		linkmap[p][l].proc = (py * dx) + px + 1;
		linkmap[p][l].link = getlink (px + 1, py, 0, netdef);
	    }
	    if (py > 0) {
		l = getlink (px, py, 1, netdef);
		linkmap[p][l].proc = ((py - 1) * dx) + px;
		linkmap[p][l].link = getlink (px, py - 1, 3, netdef);
	    }
	    if (py < dy - 1) {
		l = getlink (px, py, 3, netdef);
		linkmap[p][l].proc = ((py + 1) * dx) + px;
		linkmap[p][l].link = getlink (px, py + 1, 1, netdef);
	    }
	}
    }
}

/*
 * add the host entries to the linkmap
 */

int
addhosts (dxy)
    int dxy;
{
    char           *host;
    int             proc, link;
    int             px, py;
    int             i;

    for (i = 0; i < nhosts; i++) {
	host = hosttab[i].name;
	proc = hosttab[i].proc;
	link = hosttab[i].link;

	if (proc < 0 || proc >= dxy) {
	    fprintf (stderr, "%s ERROR: Invalid processor (%d) for host %s specified !\nValid range is [0, %d[\n",
		     myname, proc, host, dxy);
	    exit (1);
	}
	if (link < 0 || link > 3) {
	    fprintf (stderr, "%s ERROR: Invalid processor link (%d) for host %s specified !\nValid range is [0, 3]\n",
		     myname, link, host);
	    exit (1);
	}
	switch (linkmap[proc][link].proc) {
	case -1:
	    break;

	case -2:
	    fprintf (stderr, "%s ERROR: Link %d at processor %d connected to two hosts (%s %s).\n",
		     myname, link, proc, linkmap[proc][link].host, host);
	    exit (1);

	default:
	    fprintf (stderr, "%s ERROR: Link %d at processor %d not available for host %s !\nThis link is connected to processor %d, link %d\n",
		     myname, link, proc, host,
		     linkmap[proc][link].proc, linkmap[proc][link].link);
	    exit (1);
	}

	linkmap[proc][link].proc = -2;
	linkmap[proc][link].link = -2;
	linkmap[proc][link].host = host;
    }
}

/*
 * print the processor map for debugging purposes
 */

int
printmap (nmax)
    int             nmax;
{
    int             n, l;

    printf ("Proc.    Link0     Link1     Link2     Link3\n");
    for (n = 0; n < nmax; n++) {
	printf ("%4d:  ", n);
	for (l = 0; l < 4; l++) {
	    switch (linkmap[n][l].proc) {
	    case -2:
		printf ("%8s  ", linkmap[n][l].host);
		break;
	    case -1:
		printf ("  -----   ");
		break;
	    default:
		printf ("%4d:%2d   ", linkmap[n][l].proc, linkmap[n][l].link);
		break;
	    }
	}
	printf ("\n");
/*

	printf ("%2d: %3d:%2d  %3d:%2d  %3d:%2d  %2d:%3d\n", n,
		linkmap[n][1].proc, linkmap[n][1].link,
		linkmap[n][2].proc, linkmap[n][2].link,
		linkmap[n][3].proc, linkmap[n][3].link);

	if ((linkmap[n][0].proc == -2)
	    || (linkmap[n][1].proc == -2)
	    || (linkmap[n][2].proc == -2)
	    || (linkmap[n][3].proc == -2))
	    for (l = 0; l < 4; l++)
		if (linkmap[n][l].proc == -2)
		    printf ("%2d: host %s:%d connected to link %d\n",
			    n, linkmap[n][l].host, linkmap[n][l].link, l);
*/
    }
}

int
main (argc, argv)
    int             argc;
    char          **argv;
{
    int             ch;
    int             i;

    myname = argv[0];
/*
 * scan options
 */
    while ((ch = getopt (argc, argv, "ugGm:l:p:H:L:V:")) != -1) {
	switch (ch) {
	case 'u':
	    gctype = 0;
	    break;
	case 'g':
	    gctype = 1;
	    break;
	case 'G':
	    gctype = 2;
	    break;
        case 'm':
	    routmem = (int) strtol (optarg, (char **) NULL, 0);
	    break;
	case 'l':
	    link = (int) strtol (optarg, (char **) NULL, 0);
	    break;
	case 'p':
	    proc = (int) strtol (optarg, (char **) NULL, 0);
	    break;
	case 'H':
	    hlinks = (int) strtol (optarg, (char **) NULL, 0);
	    break;
	case 'L':
	    llinks = (int) strtol (optarg, (char **) NULL, 0);
	    break;
	case 'V':
	    vlinks = (int) strtol (optarg, (char **) NULL, 0);
	    break;
	case '?':
	    Usage (argv[0]);
	}
    }

/*
 * check number of remaining arguments
 */

    if (!((argc - optind == 4) || ((argc - optind - 10) % 3 == 0)))
	Usage ();

/*
 * extract parameters
 */
    usr_dx = atoi (argv[optind++]);
    usr_dy = atoi (argv[optind++]);
    pdname = argv[optind++];
    ndname = argv[optind++];

/*
 * check for NRM style parameters
 */
    if (argc - optind > 0) {
	net_dx = atoi (argv[optind++]);
	net_dy = atoi (argv[optind++]);

    /*
     * take network type if not specified before
     */
	for (i = 0; gctype < 0 && i < nNETDEFS; i++)
	    if (strcmp (argv[optind], netdefs[i].name) == 0)
		gctype = i;
	optind++;
    }
    nhosts = (argc - optind) / 3;
    hosttab = (hostref_t *) calloc (nhosts + 1, sizeof (hostref_t));

    for (i = 0; argc - optind > 0; i++, optind += 3) {
	hosttab[i].name = argv[optind];
	hosttab[i].proc = atoi (argv[optind + 1]);
	hosttab[i].link = atoi (argv[optind + 2]);
    }

/*
 * take proc and link if not specified via -pl
 */
    if (nhosts > 0) {
	if (proc == -1)
	    proc = hosttab[0].proc;
	if (link == -1)
	    link = hosttab[0].link;
    }
/*
 * set default values for nettype, proc and link if not otherwise specified
 */
    if (gctype == -1)
	gctype = 0;
    if (proc == -1)
	proc = 0;
    if (link == -1)
	link = 0;

/*
 * if no hosts were specified in NRM style, construct a default entry.
 */
    if (nhosts < 1) {
	hosttab[0].name = "unknownHost";
	hosttab[0].proc = proc;
	hosttab[0].link = link;
	nhosts = 1;
    } else {
	if (hosttab[0].proc != proc)
	    fprintf (stderr, "%s Warning: Inconsistency found between -p option and <host1> !\n",
		     myname);
	if (hosttab[0].link != link)
	    fprintf (stderr, "%s Warning: Inconsistency found between -l option and <plink1> !\n",
		     myname);
    }

    if (net_dx == 0)
	net_dx = usr_dx;
    if (net_dy == 0)
	net_dy = usr_dy;

/*
 * clear unused filenames
 */
    if (strcmp (pdname, "-") == 0)
	pdname = NULL;
    if (strcmp (ndname, "-") == 0)
	ndname = NULL;

/*
 * see if there is something to do
 */
    if (pdname == NULL && ndname == NULL)
	exit (0);
    if (routmem > 2) {
	fprintf (stderr, "%s ERROR: %d is a wrong value for router-RAM (available: [0..2]) !",
		 myname, routmem);
	exit (1);
    }
    usr_dxy = usr_dx * usr_dy;
    net_dxy = net_dx * net_dy;
    if (net_dxy < 1) {
	fprintf (stderr, "%s ERROR: Network of size 0 requested (%d x %d) !",
		 myname, usr_dx, usr_dy);
	exit (1);
    }
    if (net_dx < usr_dx) {
	fprintf (stderr, "%s ERROR: Requested X dimension (%d) is larger than Network dimension (%d) !",
		 myname, usr_dx, net_dx);
	exit (1);
    }
    if (net_dy < usr_dy) {
	fprintf (stderr, "%s ERROR: Requested Y dimension (%d) is larger than Network dimension (%d) !",
		 myname, usr_dy, net_dy);
	exit (1);
    }
/*
 * allocate and initialise the linkmap
 */
    linkmap = (linkref_t (*)) malloc (sizeof (linkref_t) * net_dxy);
    initmap (&netdefs[gctype], net_dx, net_dy);
    addhosts (net_dxy);

#ifdef	TEST1
    printmap (net_dxy);
#endif

/*
 * at this point, the linkmap is ok, but the position of the
 * user's part might be wrong. (Imagine a 4 * 4 partition with
 * a single processor used where the boot entry is not at processor 0)
 */

    {

/*
 * Algorithm: (minsx/minsy) and (maxsx/maxsy) describe the rectangle
 * in which the partition reference point (lower left corner) might reside.
 *
 * For each host, (minx/miny) and (maxx/maxy) describe the rectangle
 * in which the reference point must reside to get a connection.
 *
 * If the connection is possible, the limiting rectangle is clipped to
 * guarantee that the host will remain connected.
 */
	int             minsx = 0;
	int             maxsx = net_dx - usr_dx;
	int             minsy = 0;
	int             maxsy = net_dy - usr_dy;

	int             h;

#ifdef	TEST1
	printf ("\nnet_dx %2d net_dy %2d usr_dx %2d usr_dy %2d\n",
		net_dx, net_dy, usr_dx, usr_dy);
#endif

	for (h = 0; h < nhosts; h++) {
	    int             nethp = hosttab[h].proc;
	    int             maxhx = nethp % net_dx;
	    int             minhx = maxhx - usr_dx + 1;
	    int             maxhy = nethp / net_dx;
	    int             minhy = maxhy - usr_dy + 1;

#ifdef	TEST1
	    printf ("\nhost %s: %d/%d (%d)\n",
		    hosttab[h].name, maxhx, maxhy, nethp);
	    printf ("\t minx  %2d miny  %2d maxx  %2d maxy  %2d\n",
		    minhx, minhy, maxhx, maxhy);
	    printf ("\t minsx %2d minsy %2d maxsx %2d maxsy %2d\n",
		    minsx, minsy, maxsx, maxsy);
#endif

	    if (minhx > maxsx || maxhx < minsx
		|| minhy > maxsy || maxhy < minsy) {

#ifdef	TEST1
		printf ("Sorry, no connection\n");
#endif

		hosttab[h].link = -1;
	    } else {

		if (minhx > minsx && minsx < maxsx)
		    minsx = min (minhx, maxsx);
		if (minhy > minsy && minsy < maxsy)
		    minsy = min (minhy, maxsy);

		if (maxhx < maxsx && minsx < maxsx)
		    maxsx = max (maxhx, minsx);
		if (maxhy < maxsy && minsy < maxsy)
		    maxsy = max (maxhy, minsy);

#ifdef	TEST1
		printf ("\t minsx %2d minsy %2d maxsx %2d maxsy %2d\n",
			minsx, minsy, maxsx, maxsy);
#endif
	    }
	}
	startx = minsx;
	starty = minsy;
    }
/*
 * generate parix descriptor
 */
    if (pdname != NULL) {
	int             i, host_px, host_py;
	netdef_t       *nd = &netdefs[gctype];
	int             fd;
	FILE           *fp;

#ifndef	TEST
	if ((fd = open (pdname, O_WRONLY | O_CREAT | O_EXCL, 0444)) < 0) {
	    fprintf (stderr, "%s ERROR: failed to open PARIX descriptor,", myname);
	    perror ("cause");
	    exit (3);
	}
#else
	fd = dup (1);
#endif

	fp = fdopen (fd, "w");

    /*
     * check parameters for parix descriptor
     */
	if (hlinks < usr_dxy * 4) {
	    if (hlinks >= 0)
		fprintf (stderr, "%s Warning: %d host links specified, default is %d !\n",
			 myname, hlinks, usr_dxy * 4);
	    else
		hlinks = usr_dxy * 4;
	}
	if (llinks < 20) {
	    if (llinks >= 0)
		fprintf (stderr, "%s Warning: %d local links specified, default is %d !\n",
			 myname, llinks, 20);
	    else
		llinks = 20;
	}
	/*
	 *	calculate minimal VLink number (3 * 4 + 1 + nCube-connections)
	 */
	{
	    int vlmin,vldef,k;

	    vlmin = 1;
	    k = 2;
	    while (k < usr_dxy) {
		k *= 2;
		vlmin++;
	    }
	    vlmin += 13;
	    /***  vldef = 2 * ((int) log2 ((double) usr_dxy)) + 20;  default by chris   ***/
	    vldef = vlmin + 2 * (usr_dxy / ((usr_dx + usr_dy) / 2)) + 10; 

	    if (vlinks < vldef) {
	        if (vlinks >= vlmin) {
		    /*
		     * fprintf (stderr, "%s Warning: %d virtual links specified, default is %d !\n",
		     * myname, vlinks, vldef);
		     */
	        } else {
		    if (vlinks >= 0) {
			fprintf (stderr, "%s Warning: %d virtual links specified, minimum is %d !\n",
				 myname, vlinks, vlmin);
			fprintf (stderr, "%s          Virtulal link number is set to %d (internal minimum) !\n",
				 myname, vlmin);
			vlinks = vlmin;
		    } else {
			/* no vlink number specified */
			vlinks = vldef;
		    }
	        }
	    }
	}
	fprintf (fp, "HLINKS %d\n", hlinks);
	host_px = (hosttab[0].proc % net_dx) - startx;
	host_py = (hosttab[0].proc / net_dx) - starty;
	fprintf (fp, "ROOTPOS %d %d 0\n", host_px, host_py);
	fprintf (fp, "DIM %d %d 1\n", usr_dx, usr_dy);

	fprintf (fp, "DX %d\nDY %d\n", nd->dx, nd->dy);

	for (i = 0; i < nd->maps; i++)
	    fprintf (fp, "LINKMAP %d %d %d %d\n",
	    nd->lines[(i + startx * nd->dx + starty * nd->dy) % nd->maps][0],
	    nd->lines[(i + startx * nd->dx + starty * nd->dy) % nd->maps][1],
	    nd->lines[(i + startx * nd->dx + starty * nd->dy) % nd->maps][2],
		     nd->lines[(i + startx * nd->dx + starty * nd->dy) % nd->maps][3]);

/*xxxxx*/

	fprintf (fp, "VLINKS %d\n", vlinks);
	fprintf (fp, "LLINKS %d\n", llinks);
	fprintf (fp, "RLINKS 20\n");
	fprintf (fp, "SCBS 20\n");
	if (routmem >= 0)
	    fprintf (fp, "IMEMUSAGE %d\n", routmem);

	for (i = 0; i < nhosts; i++) {
	    if (hosttab[i].link >= 0) {
		fprintf (fp, "HOST %s\n", hosttab[i].name);
		host_px = (hosttab[i].proc % net_dx) - startx;
		host_py = (hosttab[i].proc / net_dx) - starty;
		if (host_px < 0 || host_px >= usr_dx ||
		    host_py < 0 || host_py >= usr_dy)
		    fprintf (stderr, "%s INTERNAL ERROR on host %s at %d\n",
			     myname, hosttab[i].name, hosttab[i].proc);

		fprintf (fp, "  POS %d %d 0\n", host_px, host_py);
		fprintf (fp, "  LINK %d\n", hosttab[i].link);
		fprintf (fp, "  ID %u\n", 0x8000 + i);
		fprintf (fp, "  CSIZE 0\n");
		fprintf (fp, "  DSIZE 0\n");
	    } else
		fprintf (stderr, "%s Warning: Host %s not connected to the network !\n",
			 myname, hosttab[i].name);
	}

#ifndef	TEST
	fclose (fp);
#endif
    }

/*
 * generate ncm  descriptor
 */
    if (ndname != NULL) {
	int             hp, p, l, tp, tl;
	int             fd;
	FILE           *fp;

	/*
	 *	setup map with new dimensions
	 */

	initmap(&netdefs[gctype], usr_dx, usr_dy);
	addhosts (usr_dxy);
	hp = hosttab[0].proc;

#ifndef	TEST
	if ((fd = open (ndname, O_WRONLY | O_CREAT | O_EXCL, 0444)) < 0) {
	    fprintf (stderr, "%s: Failed to open NCM descriptor, ", myname);
	    perror ("cause");
	    exit (3);
	}
#else
	fd = dup (1);
#endif

	fp = fdopen (fd, "w");
    
        fprintf (fp, "PROC %d %d type: 7 ( Processor )\n", hp, hp);
	for (p = 0; p < usr_dxy; p++) 
	    if (p != hp)
		fprintf (fp, "PROC %d %d type: 7 ( Processor )\n", p, p);

	for (p = 0; p < usr_dxy; p++) {
	    for (l = 0; l < 4; l++) {
		tp = linkmap[p][l].proc;
		tl = linkmap[p][l].link;
		if (tp < 0) {
		    if (tp == -1)	/* not a host link		 */
			fprintf (fp, "LINK %d %d, %d %d\n",
				 p, l, p, l);
		} else {
		    fprintf (fp, "LINK %d %d, %d %d\n",
			     p, l, tp, tl);
		    linkmap[p][l].proc = -3;
		    linkmap[p][l].link = -3;
		    linkmap[tp][tl].proc = -3;
		    linkmap[tp][tl].link = -3;
		}
	    }
	}
#ifndef TEST
	fclose (fp);
#endif
    }
    return 0;
}
