#! /bin/sh
#Parsytec Computer GmbH, VHO, 20.1.92
#This is the shellscript for "getpartition".
#Parameters:see usage()
#SCCSid:	@(#)getpartition.sh	1.5	5/18/92
#
#------------------------------usage---------------------------
#Procedure prints usage-information. Arguments: 
#	$1: name of this program
usageGetPartition(){
	echo "usage: $dGetPartition [-a Attrib] [-c Command] [-h] [--] [Arguments]"
	echo " -a Attrib is the name or an attribute of the desired partition."
	echo "    If you ommit Attrib or many suitable partitions are avail-"
	echo "    able, a suitable is chosen for you."
	echo " -c Command is the command executed on the chosen partition."
	echo "    Command is a symbol for a program chosen by the administrator."
	echo "    if you omit Command, the Command is derived from the name"
	echo "    which you used to call this program ($1 in this case)."
	echo " -h Print this message and terminate."
	echo " -- Used to indicate the end of options. May be necessary if you"
	echo "    want to pass options as arguments to the called command."
	echo " Arguments: The Arguments are passed to the chosen Command."
	if test $dGetPartition = $MyName
	then
		echo " Some predefined actions are taken if certain names for"
		echo " this file are chosen. They also have usage messages."
		echo "  $dReleasePartition: release partition"
		echo "  $dPrintAlloc: print allocation table"
		echo "  $dPrintConfig: print configuration table"
		echo "  $dEdConfig: edit configuration file"
		echo "  $dReturnLock: release lock"
	fi
	exit 1
}	#usageGetPartition()
#
#-------------------------usageReleasePartition---------------------------
#No parameters
usageReleasePartition(){
	echo "usage: $dReleasePartition PartitionName"
	echo " Release partition with name PartitionName."
	echo " You must be either the owner of the partition"
	echo " or the administrator."
	exit 1
}	#usageReleasePartition()
#
#-------------------------usagePrintAlloc---------------------------
#No parameters
usagePrintAlloc(){
	echo "usage: $dPrintAlloc"
	echo " Prints a summary of the allocation of partitions."
	exit 1
}	#usagePrintAlloc()
#
#-------------------------usagePrintConfig---------------------------
#No parameters
usagePrintConfig(){
	echo "usage: $dPrintConfig"
	echo " Prints a summary of the systems configuration."
	exit 1
}	#usagePrintConfig()
#
#-------------------------usageEdConfig---------------------------
#No parameters
usageEdConfig(){
	echo "usage: $dEdConfig"
	echo " Allows you to edit the configuration file. It performs"
	echo " the necessary locking and checks if you are the admini-"
	echo " strator. Afterwards, a plausability-check of the new"
	echo " configuration file is performed."
	echo " The editor is given by the environment variable"
	echo " VISUAL or EDITOR. Default is $dEdit."
	exit 1
}	#usageEdConfig()
#
#-------------------------usageReturnLock---------------------------
#No parameters
usageReturnLock(){
	echo "usage: $dReturnLock"
	echo " In case a lock you have is not release, you can use"
	echo " this command. You must be the holder of the lock"
	echo " or be the administrator."
	exit 1
}	#usageReturnLock()
#
#-------------------PrintDebug--------------------------------
#print some useful information. No parameters.
PrintDebug(){
	echo LinkAdmLib=$LINKADMLIB
	echo ConfigFile=$ConfigFile
	echo LockFile=$LockFile
	echo StatFile=$StateFile
	echo Attribute=$Attribute
	echo Command=$Command
	echo Number of args=$#
	echo args=$*
	echo Traps:; trap
}	#PrintDebug()
#
#-------------------PrintErr----------------------------------
#Print errormessage $1, execute $ExitProc, exit with status $2
#
PrintErr(){
	echo "Error $0:$1"
	$ExitProc
	exit $2
}	#PrintErr
#
#-------------------GetLock-----------------------------------
#Try to get a lock on file $1. Try $2 times with pauses
#of $3 seconds. After that print contents of $1 end exit.
#If successful, write identifying data to that file
GetLock(){
	i=$2
	while test $i -gt 0
	do
		if $LINKADMLIB/testset $1
		then
			SetExit "RelLock $1"
			i=0
		else
			i=`expr $i - 1`
			if test $i -eq 0
			then
				PrintErr "Lock held by `cat $1`" 3
			fi	#if last try
			sleep $3
		fi
	done
	echo "`whoami`&`date`&`who am i`" >> $1
}	#GetLock
#
#----------------------RelLock------------------------
#Release lock by deleting lockfile $1
RelLock(){
	if test ! -f $1
	then
		echo Warning $0: Lockfile does not exist !
	fi
	SetExit EmptyProc
	rm -f $1
}	#RelLock
#
#---------------------AllocPart--------------------------------
#Allocate Partition with name $1 in file $2
#
AllocPart(){
	echo "USER&$1&`whoami`&`date`&`who am i`" >> $2
}	#AllocPart
#
#----------------------RelPart-----------------------------
#Release Partition with name $1 in file $2 with temporary file $3
#
RelPart(){
	grep -v "^USER&$1&" $2>$3
	cp $3 $2
	rm -f $3
}
#
#
#------------------ActTrapList---------------------------------
#Modify List of signals caught. Argument: caught signal
ActTrapList(){
	NewIgnSignals=$IgnSignals
	NewActSignals=""
	for Signal in $ActSignals
	do
		if test $Signal = $1
		then
			NewIgnSignals="$NewIgnSignals $Signal"
		else
			NewActSignals="$NewActSignals $Signal"
		fi
	done
	ActSignals=$NewActSignals
	IgnSignals=$NewIgnSignals
}	#ActTrapList
#
#
#--------------------EmptyProc--------------------------------
#Default ExitProc
EmptyProc(){
	echo -n
}	#EmptyProc
#
#-------------------SetExit-----------------------------------
#Prepare for exit by trap or PrintErr. $1 is executed before
#leaving shellscript.
#Set trap on standard signals. In case of signal execute
#command $1 before terminating. Signals already caught are ignored
#in the future
SetExit(){
	for Signal in $ActSignals
	do
		trap "trap \"\" $Signal;ActTrapList $Signal;echo Trap $Signal received. Please wait a moment.;$1;echo $0: terminated by trap;exit 1" $Signal
	done
	trap "" $IgnSignals
	ExitProc="$1"
}	#SetExit
#
#-------------------PrintConfig-------------------------
#Print a configuration summary. No arguments.
PrintConfig(){
	GetLock $LockFile $MaxRetries $RetryTime
	nawk -f $LINKADMLIB/evalconfig.nawk $ConfigFile $StateFile $dAttribute t
	RelLock $LockFile
}	#PrintConfig
#
#
#-------------------PrintAlloc-------------------------
#Print partition allocation summary. No arguments.
PrintAlloc(){
	GetLock $LockFile $MaxRetries $RetryTime
	nawk -f $LINKADMLIB/evalconfig.nawk $ConfigFile $StateFile $dAttribute a
	RelLock $LockFile
}	#PrintAlloc
#
#-------------------EdConfig---------------------------
#Edit the configuration file. Testing if administrator, performing locking
#editing, plausability checks
EdConfig(){
	#GetLock
	GetLock $LockFile $MaxRetries $RetryTime
	#check id
	AdmName=`grep "^ADM:" $ConfigFile|cut -d: -f2`
	if test $AdmName != `whoami`
	then
		PrintErr "Permission for editing denied" 8
	fi	#if AdmName
	#work on temporary copy
	cp $ConfigFile $ConfigFileTmp
	Result=1	#do it at least once
	while test $Result -ne 0
	do	#until editing successful
		$Edit $ConfigFileTmp
		#PlausTest
		Message="`nawk -f $LINKADMLIB/evalconfig.nawk $ConfigFileTmp $StateFile $dAttribute c`"
		Result=$?
		if test $Result -ne 0
		then
			echo "Plausability check terminated with error:"
			echo $Message
			echo -n "(E)dit, (R)evert, (L)eave unchanged (E)? "
			read Result
			case $Result in
				[Rr])	cp $ConfigFile $ConfigFileTmp
					Result=1;;
				[Ll])	rm -f $ConfigFileTmp
					Result=0;;
				*)	Result=1;;
			esac
		else
			#The new Configuration File !
			cp $ConfigFileTmp $ConfigFile
			rm -f $ConfigFileTmp
		fi
	done
	RelLock $LockFile
}	#EdConfig
#
#-------------------ReleasePartition-------------------------
#ReleasePartition releases a partition with full locking and
#protection mechanisms. No parameters, all values are global.
ReleasePartition(){
	#get lock first
	GetLock $LockFile $MaxRetries $RetryTime
	# Test if partition allocated
	PartEntry=`grep "^USER&$Partition&" $StateFile`
	if test $? -ne 0
	then
		PrintErr "Partition not allocated" 6
	fi
	#check if user has the right to remove partition
	UserName=`echo "$PartEntry"|cut -d\& -f3`
	if test $UserName != `whoami`
		then
			AdmName=`grep "^ADM:" $ConfigFile|cut -d: -f2`
			if test $AdmName != `whoami`
			then
				PrintErr "Permission for release denied" 7
			fi	#if AdmName
	fi	#if UserName
	#now remove allocation
	RelPart $Partition $StateFile $StateFileTmp
	#and release lock
	RelLock $LockFile
}	#ReleasePartition
#
#---------------------------Return lock-----------------------
#Return lock and check the users id.
ReturnLock(){
	if test -f $LockFile
	then
		UserName=`cat $LockFile|cut -d\& -f1`
		if test "$UserName" != `whoami`
		then
			AdmName=`grep "^ADM:" $ConfigFile|cut -d: -f2`
echo $AdmName
			if test $AdmName != `whoami`
			then
				PrintErr "Permission for lock release denied" 10
			fi	#if AdmName
		fi	#UserName
		RelLock $LockFile
	else
		echo "Warning $0: No lock exists"
	fi	#Lock exists
}	#ReturnLock
#
#---------------------GetPartition----------------------------
#GetPartition allocates a Partition, starts a command and deallocates
#the Partition afterwards. Parameter: Commandline Parameters
#Result: Result of executed Command
#
GetPartition(){
	#
	#Gain access to administrativ files
	#
	GetLock $LockFile $MaxRetries $RetryTime
	#
	#Find suitable partition
	#
	Partition=`nawk -f $LINKADMLIB/evalconfig.nawk $ConfigFile $StateFile $dAttribute f $Attribute $Command`
	if test $? -ne 0
	then
		PrintErr "$Partition" 4
	fi
	if test -z "$Partition"
	then
		PrintErr "Sorry, no suitable partition available" 5
	fi
	Partition=`echo "$Partition"|cut -d: -f1`	#simply use the first
	#
	#Find Command to be executed
	#
	if grep -s "^PARCOMMAND:$Partition:$Command:" $ConfigFile
	then
		ExCommand=`grep "^PARCOMMAND:$Partition:$Command:" $ConfigFile|cut -d: -f4`
	else
		PrintErr "Sorry, command $Command is not available in partition $Partition" 2
	fi
	#
	#Allocate partition
	#
	AllocPart $Partition $StateFile
	#
	#Release lock
	#
	RelLock $LockFile
	#
	#Make sure partition is being released on signal
	#
	SetExit "ReleasePartition $Partition"
	#trap "ReleasePartition $Partition" 0
	#
	#execute command
	#note: adm should use setgid to execute a program not to
	#run with egid of this script!
	#
	eval $ExCommand
	#store Result for future use
	Result=$?
	#
	#if desired print debugging stuff
	#
	if test $Debug -eq 1
	then
		PrintDebug
	fi
	#
	#release partition
	#
	ReleasePartition $Partition
	return $Result
}	#GetPartition
#
#-------------------"main" procedure--------------------------
#
#Variables:
#  "constants" 
#	dSignals: Signals caught by default
#	dAttribute: default attribute
#	dLockFile: name of lockfile relative to LINKADM
#	dConfigFile: name of configfile relative to LINKADM
#	dStateFile: name of statusfile relative to LINKADM
#	dTmp: suffix for temporary files
#	RetryTime: Time between retrys for lock
#	MaxRetries: Maximum number of retries before error
#	dGetPartition: name of program GetPartition
#	dReleasePartition: name of program ReleasePartition
#	dPrintAlloc: name of program PrintAlloc
#	dPrintConfig: name of program PrintConfig
#	dEdit: Default editor
#	dEdConfig: name of program EdConfig
#	dReturnLock: name of program RelLock
#  "real" variables
#	LINKADM: actual LinkAdm-path
#	LINKADMLIB: actual LinkAdmLib-path
#	LockFile: name of LockFile
#	ConfigFile: name of ConfigFile
#	ConfigFileTmp: name of temporary copy
#	StateFile: name of StateFile
#	StateFileTmp: temporary StateFile
#	Command: command to execute
#	Attribute: attribute
#	Debug: 1 if printing debugging-information
#	c: for option-processing
#	i: counter
#	Partition: Name of selected Partition
#	ExCommand: Command actually executed
#	ExitProc: Called by PrintErr before leaving
#	AdmName: Name of administrator
#	UserName: Name of user
#	PartEntry: Entry in $StateFile
#	Edit: Name of editor
#	Result: ResultCode of program
#	MyName: basename of this program
#	ActSignals: signals with action
#	IgnSignals: signals ignored
#	temp: temporary variable
#	NewActSignals, NewIgnSignals: temporary
#First some measures to make the script more foolproof...
#
PATH=/bin:/usr/bin:/usr/ucb:/usr/local/bin
IFS=" 	\
"	#Blank, TAB, NewLine
set -f	#no filename extension
#
#set default constants
#
dSignals="1 2 3 13 15"
dAttribute=default
dLockFile=lock
dConfigFile=entry.config.file
dStateFile=state
dTmp=.tmp
RetryTime=1
MaxRetries=10
dGetPartition=getpartition.sh
dReleasePartition=rp
dPrintAlloc=pa
dPrintConfig=pc
dEdit=vi
dEdConfig=ec
dReturnLock=rl
readonly dAttribute dLockFile dConfigFile dStateFile
readonly RetryTime MaxRetries dTmp
readonly dGetPartition dReleasePartition dPrintAlloc dPrintConfig
readonly dEdit dEdConfig dReturnLock dSignals
#
#Set default paths
#
if test -h $0
then	#symbolic link
	LINKADMLIB=`ls -l $0 | nawk -F "->" '{print $2}' -`
else
	LINKADMLIB=$0
fi
LINKADMLIB=`dirname $LINKADMLIB`
if test 0 -ne `expr $LINKADMLIB : '/[.]*'`
then	#absolute pathname given, fine !
	LINKADMLIB=$LINKADMLIB
else	#create complete pathname relative to cwd
	LINKADMLIB=`dirname $0`/$LINKADMLIB
fi
LINKADM=$LINKADMLIB/..
LockFile=$LINKADM/$dLockFile
ConfigFile=$LINKADM/$dConfigFile
ConfigFileTmp=$ConfigFile$dTmp
StateFile=$LINKADM/$dStateFile
StateFileTmp=$StateFile$dTmp
Edit=${VISUAL:=$dEdit}
Edit=${EDITOR:=$Edit}
MyName=`basename $0`
ActSignals=$dSignals
IgnSignals=""
#
# default signal handling
#
SetExit EmptyProc
#
#check and evaluate parameters
#
if test $MyName = $dReleasePartition
then	#Release Partition command
	if test $# -ne 1
	then
		usageReleasePartition
	else
		Partition=$1
		ReleasePartition
	fi
elif test $MyName = $dPrintAlloc
then
	if test $# -ne 0
	then
		usagePrintAlloc
	else
		PrintAlloc
	fi
elif test $MyName = $dPrintConfig
then
	if test $# -ne 0
	then
		usagePrintConfig
	else
		PrintConfig
	fi
elif test $MyName = $dEdConfig
then
	if test $# -ne 0
	then
		usageEdConfig
	else
		EdConfig
	fi
elif test $MyName = $dReturnLock
then
	if test $# -ne 0
	then
		usageReturnLock
	else
		ReturnLock
	fi
else
	#main command of this script
	while getopts "a:c:h" c
	do
		case $c in
			a)	Attribute=$OPTARG;;
			c)	Command=$OPTARG;;
			h)	usageGetPartition $MyName;;
			\?)	usageGetPartition $MyName;;
		esac
	done
	shift `expr $OPTIND - 1`	#get rid of evaluated parameters
	#
	#set default parameters if yet undefined
	#
	Attribute=${Attribute:=$dAttribute}
	Command=${Command:=$MyName}
	Debug=${Debug:=0}
	#
	GetPartition $*
fi	#which command
