#include <sys/utsname.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <pwd.h>
#include <grp.h>
#include <errno.h>

#define	Fname		"px_config"		/* the file name */

#define	LINK_O		"/link.o"		/* obj. files to link */
#define	LCONF_O		"/lconf.o"		/* with the dserver */
#define	TRANS_CONF	"/trans_conf"		/* BTSS configuration file */
#define	CONFIG		"./px_config.dat"	/* Configuration file of */
						/* the installation */
#define BASIC_MODULE	01
#define SYSTEM_MODULE	02
#define C_MODULE	04
#define FORTRAN_MODULE	010
#define PATOP_MODULE	020
#define PARMACS_MODULE	040

#define SYSTEM			"PX8-Sys-1.2"
#define BASIC			"PX8-BP-1.2-FES"
#define C			"PX8-EXPERT-C-1.2"
#define FORTRAN			"PX8-EXPERT-F77-1.2"
#define PATOP_FULL		"PX8-PATOP-1.0"
#define PATOP_RESTR		"PX8-PATOP-dev-1.0"
#define PARMACS_FULL		"PX8-PARMACS-5.1.2"



static char SCCSid[] = " @(#)px_config.c	1.11 4/20/93 ";


/***************** Static Function: void Quit()*******************************************
** Exits the program with a message.							*/

static void
Quit()
{
	printf("\n\n		*** BYE! ***\n\n\n");
	exit(1);
}

/***************** Static Function: int getanswer( char * )*******************************
** Gets a character from the standard input. This character must be 'y' or 'Y' or	**
**					  			    'n' or 'N' or	**
**					  			    'q' or 'Q'.		**
** Return value: integer.								**
** The function returns 1 by 'y' and 'Y';						**
**			0 by 'n' and 'N'.						**
** The function exits	by 'q' and 'Q'.							*/

static int
getanswer(string)
char	*string;
{
	char	c;

	do{
		while((c=getchar()) == '\n') printf("%s",string);
		while(getchar() != '\n') c='\0';
		if ( c == 'y' || c == 'Y' ) return 1;
		else if (c == 'n' || c == 'N') return 0;
		else if (c == 'q' || c == 'Q') Quit();
		else printf("%s",string);
	}while(1);
	return 0;
}

/***************** Static Function: int getanswer3( char, char, char )********************
** Gets a character from the standard input. This character must be one of the given	**
** character or 'q' or 'Q'.								**
**					  			    			**
** Return value: integer (1,2 or 3).							**
** The function returns 1 by char 1;							**
**			2 by char 2;							**
**			3 by char 3;						    	**
** The function exits	by 'q' and 'Q'.							*/

static int
getanswer3(ch1, ch2, ch3)
char	ch1, ch2, ch3;
{
	char	c;

	do{
		while((c=getchar()) == '\n') printf("Please choose '%c' or '%c' or '%c' : ",
						    ch1,ch2,ch3);
		while(getchar() != '\n') c='\0';
		if ( c == ch1 ) return 1;
		else if (c == ch2 ) return 2;
		else if (c == ch3 ) return 3;
		else if (c == 'q' || c == 'Q') Quit();
		else printf("Please choose '%c' or '%c' or '%c' : ",
						    ch1,ch2,ch3);
	}while(1);
	return 0;
}


/***************** Static Function: int getname( char * )*********************************
** Gets a string from the standard input. 						**
** The function returns the length of the string.					*/

static int
getname(string)
char	*string;
{
	int	i=0;

	while((string[i]=getchar())!='\n') i++;
	string[i]='\0';
	if (!strcmp(string,"q") || !strcmp(string,"Q")) Quit();
	else return i;
	return 0;
}


/***************** Static Function: void getowner( char *, char *, char* )*****************
** Gets a valid owner id and his login directory from the standard input. 		  */ 


static void
getowner(owner,prod,homedir)
char *owner;
char *prod;
char *homedir;
{
	struct passwd 	*Passwd;
	int		i;
	char		str[1000];

	printf("\n");
	Passwd = getpwnam(owner);

	/*
	 * owner is not a valid owner id
	 */
	if (Passwd == NULL){
		do{
			printf("Enter the owner of the %s : ",
			       prod);
			i=getname(str);
			if (i && (Passwd = getpwnam(str)) == NULL)
				printf("Sorry ! No entry for user %s.\n",str);
		}while(Passwd==NULL);
	}
	/*
	 * owner is a valid owner id
	 */
	else {
		do{
			printf("Enter the owner of the %s ( %s ) : ",
			       prod,
			       owner);
			i=getname(str);
			if (i == 0) strcpy(str,owner);
			if ((Passwd = getpwnam(str)) == NULL)
				printf("Sorry ! No entry for user %s.\n",str);
		}while(Passwd==NULL);
	}
	strcpy(owner,str);
	strcpy(homedir,Passwd->pw_dir);
}

/***************** Static Function: void getgroup( char *, char * )************************
** Gets a valid group id from the standard input. 					*/


static void
getgroup(group,prod)		
char *group;
char *prod;
{	
	struct group	*Group;		
	int		i;
	char		str[1000];

	printf("\n");
	Group=getgrnam(group);

	/*
	 * group is not a valid group id
	 */
	if (Group == NULL){
		do{
			printf("Enter the name of the %s group : ",
			       prod);
			i=getname(str);
			if (i && (Group=getgrnam(str)) == NULL)
				printf("Sorry, no entry for group %s!\n",str);
		}while(Group == NULL);
	}
	/*
	 * group is a valid group id
	 */
	else{
		do{
			printf("Enter the name of the %s group ( %s ) : ",
			       prod,
			       group);
			i=getname(str);
			if (i == 0) strcpy(str,group);
			if ((Group=getgrnam(str)) == NULL){
				printf("Sorry, no entry for group %s!\n",str);
			}
		}while(Group == NULL);
	}
	strcpy(group,str);
}

/***************** Static Function: void extract( char *, char * ,int *, int, char *)**************
 ** Extract the named module? 								 */

static void
	extract(Name,str1,module,Mod,comm)		
char *Name, *str1;
int *module;
int Mod;
char *comm;
{
	char str[150];
	
	sprintf(str,"%s.tar",Name);
	if (!access(str,F_OK)){
		printf("\n%s%s (y/n): ",str1,comm);
		sprintf(str,"Please answer with 'y' or 'n'!\n%s (y/n): ",str1);
		if (getanswer(str)) *module |= Mod;
		
	}
}

/***************** Static Function: void serial( char *, char * ,char *, char *)**********
 ** Gets the serial number of the named module.	 					 */


void
	serial(str1,str2,name)
char	*str1, *str2, *name;
{
	char	str[34],r[5];
	int	i;
	
	printf("\n");
	if (!strcmp(name,"")){
		do{
			printf("Please enter the serial number of the\n%s\n					%s\n					Serial - No.: ",str1,str2);
			getname(str);
			if (2000 <= atoi(str) || atoi(str)<1000)
				printf("Not a valid serial number!\n");
		} while(2000 <= atoi(str) || atoi(str)<1000 );
	}
	else {
		for (i=0;i<4;i++)
			r[i]=name[strlen(name)-(4-i)];
		r[4]='\0';
		do {
			printf("Please enter the serial number of the\n%s\n					%s\n					Serial - No. (%s) : ",str1,str2,r);
			if (getname(str)==0)
				strcpy(str,r);
			if (2000 <= atoi(str) || atoi(str)<1000)
				printf("Not a valid serial number!\n");
		} while(2000 <= atoi(str) || atoi(str)<1000 );
	}
	
	sprintf(name,"%s-%s",str2,str);
}


/***************** Static Function: void getvar( FILE *, char * ,char *)*****************
 ** Gets variable value and his comment from the given file.				*/


static void
	getvar(config, var, var_comm)
FILE 	*config;
char 	*var, *var_comm;
{
	char	str[500];
	char	c;


	do{
		c=(char) fgetc(config);
	}while(c == '\t' || c == '\v' || c == '\r' || c == '\f' || c ==' ');

	if (c!='=') return;

	do{
		c=(char) fgetc(config);
	}while(c == '\t' || c == '\v' || c == '\r' || c == '\f' || c ==' ');

	if (c=='\n' || c==EOF) return;

	ungetc((int) c,config);

	if (c=='#'){
		fgets(str,256,config);
		sprintf(var_comm,"		%s",str);
	}
	else {
		fscanf(config,"%s",str);
		strcpy(var,str);
		do{
			c=(char) fgetc(config);
		}while(c == '\t' || c == '\v' || c == '\r' || c == '\f' || c ==' ');
		if (c!='#') return;
		fgets(str,256,config);
		sprintf(var_comm,"		#%s",str);
	}
}


/***************** main *****************************************************************/

int
	main(argc,argv)
int	argc;
char	*argv[];
{
	
	struct utsname 	name;			/* used from uname() */
	int		warning=0;		/* Warning flag */
	int		module=0;		/* Flag to the extracting modules */
	char		release[3];		/* Release number of the operating system */
	char		Parix_path[60];		/* Path of the PARIX */
	char		Px_Serial[30];		/* Serial number of the PARIX */
	char		Adm_Serial[30];		/* Serial number of the Adm */
	char		ACE_C_Serial[30];	/* Serial number of the ACE-Expert C-compiler */
	char		ACE_F77_Serial[30];	/* Serial number of the ACE-Expert Fortran-compiler */
	char		Patop[30];
	char		Patop_Serial[30];	/* Serial number of the PATOP */
	char		Parmacs[30];
	char		Parmacs_Serial[30];	/* Serial number of the PARMACS */
	char		BasicTSS[50];		/* Path of the Basic  Transputer System  software */
	char		BasicTSS_Comm[257];	/* Comment to the path of the Basic  Transputer System  software */
	char		Board[10];		/* Board type */
	char		Board_Comm[257];	/* Comment to the board type */
	char		Links[60];
	char		Links_Comm[257];
	int		defaultt=0;
	char		link_o[60];		/* Full name of the link.o */
	char		lconf_o[60];		/* Full name of the lconf.o */
	char		trans_conf[60];		/* Full name of the trans_conf */
	char		Px_Owner[20];		/* Owner of the PARIX */
	char		Px_Group[20];		/* Group of the PARIX */
	char		NRM_Owner[20];		/* Owner of the NRM */
	char		Adm_Owner[20];		/* Owner of BASE */
	char		Adm_Group[20];		/* Group of BASE */
	char		Px_Owner_Comm[257];	/* Comment to the owner of the PARIX */
	char		Px_Group_Comm[257];	/* Comment to the group of the PARIX */
	char		NRM_Owner_Comm[257];	/* Comment to the owner of the NRM */
	char		Adm_Owner_Comm[257];	/* Comment to the owner of BASE */
	char		Adm_Group_Comm[257];	/* Comment to the group of BASE */
	int		change_parix=0;		/* PARIX owner and group change flag */
	int		install_base=0;		/* BASE owner and group install flag */
	FILE		*config;		/* Configuration file */
	int		install_nrm=0;		/* NRM installation flag */
	int		installed_nrm=0;	/* NRM installation flag */
	char		NRM_home[100];		/* Home of the owner of the NRM */
	char		NRM_home_Comm[257];	/* Comment to the home of the owner of the NRM */
	long		pos;			/* Position in a file */
	int		permissions=0;	
	int		reconfigure;
#define		parix_permissions	1	
#define		no_parix_permissions	2	
#define		nrm_permissions		4	
#define		no_nrm_permissions	8	
	int 		i;			
	char		str[1000];
	char		str1[256];
	char		c;
       
	/*
	 *  Check the arguments
	 */
	if (argc != 2 || 
	    (strcmp(*++argv,"install") &&
	     strcmp(*argv,"install-nrm") &&
	     strcmp(*argv,"change-parix") &&
	     strcmp(*argv,"install-base") &&
	     strcmp(*argv,"tar"))){
		fprintf(stderr,"Wrong parameters; %s failed\n",Fname);
		exit(1);
	}
	
	if (!strcmp(*argv,"tar")){
		printf("**********************************************************************\n");
		printf("** This  is  the  installation  programm of  the PARIX  Release 1.2 **\n");
		printf("*                                                                    *\n");
		printf("* As the  frontend a SUN 4 (with SPARC or a compatible machine) with *\n");
		printf("* either VMEbus or Sbus is  required. The  operating system  must be *\n");
		printf("* SunOS 4.1.1 or higher and it must  have been  installed  including *\n");
		printf("* the System V software installation  option (you can check this via *\n");
		printf("* \"which cut\",  because  the  cut-command  is  only  available  with *\n");
		printf("* System V software installation option).                            *\n");
		printf("*                                                                    *\n");
		printf("* We assume  that you  have  installed the Basic  Transputer System  *\n");
		printf("* software 1.3 succesfully. (Refer to the resp. installation manuals *\n");
		printf("* BTSS)                                                              *\n");
		printf("*                                                                    *\n");
		printf("* To  install  PARIX User  Base Package you  must be  the Superuser. *\n");
		printf("**********************************************************************\n");
	}/*end if (!strcmp(*argv,"tar"))*/
	
	else {
		sprintf(str,"touch ./ 2>/dev/null /dev/null");
		if (system(str)){
			printf("\n\n	** You don't have write permission for this directory!\n\n");
			if (geteuid()==0)
				printf("	** Maybe you are on other host!\n\n");
			Quit();
		}
	}
	
	if (!strcmp(*argv,"install")){
		printf("**********************************************************************\n");
		printf("** This  is  the  installation  programm of  the PARIX  Release 1.2 **\n");
		printf("*                                                                    *\n");
		printf("* To install PARIX on  additional hosts, all hosts must be connected *\n");
		printf("* together via NFS. The  PARIX directory and  the home  directory of *\n");
		printf("* the PARIX users must be available on all hosts.                    *\n");
		printf("*                                                                    *\n");
		printf("* To the installation you must be the Superuser.                     *\n");
		printf("**********************************************************************\n");
	}/*end if (!strcmp(*argv,"install"))*/

	strcpy(Parix_path,"");
	strcpy(Px_Serial,"");
	strcpy(Adm_Serial,"");
	strcpy(ACE_C_Serial,"");
	strcpy(ACE_F77_Serial,"");
	strcpy(Patop_Serial,"");
	strcpy(Parmacs_Serial,"");

do{
	reconfigure=module=warning=defaultt=change_parix=install_base=install_nrm=installed_nrm=permissions=0;

	/*
	 *  Init data
	 */
	strcpy(BasicTSS,"");
	strcpy(Px_Owner,"");
	strcpy(Px_Group,"");
	strcpy(NRM_Owner,"");
	strcpy(NRM_home,"");
	strcpy(Adm_Owner,"");
	strcpy(Adm_Group,"");
	strcpy(Board,"");
	strcpy(Links,"");
	
	/*
	 *  Read the data from the CONFIG file
	 */
	if ((config=fopen(CONFIG,"r")) !=NULL){
		
		do{
			do{
				c=(char) fgetc(config);
				if (c == '#')	
					fgets(str,256,config);
			}while(c == '\n' || c == '\t' || c == '#'  ||
			       c == '\v' || c == '\r' || c == '\f' || c ==' ');
			
			if (c==EOF) break;

			i=0;

			while(c!='=' && (isalpha(c) || c=='_')){
				str[i++]=c;
				c=(char) fgetc(config);
			}
			ungetc((int) c,config);
			str[i]='\0';

			if (!strcmp("BTSS",str))
				getvar(config,BasicTSS,BasicTSS_Comm);
			else if (!strcmp("PX_OWN",str))
				getvar(config,Px_Owner,Px_Owner_Comm);
			else if (!strcmp("PX_GRP",str))
				getvar(config,Px_Group,Px_Group_Comm);
			else if (!strcmp("NRM_OWN",str))
				getvar(config,NRM_Owner,NRM_Owner_Comm);
			else if (!strcmp("NRM_HOME",str))
				getvar(config,NRM_home,NRM_home_Comm);
			else if (!strcmp("BASE_OWN",str))
				getvar(config,Adm_Owner,Adm_Owner_Comm);
			else if (!strcmp("BASE_GRP",str))
				getvar(config,Adm_Group,Adm_Group_Comm);
			else if (!strcmp("BOARD_TYPE",str))
				getvar(config,Board,Board_Comm);
			else if (!strcmp("LINKS",str))
				getvar(config,Links,Links_Comm);
			else fgets(str,256,config);
			
		}while(1);
		
		fclose(config);
	}/* end if ((config=... */
	
	if (!strcmp(*argv,"install") || !strcmp(*argv,"tar")){
		/*
		 *  Get the path of the Basic Transputer System software binaries
		 */
		
		if(!strcmp(Links,"") || 
		   strstr(Links,"links") == NULL || 
		   strcmp(strstr(Links,"links"),"links"))
			strcpy(Links,"/usr/bin/links");
		strcpy(str,Links);
		if (access(str,F_OK))
			printf("%s\n%s\n%s (%s): ",
			       "\n\nWhere is the 'links' command of the BTSS installed?",
			       "(You can find it with 'which links' or 'find / -name links -print')", 
			       "Please enter the full name",
			       Links);
		while(access(str,F_OK)){
			if(!getname(str))
				strcpy(str,Links);;
			if (strstr(str,"links") == NULL || strcmp(strstr(str,"links"),"links")){
				printf("Wrong name '%s'!\n",str);	
				printf("Please enter the full name of the command 'links'!\n(%s): ",Links);
				strcpy(str,Links);
			}

			else if (*str!='/'){
				printf("Please enter the full name of the command 'links'!\n(%s): ",Links);
				strcpy(str,Links);
			}
				
			else if (access(str,F_OK)){
				printf("Can't find '%s'!\n",str);
				printf("Please enter the full name of the command 'links'!\n(%s): ",Links);
			}
		}
		strcpy(Links,str);
		
	
		printf("\n\n");
		/*
		 *  Check the version of the Basic Transputer System software
		 */
		strcat(str,
		       " 2>/dev/null | grep \"Id   Status       Link-Name         Bus-Address  SBus-slot\" >/dev/null 2>/dev/null");
		if (system(str)){
			printf("\n	*** The Basic Transputer System software version is not 1.3\n");
			warning=1;
		}

		/* 
		 *  Get information about the system !
		 */
		if (uname(&name)==-1){
			fprintf(stderr,"Error: Can't execute %s\n",Fname);
			exit(1);
		}
		
		
		/*
		 *  Operating system name == SunOS || SPARC/OS?
		 */
		if (strcmp("SunOS",name.sysname) && strcmp("SPARC/OS",name.sysname)){
			printf("        *** Your operating system is %s, not SunOS.\n",name.sysname);
			warning=1;
		}
		
		/*
		 *  Operating system release ok. ?
		 */
		if( !strcmp("SunOS",name.sysname) ){
			release[0]=name.release[0];
			release[1]=name.release[2];
			release[2]=name.release[4];
			if (atoi(release) < 411){
				printf("        *** The release number is %s, lower than 4.1.1 .\n",
				       name.release);
				warning=1;
			}
		}
		if( !strcmp("SPARC/OS",name.sysname) ){
			release[0]=name.release[0];
			release[1]=name.release[2];
			release[2]=name.release[4];
			if (atoi(release) < 111){
				printf("        *** For SPARC/OS the release number (%s) must be 1.1.1 or higher.\n",
				       name.release);
				warning=1;
			}
		}

		/*
		 *  SUN 4 machine ?
		 */
		if (strncmp("sun4",name.machine,4) ){
			printf("	*** The machine is %s, not SUN 4.\n",name.machine);
			warning=1;
		}
		
		/*
		 *  SPARC-family processor ?
		 */
		if (system("sparc")){
			printf("	*** The processor is not SPARC compatible.\n");
			warning=1;
		}
		
		
		
		/*
		 *  Any Problem!!
		 */
		if (warning){
			printf("\n%s\n\n%s",
			       "	!!! PARIX might or might not work",
			       "	Would you like to continue the installation procedure? (y/n): ");
			if (!getanswer( "	Please answer with 'y' or 'n'        continue? (y/n): "))
				Quit();
		}
	} /*end if (!strcmp(*argv,"install") || !strcmp(*argv,"tar"))*/
	
	

	if ( !strcmp(*argv,"tar")){
		char		*string;

		/*
		 *  the path name of the PARIX Release 1.2 User Base Package
		 */
		printf("\n");
		if (!strcmp(Parix_path,""))
			string=getenv("PARIX");
		else{
			string=(char *)malloc(100);
			strcpy(string,Parix_path);
		}

		if (string == (char *)0)
			string="/parix";
		
		do{
			do{
				printf("%s\n%s ( %s ) : ",
				       "Where do you want to install (or where is installed) PARIX Release 1.2 ?",
				       "Please enter the full path",
				       string);
				i=getname(str);
				if (i==0) strcpy(str,string);
				i=strlen(str);
				if (*str!='/')
					printf("Please enter the full pathname!\n");
				else if (access(str,F_OK) ) {
					printf("No such directory: '%s'. Create? (y/n): ",str);
					sprintf(str1,"Create this directory? (%s) (y/n): ",str);
					if (getanswer(str1))
						if (mkdir(str,0755))
							printf("I can't create '%s'.\n",str);
				}
			} while(access(str,F_OK) || *str!='/');
			sprintf(str1,"touch %s 2>/dev/null /dev/null",str);
			if (i=system(str1)){
				printf("\n\n	** You don't have write permission for this directory (%s)!\n",str);
			}
		}while(i);
		strcpy(Parix_path,str);
		
		/*
		 *  Which files are to extract?
		 */
		/*
		 *  Basic PARIX
		 */
		extract(BASIC,
			"Would you like to install the basic PARIX module?",
			&module,
			BASIC_MODULE,
			"\n(Minimal software to run PARIX application)	");
		
		/*
		 *  SYSTEM
		 */
		extract(SYSTEM,
			"Would you like to install the PARIX  System module?",
			&module,
			SYSTEM_MODULE,
			"\n(Management tools)				");
		
		/*
		 *  C
		 */
		extract(C,
			"Would you like to install the ACE-Expert C-compiler module?",
			&module,
			C_MODULE,
			"");
		
		/*
		 *  FORTRAN
		 */
		extract(FORTRAN,
			"Would you like to install the ACE-Expert Fortran-compiler module?",
			&module,
			FORTRAN_MODULE,
			"");
		
		/*
		 *  PATOP
		 */
		sprintf(str,"%s.tar",PATOP_FULL);
		if (!access(str,F_OK))
			strcpy(Patop,PATOP_FULL);
		else
			strcpy(Patop,PATOP_RESTR);

		extract(Patop,
			"Would you like to install the PATOP module?",
			&module,
			PATOP_MODULE,
			"");
		
		/*
		 *  PARMACS
		 */
		sprintf(str,"%s.tar",PARMACS_FULL);
		if (!access(str,F_OK))
			strcpy(Parmacs,PARMACS_FULL);
/*		else
			strcpy(Parmacs,PARMACS_RESTR); */

		extract(Parmacs,
			"Would you like to install the PARMACS module?",
			&module,
			PARMACS_MODULE,
			"");
		/* 
		 * No module to extract 
		 */	
		if (!module) Quit();
		
		/*
		 *  Serial numbers
		 */
		printf("\n!! You can find the serial numbers on the DELIVERY-NOTE !!\n");
       		if (module & BASIC_MODULE)
			/*
			 *  Serial number for basic PARIX
			 */
			serial("PARIX Base Package",BASIC,Px_Serial);
		if (module & SYSTEM_MODULE)
			/*
			 *  Serial number for PARIX system
			 */
			serial("PARIX System",SYSTEM,Adm_Serial);
		if (module & C_MODULE)
			/*
			 *  Serial number for ACE-Expert C-compiler
			 */
			serial("ACE-Expert C-compiler",C,ACE_C_Serial);
		if (module & FORTRAN_MODULE)
			/*
			 *  Serial number for ACE-Expert Fortran-compiler
			 */
			serial("ACE-Expert Fortran-compiler",FORTRAN,
			       ACE_F77_Serial);
		if (module & PATOP_MODULE)
			/*
			 *  Serial number for PATOP
			 */
			serial("PATOP module",Patop,Patop_Serial);
		if (module & PARMACS_MODULE)
			/*
			 *  Serial number for PARMACS
			 */
			serial("PARMACS module",Parmacs,Parmacs_Serial);
		
        } /* end if ( !strcmp(*argv,"tar")) */

	if (!strcmp(*argv,"tar") && !(module & BASIC_MODULE)){
		sprintf(str,"%s/%s",Parix_path,CONFIG);
	if (!access(str,R_OK)){
			sprintf(str,"cp %s/%s .",Parix_path,CONFIG);
			system(str);
		}
	}	
		

	if (!strcmp(*argv,"install") || (!strcmp(*argv,"tar") && (module & BASIC_MODULE))){
		
		char		*string;
		/*
		 *  The path of the Basic Transputer System software object files
		 */
		printf("\n");
		string=getenv("TRANSP_CONFIG");
		if ( string == (char *)0)
			sprintf(trans_conf,"%s%s",BasicTSS,TRANS_CONF);
		else
			sprintf(trans_conf,"%s",string);
		sprintf(link_o,"%s%s",BasicTSS,LINK_O);
		sprintf(lconf_o,"%s%s",BasicTSS,LCONF_O);
		if (!access(BasicTSS,F_OK) && !access(trans_conf,F_OK) &&
		    !access(link_o,F_OK) && !access(lconf_o,F_OK)) defaultt=1;
		
		do{
			int	not_ok1,not_ok2;
			
			if (defaultt) 
				printf("%s\n%s( %s ) : ",
				       "Please enter the path where the Basic Transputer System software",
				       "is installed ",
				       BasicTSS);
			else
				printf("%s\n%s : ",
				       "Please enter the path where the Basic Transputer",
				       "System software is installed");
			i=getname(str);
			if (i ==0 && defaultt) strcpy(str,BasicTSS);
			/*
			 *  Exist TRANS_CONF?
			 */
			if ( string == (char *)0 || (string != (char *)0 && access(trans_conf,F_OK)))
				sprintf(trans_conf,"%s%s",str,TRANS_CONF);
			
			not_ok1=access(trans_conf,F_OK);
			if (not_ok1 && errno == ENOENT) printf("I can't find %s\n",trans_conf);
			
			/*
			 *  Exist LINK_O?
			 */
			sprintf(link_o,"%s%s",str,LINK_O);
			not_ok2=access(link_o,F_OK);
			if (not_ok2 && errno == ENOENT) printf("I can't find %s\n",link_o);
			
			not_ok1|=not_ok2;
			
			/*
			 *  Exist lconf.o?
			 */
			sprintf(lconf_o,"%s%s",str,LCONF_O);
			not_ok2=access(lconf_o,F_OK);
			if (not_ok2) {
				if (errno == ENOENT) printf("I can't find %s\n",lconf_o);
				else if (errno == ENOTDIR) printf("%s is not a directory\n",str);
				else if (errno == EACCES) printf("%s: permission is denied\n",str);
				else if (errno == ENAMETOOLONG) printf("The length of %s is too long\n",str);
			}		
			else if(!not_ok1) break;
		}while(1);
		strcpy(BasicTSS,str);

		/*
		 * Read the board type from BasicTSS/trans_conf
		 */
		config=fopen(trans_conf,"r");
		do{
			c=fgetc(config);
			if (c == '#'  || c == '\t' ||
			    c == '\v' || c == '\r' ||
			    c == '\f' || c == ' ') fgets(str,256,config);
		}while(c == '\n' || c == '\t' || c == '#'  ||
		       c == '\v' || c == '\r' || c == '\f' || c ==' ');
		ungetc((int) c,config);
		fscanf(config,"%s",str);
		fscanf(config,"%s",Board);
		fclose(config);
		/*
		 *  Get the board type (S-Bus or VME-Bus board?)
		 */
		defaultt=1;
		if ( !strcmp(Board,"BBKS4") )  
			strcpy(Board,"S-Bus");
		else if ( !strncmp(Board,"MTM-SUN",7) ||  !strcmp(Board,"VMTM") ||
			 !strcmp(Board,"vmtm") || !strncmp(Board,"bbkv",4))
			strcpy(Board,"VME-Bus");
		else defaultt=0;
		printf("\n");
		do{
			if (defaultt)
				printf("%s ( %s ) : ",
				       "Do you have a S-Bus or a VME-Bus link interface?",
				       Board);
			else
				printf("%s : ",
				       "Do you have a S-Bus or a VME-Bus link interface?");
			i=getname(str);
			if (defaultt && i==0) strcpy(str,Board);
		}while(strcmp(str,"VME-Bus") && strcmp(str,"S-Bus")); 
		strcpy(Board,str);
		
	}/*if (!strcmp(*argv,"install") || (!strcmp(*argv,"tar") && (module & BASIC_MODULE))) */
	
	/*
	 *  Change the owner and/or the group of the PARIX ?
	 */
	if ( !strcmp(*argv,"install") && (!strcmp(Px_Owner,"") || !strcmp(Px_Group,""))
	    || !strcmp(*argv,"change-parix") 
	    || !strcmp(*argv,"tar")) 
		{
			getowner(Px_Owner,"PARIX",str);
			getgroup(Px_Group,"PARIX");
			change_parix=1;
	}

	if ( !strcmp(*argv,"change-parix") && !strcmp(NRM_Owner,"")){
		printf("\nHave you installed the NRM? (y/n) :");
		if (getanswer("Please answer with 'y' or 'n'! Have you installed the NRM? (y/n) :")){	
			getowner(NRM_Owner,"NRM",NRM_home);
			sprintf(str,"touch %s 2>/dev/null /dev/null",NRM_home);
			if (system(str)){
				printf("\n ** You don't have permission to change the group id of NRM\n\n");
				Quit();
			}
		}	
	}	
	
	
		
		

	/*
	 * give group class permissions to the other class?
	 */
	if (!strcmp(*argv,"tar")){
		printf("\n%s",
		       "Would you like to give access rights to PARIX to others? (y/n): ");
		if (getanswer( "Please answer with 'y' or 'n'!\nWould you like to give access rights to PARIX to others? (y/n): "))
			permissions |=parix_permissions;
		else permissions |=no_parix_permissions;
	}

	/*
	 *  Network Resource Manager
	 */
	
	/*
	 *  Install NRM ?
	 */
	if (!strcmp(*argv,"tar") && (module & SYSTEM_MODULE)){
		if (module & BASIC_MODULE)
		printf("\n*       You can use PARIX without installing NRM or BASE             *\n\n");
		printf("* We strongly  recommend to use the  Network Resource  Manager (NRM) *\n");
		printf("* for all computers except for configurable MultiCluster 2 or Super- *\n");
		printf("* clusters.  The  NRM  is  used to  administrate  massively parallel *\n");
		printf("* computers  and  fixed  topology   computers.  Refer  to  the  NRM  *\n");
		printf("* documentation  for  more details.                                  *\n");
		
		printf("\n%s",
		       "Would you like to install NRM ? (y/n): ");
		install_nrm=getanswer( "Please answer with 'y' or 'n'!! Would you like to install NRM ? (y/n): ");
	}
	else if (!strcmp(*argv,"install-nrm")){
		install_nrm=1;
		if (!strcmp(Px_Owner,"")) getowner(Px_Owner,"PARIX",str);
		if (!strcmp(Px_Group,"")) getgroup(Px_Group,"PARIX");
	}

	if (install_nrm){
		printf("\nThe login directory for the owner of the NRM must exist!");
		getowner(NRM_Owner,"NRM",NRM_home);
		sprintf(str,"touch %s 2>/dev/null /dev/null",NRM_home);
		if (system(str)){
			printf("\nYou don't have write permission for the login directory\n(%s) of the  owner of the NRM\n",NRM_home);
			if (strcmp(*argv,"install-nrm")){
				printf("Would you continoue the installation without installing NRM (y/n): ");
				if (!getanswer("Please answer with 'y' or 'n'! : "))
					Quit();
				else{
					strcpy(NRM_home,"");
					install_nrm=0;
				}
			}
			else
				Quit();
		}
	}
	if (install_nrm){
		printf("\n%s",
		       "Would you like to give access rights to NRM to others? (y/n): ");
		if (getanswer( "Please answer with 'y' or 'n'!\nWould you like to give access rights to NRM to others? (y/n): "))
			permissions |=nrm_permissions;
		else permissions |=no_nrm_permissions;
		printf("\n");
		printf("* The NRM will be installed. To use the NRM the user %s must\n",NRM_Owner);
		printf("* create a configuration for the system. See the NRM documentation.\n"); 
	}
	
	
	/*
	 *  Basic Administration Software Environment
	 */
	/*
	 *  install BASE ?
	 */
	if (!strcmp(*argv,"tar") && (module & SYSTEM_MODULE)){
		printf("\n");
		printf("* The Basic  Administration  Software Environment (BASE) is a script *\n");
		printf("* based  tool for  administration  purposes  to handle a  multi-user *\n");
		printf("* environment properly. Refer to the  Administrator's Guide for more *\n");
		printf("* details. For  the full installation of BASE see the 'Step  by Step *\n");
		printf("* Installation  Guide'  in  the Administrator's  Guide.              *\n");
		printf("\n%s",
		       "Would you like to install BASE ? (y/n): ");
		install_base=getanswer( "Please answer with 'y' or 'n'!! Would you like to install BASE? (y/n): ");
	}
	else if (!strcmp(*argv,"install-base")) install_base=1;
	if (install_base){
		getowner(Adm_Owner,"BASE",str);
		getgroup(Adm_Group,"BASE");
	}

	/*
	 *  Write the data onto the CONFIG file
	 */
	if ((config=fopen(CONFIG,"r"))!=NULL){
		do{
			c=(char) fgetc(config);
			if (c == '#') fgets(str,256,config);
		}while(c == '\n' || c == '\t' || c == '#'  ||
		       c == '\v' || c == '\r' || c == '\f' || c ==' ');

		ungetc((int) c,config);
		pos=ftell(config);
		fclose(config);
		truncate(CONFIG,pos);
		if ((config=fopen(CONFIG,"a+"))==NULL){
			printf("\n\n	** Insufficient privileges to open %s to write\n\n",CONFIG);
			Quit();
		}
		fseek(config,pos,SEEK_SET);
	}/* end if (!access(CONFIG,R_OK)) */
	
	/*
	 * if the CONFIG file don't exist then create comments
	 */
	else {
		strcpy(Links_Comm,"	# The full name of the links command\n");
		strcpy(BasicTSS_Comm,"	# The directory where the BTSS installed is\n");
		strcpy(Px_Owner_Comm,"	# The owner of the PARIX User Base Package\n");
		strcpy(Px_Group_Comm,"	# The group of the PARIX User Base Package\n");
		strcpy(NRM_Owner_Comm,"	# The owner of the NRM\n");
		strcpy(NRM_home_Comm,"	# The login directory of the owner of the NRM\n");
		strcpy(Adm_Owner_Comm,"	# The owner of BASE\n");
		strcpy(Adm_Group_Comm,"	# The group of BASE\n");
		strcpy(Board_Comm,"	# The type of the link interface ( S-Bus or VME-Bus)\n");
		if ((config=fopen(CONFIG,"w"))==NULL){
			printf("\n\n	** Insufficient privileges to open %s to write\n\n",CONFIG);
			Quit();
		}

		fprintf(config,"%s\n%s\n#\n# SCCSid:.	%s\n\n",
			"# Configuration file for PARIX Release installation",
			"#              (C) Parsytec GmbH 1993",
			SCCSid);
	}

	if (module){
		fprintf(config,"PARIX = %s\n\n",Parix_path);
		fprintf(config,"TAR = ");
		if (module & BASIC_MODULE) fprintf(config," %s",BASIC);
		if (module & SYSTEM_MODULE) fprintf(config," %s",SYSTEM);
		if (module & C_MODULE) fprintf(config," %s",C);
		if (module & FORTRAN_MODULE) fprintf(config," %s",FORTRAN);
		if (module & PATOP_MODULE) fprintf(config," %s",Patop);
		if (module & PARMACS_MODULE) fprintf(config," %s",Parmacs);
		fprintf(config,"\n\nLICENSES = ");
		if (module & BASIC_MODULE) fprintf(config," %s",Px_Serial);
		if (module & SYSTEM_MODULE) fprintf(config," %s",Adm_Serial);
		if (module & C_MODULE) fprintf(config," %s",ACE_C_Serial);
		if (module & FORTRAN_MODULE) fprintf(config," %s",ACE_F77_Serial);
		if (module & PATOP_MODULE) fprintf(config," %s",Patop_Serial);
		if (module & PARMACS_MODULE) fprintf(config," %s",Parmacs_Serial);
		fprintf(config,"\n\n");
	} /* end if (module) */
	
	fprintf(config,"LINKS = %s%s\n",
		Links,
		Links_Comm);
	fprintf(config,"BTSS = %s%s\n",
		BasicTSS,
		BasicTSS_Comm);
	fprintf(config,"BOARD_TYPE = %s",Board);
	if (!strcmp(Board,"S-Bus")){
		fprintf(config,"		# Packet Emulation\n\n");
		fprintf(config,"LD_DSERVER = %s\n\n",
			"ld_dserverp.o");
		fprintf(config,"LD_DSERVER_DB = %s\n\n",
			"ld_dserverp.db.o");
		fprintf(config,"LD_DSERVER_LR = %s\n\n",
			"ld_dserverp.lr.o");
	}
	else{
		fprintf(config,"		# No Packet Emulation\n\n");
		fprintf(config,"LD_DSERVER = %s\n\n",
			"ld_dserver.o");
		fprintf(config,"LD_DSERVER_DB = %s\n\n",
			"ld_dserver.db.o");
		fprintf(config,"LD_DSERVER_LR = %s\n\n",
			"ld_dserver.lr.o");
	}
		
	fprintf(config,"PX_OWN = %s%s\n",
		Px_Owner,	
		Px_Owner_Comm);
	fprintf(config,"PX_GRP = %s%s\n",
		Px_Group,
		Px_Group_Comm);
	fprintf(config,"NRM_OWN = %s%s\n",
		NRM_Owner,	
		NRM_Owner_Comm);
	fprintf(config,"NRM_HOME = %s%s\n",
		NRM_home,
		NRM_home_Comm);
	fprintf(config,"BASE_OWN = %s%s\n",
		Adm_Owner,	
		Adm_Owner_Comm);
	fprintf(config,"BASE_GRP = %s%s\n",
		Adm_Group,	
		Adm_Group_Comm);

	fprintf(config,"MAKE_TARGETS =");
	if (!strcmp(*argv,"install") || (!strcmp(*argv,"tar") && (module & BASIC_MODULE)))
		fprintf(config," install");
	if (install_nrm) fprintf(config," install-nrm");
	if (change_parix) fprintf(config," change-parix");
	if (install_base) fprintf(config," install-base");
	if (permissions & no_parix_permissions) fprintf(config," parix-permissions");
	if (permissions & no_nrm_permissions) fprintf(config," nrm-permissions");
	fprintf(config,"\n\n# EOF\n");
		
	fclose(config);

	printf("\n* The PARIX configuration file %s has been modified",CONFIG);

       	/*
	 * Are all data correct?
	 */

	printf("\n");

	if (module){
		printf("The following modules will be installed:\n     ");
		if (module & BASIC_MODULE) printf("Basic PARIX,");
		if (module & SYSTEM_MODULE) printf(" PARIX System,");
		if (module & C_MODULE) printf(" C-compiler,");
		if (module & FORTRAN_MODULE) printf(" Fortran-compiler,");
		if (module & PATOP_MODULE) printf(" PATOP,");
		if (module & PARMACS_MODULE) printf(" PARMACS");
		printf("\nThe destination directory		  %s\n",Parix_path);		
		if (module & BASIC_MODULE)
			printf("Basic PARIX license:			  %s\n",
			       Px_Serial);
		if (module & SYSTEM_MODULE)
			printf("PARIX System license:			  %s\n",
			       Adm_Serial);
		if (module & C_MODULE)	
			printf("ACE-Expert C-compiler license:		  %s\n",
			       ACE_C_Serial);
		if (module & FORTRAN_MODULE) 
			printf("ACE-Expert Fortran-compiler license:	  %s\n",
			       ACE_F77_Serial);
		if (module & PATOP_MODULE)
			printf("PATOP license:				  %s\n",
			       Patop_Serial);
		if (module & PARMACS_MODULE)
			printf("PARMACS license:			  %s\n",
			       Parmacs_Serial);
	} /* end if (module) */

	if (!strcmp(*argv,"install") || (!strcmp(*argv,"tar") && (module & BASIC_MODULE))){
		printf("Directory of the BTSS :			  %s\n",
		       BasicTSS);
		printf("Link interface :       		   	  %s\n",
		       Board);

	}/* end if (!strcmp(*argv,"install") || (!strcmp(*argv,"tar") && (module & BASIC_MODULE))*/
	if (change_parix)
		printf("PARIX User Base Package owner:	   	  %s\n",
		       Px_Owner);
	if (change_parix || install_nrm)
		printf("PARIX User Base Package group:	  	  %s\n",
		       Px_Group);
	if (permissions & parix_permissions) 
		printf("Usage of PARIX for others:		  not restricted\n");
	else if(permissions & no_parix_permissions) 
		printf("Usage of PARIX for others:		  restricted\n");
	if (install_nrm || installed_nrm){
		printf("Network Resource Manager owner:    	  %s\n",
		       NRM_Owner);
		if (permissions & nrm_permissions) 
		printf("Usage of NRM for others:		  not restricted\n");
		else if(permissions & no_nrm_permissions) 
		printf("Usage of NRM for others:		  restricted\n");
	}

	if (install_base){
		printf("BASE owner:  				  %s\n",
		       Adm_Owner);
		printf("BASE group:				  %s\n",
		       Adm_Group);
	}

	/*
	 * 	With Superuser privileges
	 */
	if (geteuid()!=0) 
		Quit();
	else {
		if (!strcmp(*argv,"change-parix")){
			printf("\nChange the ownership (o)   Reconfigure (r)   Cancel (c)     : ");
			switch (getanswer3('o','r','c')) {
			case 1:
				return 0;
				break;
			case 2:
				reconfigure=1;
				break;
			default:
				Quit();
				break;
			}
		}
	        else {
			printf("\nInstall (i)    Reconfigure (r)    Cancel (c)     : ");
			switch (getanswer3('i','r','c')) {
			case 1:
				return 0;
				break;
			case 2:
				reconfigure=1;
				break;
			default:
				Quit();
				break;
			}
		}
	}
}while(reconfigure);

	return 0;
}
